/******************************************************************************
*******************************************************************************
*                European Southern Observatory
*              VLTI MIDI Data Reduction Software
*
* Module name:    complex.c
* Description:    THIS MODULE CONTAINS ALL COMPLEX ARITHMETIC FUNCTIONS (except assembler ones)
*
* History:        
* 03-Feb-03        (jmeisner) Created
* 25-Apr-03        (csabet) Added title, directives and included in the MIDI pipeline
*******************************************************************************
******************************************************************************/

/******************************************************************************
*    Compiler directives
******************************************************************************/

/******************************************************************************
*    Include files
******************************************************************************/
#include <stdio.h>
#include <cpl.h>
#include <stdlib.h>
#include <math.h>
#include "complex_midi.h"


/*============================ C O D E    A R E A ===========================*/
struct Complex Czero={0.F, 0.F};    //    Available for all apps. DO NOT CHANGE IT!!!!

struct Complex Addc(struct Complex z1, struct Complex z2)
{
   struct Complex result;
   result.r = z1.r + z2.r;
   result.i = z1.i + z2.i;
   return result;
}

struct Complex Subtractc(struct Complex z1, struct Complex z2)
{
   struct Complex result;
   result.r = z1.r - z2.r;
   result.i = z1.i - z2.i;
   return result;
}

struct Complex Multiplyc(struct Complex z1, struct Complex z2)
{
   struct Complex result;
   result.r = z1.r*z2.r - z1.i*z2.i;
   result.i = z1.i*z2.r + z1.r*z2.i;
   return result;
}

struct Complex Dividec(struct Complex z1, struct Complex z2)
{
   struct Complex temp;

   MultiplyConjPtr(&temp, &z1, &z2);
   return Scalec(temp, 1.F/Cmag2(z2));
}


// Definitions which act on pointers:

void SubtractCptr(struct Complex * result, struct Complex *  z1, struct Complex *  z2)
{
    result->r = z1->r - z2->r;
    result->i = z1->i - z2->i;
}

void AddCptr (struct Complex * result, struct Complex *  z1, struct Complex *  z2)
{
    result->r = z1->r + z2->r;
    result->i = z1->i + z2->i;
}

void MultiplyCptr(struct Complex * result, struct Complex *  z1, struct Complex *  z2)
{
    float temp;
    temp=z1->r;
    result->r = z1->r*z2->r - z1->i*z2->i;
//    result.i = z1.i*z2.r + z1.r*z2.i;    //    Old way before pointers passed
    result->i = z1->i*z2->r + temp*z2->i ;
}

void MultiplyConjPtr(struct Complex * result, struct Complex *  z1, struct Complex *  z2)
{
    float temp;
    temp=z1->r;
    result->r = z1->r*z2->r + z1->i*z2->i;
//    result.i = z1.i*z2.r - z1.r*z2.i;
    result->i = z1->i*z2->r - temp*z2->i;
}

struct Complex Scalec(struct Complex z, float scale)
{
   struct Complex result;
   result.r = scale * z.r;
   result.i = scale * z.i;
   return result;
}

struct Complex Unitc(struct Complex z)
{
   double temp;
RESTART:
   temp = z.r*z.r + z.i*z.i;
   if(temp < .0000000000000000000000000001)
   {
      z.r += .00000000000001;
      goto RESTART;
   }
   return Scalec(z, 1.F/sqrt(temp));
}

float Cmag(struct Complex z)
{
   return (float) sqrt(z.r*z.r + z.i*z.i);
}

float Cmag2(struct Complex z)
{
    return (z.r*z.r + z.i*z.i);
}


//    New Nov 9, 1999:
//     Returns a positive real part, if 0 then returns neg imag part.
struct Complex Csqrt(struct Complex value)
{
    float mag;
    struct Complex result;

    // Special case:
    if(value.i==0.F)
    {
        if(value.r>0)
        {
            result.i = 0.F;
            result.r = (float) sqrt(value.r);
            return result;
        }
        //    Else:
        result.r = 0.F;
        result.i = -((float) sqrt(-value.r));
        return result;
    }

    mag = Cmag(value);
    result.r = (float) sqrt(.5*(mag+value.r));
    result.i = (float) sqrt(.5*(mag-value.r));

    if(value.i < 0.F)
        result.i = -result.i;

    return result;
}

//    Now defined in c:
void GetScaledEtoJtimes(float Angle, struct Complex *Result, float scale)
{
    Result->r = scale * (float) cos(Angle);
    Result->i = scale * (float) sin(Angle);
}

void GetEtoJtimes(float Angle, struct Complex *Result)
{
    Result->r = (float) cos(Angle);
    Result->i = (float) sin(Angle);
}

void GetExpc(struct Complex *Result, struct Complex z)
{
    
    GetScaledEtoJtimes(z.i, Result, (float) exp(z.r));
}

struct Complex jtimes(struct Complex z)
{
   struct Complex result;
   result.i = z.r;
   result.r = -z.i;
   return result;
}



//    Richard added on these functions
float Arg(struct Complex z)
{
    if(fabs(z.r)<.000000000000000001)
        if(fabs(z.i)<.000000000000000001)
           return 0.F;
//         NAH:   if (temp<0) temp = temp + 360.F;
    return (float) atan2(z.i,z.r);
}

void printC(struct Complex z)
{
    if(fabs(z.i) < .00001F*fabs(z.r))
        z.i = 0.F;
/***************************************
    {
        if(z.r < 0.F)
           cpl_msg_info(cpl_func,"%.4g + j 0 = %.4g @ 180\n", z.r, -z.r);
        else
           cpl_msg_info(cpl_func,"%.4g + j 0 = %.4g @ 0\n", z.r, z.r);
    }
    else
************************************************/    
   cpl_msg_info(cpl_func,"%.4g + j %.4g = %.4g @ %4.1f\n",
                z.r,z.i,Cmag(z), 57.2958F*Arg(z));
//  cpl_msg_info(cpl_func,"real -  %.2f   imag -  %.2f  mag -  %.2f  phi -  %.2f\n",
}

/*****************************************************************************/
