/******************************************************************************
*******************************************************************************
*               European Southern Observatory
*          VLTI MIDI Maintenance Templates Software
*
* Module name:  fitsAnalysisFrgSP.c
* Description:  Contains all the routines for FITS format analysis related to
*                Kappa computation
*
* History:      
* 05-Dec-05     (csabet) created
*******************************************************************************
******************************************************************************/

/******************************************************************************
*   Compiler directives
******************************************************************************/

/******************************************************************************
*   Include files
******************************************************************************/
#include <stdio.h>
#include <cpl.h>
#include <math.h>
#include "midiGlobal.h"
#include "midiLib.h"
#include "fft.h"
#include "errorHandling.h"
#include "memoryHandling.h"
#include "midiFitsUtility.h"
#include "fitsAnalysisFrgSP.h"

/**********************************************************
*   Global Variables 
**********************************************************/



/*============================ C O D E    A R E A ===========================*/




/******************************************************************************
*               European Southern Observatory
*             VLTI MIDI Data Reduction Software
*
* Module name:  analyseFitsFrgSP
* Input/Output: See function arguments to avoid duplication
* Description:  Analyses all the FITS files in the given list. This routine
*                determines if the FITS list contains useful data files. If
*                not it will discard the list.
*
* History:      
* 05-Dec-05     (csabet) Created
******************************************************************************/
void analyseFitsFrgSP (
    MidiFiles    *fileNames,            // IO: Pointer to midi files structure
    ImageFormat    *formatInterf,        // Ou: Size for Interferometry file
    ImageFormat    *formatPhotomA,        // Ou: Size for Photometry A file
    ImageFormat    *formatPhotomB,        // Ou: Size for Photometry B file
    int            *error)                // Ou: Error status
{

    //    Local Declarations
    //    ------------------
    const char            routine[] = "analyseFitsFrgSP";
    char                *fileTemp, *classification;
    FILE                *inFitsBatchPtr=NULL;
    int                    extNumOfImagingDataFile, aprioriDataIsDisp=0;
    ImageFormat            *localFormat=NULL;
    char                *cleanString=NULL, *aprioriCatg=NULL;
    unsigned int        numOfInterf;
        
    //    Algorithm
    //    ---------
    if (diagnostic > 4)cpl_msg_info(cpl_func,"Invoking      routine   '%s' \n", routine);
    if (diagnostic > 4) fprintf(midiReportPtr, "Invoking      routine   '%s' \n", routine);

   cpl_msg_info(cpl_func,"\nAnalysing batch  %d \n", batchNumber);
   cpl_msg_info(cpl_func,"--------------- \n");
    fprintf (midiReportPtr, "\nAnalysing batch  %d \n", batchNumber);
    fprintf (midiReportPtr, "--------------- \n");

    //    Reset status
    *error = 0;
    
    //    Allocate memory for a temporary file name buffer
    classification = (char *) calloc (MAX_STRING_LENGTH, sizeof (char));
    cleanString = (char *) calloc (MAX_STRING_LENGTH, sizeof (char));
    fileTemp = (char *) calloc (MAX_STRING_LENGTH, sizeof (char));
    aprioriCatg = (char *) calloc (MAX_STRING_LENGTH, sizeof (char));
    localFormat = callocImageFormat ();
    
    //    Open the list of files
    if ((inFitsBatchPtr = fopen (fileNames->inFitsBatch, "r")) == NULL)
    {
        midiReportWarning (midiReportPtr, routine, __FILE__, __LINE__, "Cannot open input FITS file list");
        midiReportWarning (midiReportPtr, routine, __FILE__, __LINE__, "No analysis has been carried out for this batch");
        free (fileTemp);
        free (cleanString);
        freeImageFormat (localFormat);
        free (classification);
        *error = 1;
        return;
    }

    //    Reset image format
    numOfInterf = 0;
    midiInitFormat (formatInterf);

    //    Loop through the files and analyse
    while (fgets (fileTemp, MAX_STRING_LENGTH, inFitsBatchPtr) != NULL)
    {
        sprintf (classification, "%s", "");
        sscanf (fileTemp, "%s%s", fileNames->inFitsName, classification);
        if (diagnostic)cpl_msg_info(cpl_func,"\nAnalysing file   %s \n", fileNames->inFitsName);
        fprintf(midiReportPtr, "\nAnalysing file   %s \n", fileNames->inFitsName);

        //    Get 'extNumOfImagingDataFile' extension number of IMAGING_DATA in input file
        extNumOfImagingDataFile = findImagingDataExtension (fileNames->inFitsName, TAB_IMAGING_DATA, error);
        if (*error) break;

        //    Get Image Format parameters
        if (extNumOfImagingDataFile > 0)
        {
            getImageFormat (fileNames->inFitsName, extNumOfImagingDataFile, localFormat, error);
            if (*error) break;
        }
        else localFormat->hasData = 0;

        //    Check if the file has data
        if (localFormat->hasData)
        {
            //    Save initial Beam Combiner and Category
            if (numOfInterf == 0)
            {
                aprioriDataIsDisp = localFormat->isDisp;
                sprintf (aprioriCatg, "%s", localFormat->obsCatg);
            }
            
            //    Check Beam Combiner consistency
            if (localFormat->isDisp != aprioriDataIsDisp)
            {
                midiReportWarning (midiReportPtr, routine, __FILE__, __LINE__, "Inconsistent Beam Combiner");
                *error = 1;
                break;
            }
            else
                aprioriDataIsDisp = localFormat->isDisp;

            //    Check Category consistency
            if (strcmp (localFormat->obsCatg, aprioriCatg) != 0)
            {
                sprintf (midiMessage, "Inconsistent Observation Category. Expected: %s", aprioriCatg);
                midiReportWarning (midiReportPtr, routine, __FILE__, __LINE__, midiMessage);
                *error = 1;
                break;
            }
            else
                sprintf (aprioriCatg, "%s", localFormat->obsCatg);

            //    Check Tech and Shutter ID
            if (strcmp (localFormat->obsTech, "INTERFEROMETRY") == 0)
            {
                //    Save local parameters
                if (numOfInterf == 0)
                {
                    formatInterf->hasData = 1;
                    formatInterf->chopped = 1;
                    midiCopyFormat (localFormat, formatInterf);

                    formatPhotomA->hasData = 1;
                    formatPhotomA->chopped = 1;
                    midiCopyFormat (localFormat, formatPhotomA);
                    formatPhotomA->numOfRegionsToProcess = 1;

                    formatPhotomB->hasData = 1;
                    formatPhotomB->chopped = 1;
                    midiCopyFormat (localFormat, formatPhotomB);
                    formatPhotomB->numOfRegionsToProcess = 1;
                }

                //    Get cummulative results
                formatInterf->numOfFrames += localFormat->numOfFrames;
                formatPhotomA->numOfFrames += localFormat->numOfFrames;
                formatPhotomB->numOfFrames += localFormat->numOfFrames;

                //    Increment file counter
                ++numOfInterf;
            }
            else if (strcmp (localFormat->obsTech, "IMAGE,WINDOW,CHOPNOD") == 0)
            {
                if (diagnostic > 1)    midiReportInfo (midiReportPtr, 
                    routine, __FILE__, __LINE__, "The above Photometry file will not be processed");
            }
            else
            {
                sprintf (midiMessage, "Unknown Category, Tech or Type in ... %s. Not processed", 
                    fileNames->inFitsName);
                midiReportWarning (midiReportPtr, routine, __FILE__, __LINE__, midiMessage);
            }
        }
        else
        {
            if (diagnostic)
            {
                sprintf (midiMessage, "No data tables in ... %s. Not processed", fileNames->inFitsName);
                midiReportInfo (midiReportPtr, routine, __FILE__, __LINE__, midiMessage);
            }
        }
    }

    //    Close the file list
    if (fclose (inFitsBatchPtr))
        midiReportError (midiReportPtr, routine, __FILE__, __LINE__, "Cannot close batch file");
    
    //    Check if the FITS files contained useful data
    if (!(*error) && (numOfInterf > 0))
    {
        //    Check potential singularities
        if ((formatInterf->framesPerScan == 0) || (formatPhotomA->framesPerScan == 0) || 
            (formatPhotomB->framesPerScan == 0))
        {
            if (fileTemp) free (fileTemp);
            if (localFormat) freeImageFormat (localFormat);
            if (cleanString) free (cleanString);
            if (classification) free (classification);
            if (aprioriCatg) free (aprioriCatg);
            midiReportWarning (midiReportPtr, routine, __FILE__, __LINE__, "Potential singularity detected");
            *error = 1;
            return;
        }

        //    Compute number of scans
        formatInterf->numOfScans = formatInterf->numOfFrames / formatInterf->framesPerScan;
        formatPhotomA->numOfScans = formatPhotomA->numOfFrames / formatPhotomA->framesPerScan;
        formatPhotomB->numOfScans = formatPhotomB->numOfFrames / formatPhotomB->framesPerScan;

        //    Make sure numberOfFrames is a muliple number of the framesPerScan
        formatInterf->numOfFrames = formatInterf->numOfScans * formatInterf->framesPerScan;
        formatPhotomA->numOfFrames = formatPhotomA->numOfScans * formatPhotomA->framesPerScan;
        formatPhotomB->numOfFrames = formatPhotomB->numOfScans * formatPhotomB->framesPerScan;

        //    Compute default FFT size
        formatInterf->fftsize = determineFFTsize (formatInterf->framesPerScan);

        //    Compute length of allSpectrum
        formatInterf->allSpectrumLength = REGIONS_UNDISPERSED * formatInterf->numOfScans * (formatInterf->fftsize/2);
        
        //    Log results
        midiReportFormat ("INTERFEROMETRY", aprioriDataIsDisp, formatInterf);
        midiReportFormat ("PHOTOMETRY A", aprioriDataIsDisp, formatPhotomA);
        midiReportFormat ("PHOTOMETRY B", aprioriDataIsDisp, formatPhotomB);

        sprintf (midiMessage, "Detected %d Interferometry files", numOfInterf);
        midiReportInfo (midiReportPtr, routine, __FILE__, __LINE__, midiMessage);
    }
    else
    {
        sprintf (fileTemp, "%d", batchNumber);
        midiReportWarning (midiReportPtr, routine, __FILE__, __LINE__, "Inconsistent batch files");
        *error = 1;
    }
 
    //    Check if requested processing is valid. For now just report warning.
    if (!(*error))
    {
        if (!aprioriDataIsDisp)
        {
            midiReportWarning (midiReportPtr, routine, __FILE__, __LINE__, 
                "Data is UNDISPERSED\n                 Algorithm for processing this type of data is not ready");
            *error = 1;
        }
    }

    //    Release memory
    if (fileTemp) free (fileTemp);
    if (localFormat) freeImageFormat (localFormat);
    if (cleanString) free (cleanString);
    if (classification) free (classification);
    if (aprioriCatg) free (aprioriCatg);
     
    return;
}
/******************************************************************************/

