/******************************************************************************
*******************************************************************************
*               European Southern Observatory
*               MIDI Data Reduction Software
*
* File name:    memoryHandling.c
* Description:  Contains the routines for dynamic memory allocation
*               and freeing
*
*
* History:
* 25-Sep-03     (csabet) Created
*******************************************************************************
******************************************************************************/

/******************************************************************************
*   Compiler directives
******************************************************************************/

/******************************************************************************
*   Include files
******************************************************************************/
#include <stdio.h>
#include <cpl.h>
#include <string.h>
#include <time.h>
#include <stdlib.h>
#include "midiGlobal.h"
#include "midiLib.h"
#include "memoryHandling.h"
#include "errorHandling.h"

/******************************************************************************
*   Global Variables
******************************************************************************/

/*============================ C O D E    A R E A ===========================*/


/******************************************************************************
*               European Southern Observatory
*               MIDI Data Reduction Software
*
* Module name:  callocKappaCoefficients
* Input/Output: See function arguments to avoid duplication
* Description:  Allocates memory for the Kappa Coefficients
*
* History:
* 01-Feb-06     (csabet) Created
******************************************************************************/
KappaCoefficients *callocKappaCoefficients (
    int    numOfChannels)
{
    //    Local Declarations
    //    ------------------
    KappaCoefficients    *array=NULL;

    //    Algorithm
    //    ---------
    array = (KappaCoefficients *) calloc (1, sizeof (KappaCoefficients));

    array->k11 = (float *) calloc (numOfChannels, sizeof (float));
    array->k21 = (float *) calloc (numOfChannels, sizeof (float));
    array->k31 = (float *) calloc (numOfChannels, sizeof (float));
    array->k22 = (float *) calloc (numOfChannels, sizeof (float));
    array->k32 = (float *) calloc (numOfChannels, sizeof (float));
    array->k42 = (float *) calloc (numOfChannels, sizeof (float));
    array->sig11 = (float *) calloc (numOfChannels, sizeof (float));
    array->sig21 = (float *) calloc (numOfChannels, sizeof (float));
    array->sig31 = (float *) calloc (numOfChannels, sizeof (float));
    array->sig22 = (float *) calloc (numOfChannels, sizeof (float));
    array->sig32 = (float *) calloc (numOfChannels, sizeof (float));
    array->sig42 = (float *) calloc (numOfChannels, sizeof (float));
    
    return (array);
}
/*****************************************************************************/



/******************************************************************************
*               European Southern Observatory
*               MIDI Data Reduction Software
*
* Module name:  freeKappaCoefficients
* Input/Output: See function arguments to avoid duplication
* Description:  Releases memory for the Kappa coefficients
*
* History:
* 01-Feb-06     (csabet) Created
******************************************************************************/
void freeKappaCoefficients (
    KappaCoefficients    *array)
{
    //    Local Declarations
    //--------------------

    //    Algorithm
    //    ---------
    free (array->k11);
    free (array->sig11);
    free (array->k21);
    free (array->sig21);
    free (array->k31);
    free (array->sig31);
    free (array->k22);
    free (array->sig22);
    free (array->k32);
    free (array->sig32);
    free (array->k42);
    free (array->sig42);
    
    free (array);
    array = NULL;

    return;
}
/*****************************************************************************/



/******************************************************************************
*               European Southern Observatory
*               MIDI Data Reduction Software
*
* Module name:  callocTransferRatios
* Input/Output: See function arguments to avoid duplication
* Description:  Allocates memory for the Transfer Ratios
*
* History:
* 01-Feb-06     (csabet) Created
******************************************************************************/
TransferRatios *callocTransferRatios (
    int    numOfChannels)
{
    //    Local Declarations
    //    ------------------
    TransferRatios    *array=NULL;

    //    Algorithm
    //    ---------
    array = (TransferRatios *) calloc (1, sizeof (TransferRatios));

    array->ka1 = (float *) calloc (numOfChannels, sizeof (float));
    array->siga1 = (float *) calloc (numOfChannels, sizeof (float));
    array->ka2 = (float *) calloc (numOfChannels, sizeof (float));
    array->siga2 = (float *) calloc (numOfChannels, sizeof (float));
    array->kb1 = (float *) calloc (numOfChannels, sizeof (float));
    array->sigb1 = (float *) calloc (numOfChannels, sizeof (float));
    array->kb2 = (float *) calloc (numOfChannels, sizeof (float));
    array->sigb2 = (float *) calloc (numOfChannels, sizeof (float));
    
    return (array);
}
/*****************************************************************************/



/******************************************************************************
*               European Southern Observatory
*               MIDI Data Reduction Software
*
* Module name:  freeTransferRatios
* Input/Output: See function arguments to avoid duplication
* Description:  Releases memory for the Transfer Ratios
*
* History:
* 01-Feb-06     (csabet) Created
******************************************************************************/
void freeTransferRatios (
    TransferRatios    *array)
{
    //    Local Declarations
    //--------------------

    //    Algorithm
    //    ---------
    free (array->ka1);
    free (array->siga1);
    free (array->ka2);
    free (array->siga2);
    free (array->kb1);
    free (array->sigb1);
    free (array->kb2);
    free (array->sigb2);
    
    free (array);
    array = NULL;

    return;
}
/*****************************************************************************/


/******************************************************************************
*               European Southern Observatory
*               MIDI Data Reduction Software
*
* Module name:  callocWaveCalibration
* Input/Output: See function arguments to avoid duplication
* Description:  Allocates memory for an array of 2D float
*
* History:
* 08-Feb-05     (csabet) Created
******************************************************************************/
float **callocWaveCalibration (
    int            regions,
    ImageFormat    *format)
{
    /*  Local Declarations
    --------------------*/
    float    **array=NULL;
    int        i;

    /*  Algorithm
    -----------*/
    array = (float **) calloc (regions, sizeof (float *));
    for (i = 0; i < regions; i++)
        array[i] = (float *) calloc (format->iXWidth, sizeof (float));

    return (array);
}
/*****************************************************************************/



/******************************************************************************
*               European Southern Observatory
*               MIDI Data Reduction Software
*
* Module name:  freeWaveCalibration
* Input/Output: See function arguments to avoid duplication
* Description:  Releases memory for a 2D array of float
*
* History:
* 08-Feb-05     (csabet) Created
******************************************************************************/
void freeWaveCalibration (
    int        regions,
    float    **array)
{
    /*  Local Declarations
    --------------------*/
    int    i;

    /*  Algorithm
    -----------*/
    for (i = 0; i < regions; i++)
        free (array[i]);
    
    free (array);
    array = NULL;

    return;
}
/*****************************************************************************/



/******************************************************************************
*               European Southern Observatory
*               MIDI Data Reduction Software
*
* Module name:  callocDispResult
* Input/Output: See function arguments to avoid duplication
* Description:  Allocates memory for the dispResult array
*
* History:
* 08-Feb-05     (csabet) Created
******************************************************************************/
DispersedResult *callocDispResult (
    int    numOfChannels)
{
    /*  Local Declarations
    --------------------*/
    DispersedResult    *array=NULL;
    int                region;

    /*  Algorithm
    -----------*/
    array = (DispersedResult *) calloc (1, sizeof (DispersedResult));
    
    array->photomA = (float **) calloc (2, sizeof (float *));
    array->photomB = (float **) calloc (2, sizeof (float *));
    array->photomI = (float **) calloc (2, sizeof (float *));
    array->photomAErr = (float **) calloc (2, sizeof (float *));
    array->photomBErr = (float **) calloc (2, sizeof (float *));
    array->photomIErr = (float **) calloc (2, sizeof (float *));

    for (region = 0; region < 2; region++)
    {
        (array->photomA)[region] = (float *) calloc (numOfChannels, sizeof (float));
        (array->photomB)[region] = (float *) calloc (numOfChannels, sizeof (float));
        (array->photomI)[region] = (float *) calloc (numOfChannels, sizeof (float));
        (array->photomAErr)[region] = (float *) calloc (numOfChannels, sizeof (float));
        (array->photomBErr)[region] = (float *) calloc (numOfChannels, sizeof (float));
        (array->photomIErr)[region] = (float *) calloc (numOfChannels, sizeof (float));
    }

    array->trf = (float *) calloc (numOfChannels, sizeof (float));
    array->trfErr = (float *) calloc (numOfChannels, sizeof (float));
    array->normVis2 = (float *) calloc (numOfChannels, sizeof (float));
    array->normVis2Err = (float *) calloc (numOfChannels, sizeof (float));
    array->rawVis2 = (float *) calloc (numOfChannels, sizeof (float));
    array->rawVis2Err = (float *) calloc (numOfChannels, sizeof (float));
    array->calibVis2 = (float *) calloc (numOfChannels, sizeof (float));
    array->calibVis2Err = (float *) calloc (numOfChannels, sizeof (float));
    array->photomNet2 = (float *) calloc (numOfChannels, sizeof (float));
    array->photomNet2Err = (float *) calloc (numOfChannels, sizeof (float));

    return (array);
}
/*****************************************************************************/



/******************************************************************************
*               European Southern Observatory
*               MIDI Data Reduction Software
*
* Module name:  freeDispResult
* Input/Output: See function arguments to avoid duplication
* Description:  Releases memory for the dispResult array
*
* History:
* 08-Feb-05     (csabet) Created
******************************************************************************/
void freeDispResult (
    DispersedResult    *array)
{
    //    Local Declarations
    //    ------------------
    int    region;
    
    //    Algorithm
    //    ---------
    for (region = 0; region < 2; region++)
    {
        free (array->photomA[region]);
        free (array->photomB[region]);
        free (array->photomI[region]);
        free (array->photomAErr[region]);
        free (array->photomBErr[region]);
        free (array->photomIErr[region]);
    }


    free (array->trf);
    free (array->trfErr);
    free (array->normVis2);
    free (array->normVis2Err);
    free (array->rawVis2);
    free (array->rawVis2Err);
    free (array->calibVis2);
    free (array->calibVis2Err);

    free (array->photomA);
    free (array->photomB);
    free (array->photomI);
    free (array->photomAErr);
    free (array->photomBErr);
    free (array->photomIErr);
    free (array->photomNet2);
    free (array->photomNet2Err);
    
    free (array);
    array = NULL;

    return;
}
/*****************************************************************************/



/******************************************************************************
*               European Southern Observatory
*               MIDI Data Reduction Software
*
* Module name:    callocImageFormat
* Input/Output:    See function arguments to avoid duplication
* Description:    Allocates memory for the array image format
*
* History:        
* 29-Mar-04        (csabet) Created
******************************************************************************/
ImageFormat *callocImageFormat (void)
{
    /*    Local Declarations
    --------------------*/
    ImageFormat *array=NULL;

    /*    Algorithm
    -----------*/
    array = (ImageFormat *) calloc (1, sizeof(ImageFormat));
    array->shutterId = (char *) calloc (MIN_STRING_LENGTH, sizeof (char));
    array->grismId = (char *) calloc (MIN_STRING_LENGTH, sizeof (char));
    array->slitName = (char *) calloc (MIN_STRING_LENGTH, sizeof (char));
    array->targetName = (char *) calloc (MIN_STRING_LENGTH, sizeof (char));
    array->cameraId = (char *) calloc (MIN_STRING_LENGTH, sizeof (char));
    array->filterName = (char *) calloc (MIN_STRING_LENGTH, sizeof (char));
    array->tplStart = (char *) calloc (MIN_STRING_LENGTH, sizeof (char));
    array->beamCombiner = (char *) calloc (MIN_STRING_LENGTH, sizeof (char));
    array->tplName = (char *) calloc (MIN_STRING_LENGTH, sizeof (char));
    array->obsTech = (char *) calloc (MIN_STRING_LENGTH, sizeof (char));
    array->obsCatg = (char *) calloc (MIN_STRING_LENGTH, sizeof (char));
    array->obsType = (char *) calloc (MIN_STRING_LENGTH, sizeof (char));
    
    return (array);
}
/*****************************************************************************/


/******************************************************************************
*                European Southern Observatory
*                MIDI Data Reduction Software
*
* Module name:    freeImageFormat
* Input/Output:    See function arguments to avoid duplication
* Description:    Releases memory for the array image format
*
* History:        
* 29-Mar-04        (csabet) Created
******************************************************************************/
void freeImageFormat (
    ImageFormat *array)
{
    /*    Local Declarations
    --------------------*/

    /*    Algorithm
    -----------*/
    free (array->shutterId);
    free (array->grismId);
    free (array->slitName);
    free (array->targetName);
    free (array->tplStart);
    free (array->beamCombiner);
    free (array->cameraId);
    free (array->filterName);
    free (array->tplName);
    free (array->obsTech);
    free (array->obsCatg);
    free (array->obsType);
    free (array);
    array = NULL;
    
    return;
}
/*****************************************************************************/



/******************************************************************************
*                European Southern Observatory
*                MIDI Data Reduction Software
*
* Module name:  callocFilterInfo
* Input/Output: See function arguments to avoid duplication
* Description:  Allocates memory for the filterInfo array
*
* History:
* 29-Mar-04     (csabet) Created
******************************************************************************/
FilterData *callocFilterInfo (void)
{
    /*  Local Declarations
    --------------------*/
    FilterData *array=NULL;

    /*  Algorithm
    -----------*/
    array = (FilterData *) calloc (1, sizeof(FilterData));
    array->filterName = (char *) calloc (MAX_STRING_LENGTH, sizeof (char));

    return (array);
}
/*****************************************************************************/


/******************************************************************************
*               European Southern Observatory
*               MIDI Data Reduction Software
*
* Module name:  freeFilterInfo
* Input/Output: See function arguments to avoid duplication
* Description:  Releases memory for the filterInfo array
*
* History:
* 29-Mar-04     (csabet) Created
******************************************************************************/
void freeFilterInfo (
    FilterData *array)
{
    /*  Local Declarations
    --------------------*/

    /*  Algorithm
    -----------*/
    free (array->filterName);
    free (array);
    array = NULL;

    return;
}
/*****************************************************************************/


/******************************************************************************
*               European Southern Observatory
*               MIDI Data Reduction Software
*
* Module name:  callocCalibVis
* Input/Output: See function arguments to avoid duplication
* Description:  Allocates memory for the trf array
*
* History:
* 29-Mar-04     (csabet) Created
******************************************************************************/
CalibratedVisibility *callocCalibVis (void)
{
    /*  Local Declarations
    --------------------*/
    CalibratedVisibility *array=NULL;

    /*  Algorithm
    -----------*/
    array = (CalibratedVisibility *) calloc (1, sizeof (CalibratedVisibility));
    array->vis = (float *) calloc (REGIONS_UNDISPERSED, sizeof (float));
    array->visErr = (float *) calloc (REGIONS_UNDISPERSED, sizeof (float));
    array->visSqrd = (float *) calloc (REGIONS_UNDISPERSED, sizeof (float));
    array->visSqrdErr = (float *) calloc (REGIONS_UNDISPERSED, sizeof (float));


    return (array);
}
/*****************************************************************************/


/******************************************************************************
*               European Southern Observatory
*               MIDI Data Reduction Software
*
* Module name:  freeCalibVis
* Input/Output: See function arguments to avoid duplication
* Description:  Releases memory for the calibratedVis array
*
* History:
* 29-Mar-04     (csabet) Created
******************************************************************************/
void freeCalibVis (
    CalibratedVisibility *array)
{
    /*  Local Declarations
    --------------------*/

    /*  Algorithm
    -----------*/
    free (array->vis);
    free (array->visErr);
    free (array->visSqrd);
    free (array->visSqrdErr);
    free (array);
    array = NULL;

    return;
}
/*****************************************************************************/



/******************************************************************************
*               European Southern Observatory
*               MIDI Data Reduction Software
*
* Module name:  callocTrFunction
* Input/Output: See function arguments to avoid duplication
* Description:  Allocates memory for the trf array
*
* History:
* 29-Mar-04     (csabet) Created
******************************************************************************/
TransferFunction *callocTrFunction (
    int    size)
{
    /*  Local Declarations
    --------------------*/
    TransferFunction *array=NULL;

    /*  Algorithm
    -----------*/
    array = (TransferFunction *) calloc (1, sizeof (TransferFunction));
    array->trf = (float *) calloc (size , sizeof (float));
    array->trfErr = (float *) calloc (size , sizeof (float));

    return (array);
}
/*****************************************************************************/


/******************************************************************************
*               European Southern Observatory
*               MIDI Data Reduction Software
*
* Module name:  freeTrFunction
* Input/Output: See function arguments to avoid duplication
* Description:  Releases memory for the trf array
*
* History:
* 29-Mar-04     (csabet) Created
******************************************************************************/
void freeTrFunction (
    TransferFunction *array)
{
    /*  Local Declarations
    --------------------*/

    /*  Algorithm
    -----------*/
    free (array->trfErr);
    free (array->trf);
    free (array);
    array = NULL;

    return;
}
/*****************************************************************************/


/******************************************************************************
*               European Southern Observatory
*               MIDI Data Reduction Software
*
* Module name:  callocCalibrator
* Input/Output: See function arguments to avoid duplication
* Description:  Allocates memory for the Calibrator array
*
* History:
* 29-Mar-04     (csabet) Created
******************************************************************************/
CalibratorParam *callocCalibrator (void)
{
    /*  Local Declarations
    --------------------*/
    CalibratorParam *array=NULL;

    /*  Algorithm
    -----------*/
    array = (CalibratorParam *) calloc (1, sizeof (CalibratorParam));
    array->calibName = (char *) calloc (MIN_STRING_LENGTH, sizeof (char));

    return (array);
}
/*****************************************************************************/


/******************************************************************************
*               European Southern Observatory
*               MIDI Data Reduction Software
*
* Module name:  freeCalibrator
* Input/Output: See function arguments to avoid duplication
* Description:  Releases memory for the calibrator array
*
* History:
* 29-Mar-04     (csabet) Created
******************************************************************************/
void freeCalibrator (
    CalibratorParam *array)
{
    /*  Local Declarations
    --------------------*/

    /*  Algorithm
    -----------*/
    free (array->calibName);
    free (array);
    array = NULL;

    return;
}
/*****************************************************************************/


/******************************************************************************
*               European Southern Observatory
*               MIDI Data Reduction Software
*
* Module name:  callocIauExchange
* Input/Output: See function arguments to avoid duplication
* Description:  Allocates memory for the IAU data structure
*
* History:
* 09-Feb-05     (csabet) Created
******************************************************************************/
IauExchange *callocIauExchange (
    int numOfTelescopes,
    int    numOfChannels)
{
    //    Local Declarations
    //    ------------------
    IauExchange *iauArray=NULL;
    int         i;

    //    Algorithm
    //    ---------
    iauArray = (IauExchange *) calloc (1, sizeof (IauExchange));

    //    OI_ARRAY
    iauArray->array = (OiArray *) calloc (1, sizeof (OiArray));
    iauArray->array->revision = (char *) calloc (MAX_STRING_LENGTH, sizeof (char));
    iauArray->array->arrname = (char *) calloc (MAX_STRING_LENGTH, sizeof (char));
    iauArray->array->frame = (char *) calloc (MAX_STRING_LENGTH, sizeof (char));
    iauArray->array->nelement = numOfTelescopes;
    iauArray->array->elem = (Element *) calloc (iauArray->array->nelement, sizeof(Element));
    for (i = 0; i < iauArray->array->nelement; i++)
    {
        iauArray->array->elem[i].tel_name = (char *) calloc (MAX_STRING_LENGTH, sizeof (char));
        iauArray->array->elem[i].sta_name = (char *) calloc (MAX_STRING_LENGTH, sizeof (char));
        iauArray->array->elem[i].staxyz = (double *) calloc (3, sizeof (double));
        
    }
    
    //    OI_TARGET
    iauArray->targets = (OiTarget *) calloc (1, sizeof (OiTarget));
    iauArray->targets->revision = (char *) calloc (MAX_STRING_LENGTH, sizeof (char));
    iauArray->targets->ntarget = 1;
    iauArray->targets->targ = (Target *) calloc (iauArray->targets->ntarget, sizeof(Target));
    iauArray->targets->targ->spectyp = (char *) calloc (MAX_STRING_LENGTH, sizeof (char));
    iauArray->targets->targ->veldef = (char *) calloc (MAX_STRING_LENGTH, sizeof (char));
    iauArray->targets->targ->veltyp = (char *) calloc (MAX_STRING_LENGTH, sizeof (char));
    iauArray->targets->targ->target = (char *) calloc (MAX_STRING_LENGTH, sizeof (char));

    //    OI_WAVELENGTH
    iauArray->wavelength = (OiWavelength *) calloc (1, sizeof (OiWavelength));
    iauArray->wavelength->revision = (char *) calloc (MAX_STRING_LENGTH, sizeof (char));
    iauArray->wavelength->insname = (char *) calloc (MAX_STRING_LENGTH, sizeof (char));
    iauArray->wavelength->nwave = numOfChannels;
    iauArray->wavelength->eff_wave = (float *) calloc (iauArray->wavelength->nwave, sizeof(float));
    iauArray->wavelength->eff_band = (float *) calloc (iauArray->wavelength->nwave, sizeof(float));

    //    OI_VIS
    iauArray->vis = (OiVis *) calloc (1, sizeof (OiVis));
    iauArray->vis->revision = (char *) calloc (MAX_STRING_LENGTH, sizeof (char));
    iauArray->vis->date_obs = (char *) calloc (MAX_STRING_LENGTH, sizeof (char));
    iauArray->vis->arrname = (char *) calloc (MAX_STRING_LENGTH, sizeof (char));
    iauArray->vis->insname = (char *) calloc (MAX_STRING_LENGTH, sizeof (char));
    iauArray->vis->numrec = 1;
    iauArray->vis->record = (OiVisRecord *) calloc (iauArray->vis->numrec, sizeof(OiVisRecord));
    for (i = 0; i < iauArray->vis->numrec; i++)
    {
        iauArray->vis->record[i].sta_index = (int *) calloc (2, sizeof(int));
        iauArray->vis->record[i].visamp = (double *) calloc (iauArray->wavelength->nwave, sizeof(double));
        iauArray->vis->record[i].visamperr = (double *) calloc (iauArray->wavelength->nwave, sizeof(double));
        iauArray->vis->record[i].visphi = (double *) calloc (iauArray->wavelength->nwave, sizeof(double));
        iauArray->vis->record[i].visphierr = (double *) calloc (iauArray->wavelength->nwave, sizeof(double));
        iauArray->vis->record[i].flag = (char *) calloc (MIN_STRING_LENGTH, sizeof(char));
    }

    //    OI_VIS2
    iauArray->vis2 = (OiVis2 *) calloc (1, sizeof (OiVis2));
    iauArray->vis2->revision = (char *) calloc (MAX_STRING_LENGTH, sizeof (char));
    iauArray->vis2->date_obs = (char *) calloc (MAX_STRING_LENGTH, sizeof (char));
    iauArray->vis2->arrname = (char *) calloc (MAX_STRING_LENGTH, sizeof (char));
    iauArray->vis2->insname = (char *) calloc (MAX_STRING_LENGTH, sizeof (char));
    iauArray->vis2->numrec = 1;
    iauArray->vis2->record = (OiVis2Record *) calloc (iauArray->vis2->numrec, sizeof (OiVis2Record));
    for(i = 0; i < iauArray->vis2->numrec; i++)
    {
        iauArray->vis2->record[i].vis2data = (double *) calloc (iauArray->wavelength->nwave, sizeof (double));
        iauArray->vis2->record[i].vis2err = (double *) calloc (iauArray->wavelength->nwave, sizeof (double));
        iauArray->vis2->record[i].flag = (char *) calloc (MIN_STRING_LENGTH, sizeof (char));
        iauArray->vis2->record[i].sta_index = (int *) calloc (2, sizeof (int));
    }

    return (iauArray);
}
/*****************************************************************************/


/******************************************************************************
*               European Southern Observatory
*               MIDI Data Reduction Software
*
* Module name:  freeIauExchange
* Input/Output: See function arguments to avoid duplication
* Description:  Releases memory for the IAU data structure
*
* History:
* 23-Feb-04     (csabet) Created
******************************************************************************/
void freeIauExchange (
    IauExchange *iauArray)
{
    //    Local Declarations
    //    ------------------
    int i;

    //    Algorithm
    //    ---------

    //    Release OI_ARRAY
    free (iauArray->array->frame);
    free (iauArray->array->arrname);
    free (iauArray->array->revision);
    for (i = 0; i < iauArray->array->nelement; i++)
    {
        free (iauArray->array->elem[i].tel_name);
        free (iauArray->array->elem[i].sta_name);
        free (iauArray->array->elem[i].staxyz);
    }
    free (iauArray->array->elem);
    free (iauArray->array);

    //    Release OI_TARGET
    free (iauArray->targets->revision);
    free (iauArray->targets->targ->spectyp);
    free (iauArray->targets->targ->veldef);
    free (iauArray->targets->targ->veltyp);
    free (iauArray->targets->targ->target);
    free (iauArray->targets->targ);
    free (iauArray->targets);
    
    //    Release OI_WAVELENGTH
    free (iauArray->wavelength->eff_wave);
    free (iauArray->wavelength->eff_band);
    free (iauArray->wavelength->revision);
    free (iauArray->wavelength->insname);
    free (iauArray->wavelength);

    //    Release OI_VIS
    free (iauArray->vis->revision);
    free (iauArray->vis->date_obs);
    free (iauArray->vis->arrname);
    free (iauArray->vis->insname);
    for (i = 0; i < iauArray->vis->numrec; i++)
    {
        free (iauArray->vis->record[i].visamp);
        free (iauArray->vis->record[i].visamperr);
        free (iauArray->vis->record[i].visphi);
        free (iauArray->vis->record[i].visphierr);
        free (iauArray->vis->record[i].sta_index);
        free (iauArray->vis->record[i].flag);
    }
    free (iauArray->vis->record);
    free (iauArray->vis);

    //    Release OI_VIS2
    free (iauArray->vis2->revision);
    free (iauArray->vis2->date_obs);
    free (iauArray->vis2->arrname);
    free (iauArray->vis2->insname);
    for(i = 0; i < iauArray->vis2->numrec; i++)
    {
        free (iauArray->vis2->record[i].vis2data);
        free (iauArray->vis2->record[i].vis2err);;
        free (iauArray->vis2->record[i].sta_index);
        free (iauArray->vis2->record[i].flag);
    }
    free (iauArray->vis2->record);
    free (iauArray->vis2);

    free (iauArray);
    iauArray = NULL;

    return;
}
/*****************************************************************************/



/******************************************************************************
*               European Southern Observatory
*               MIDI Data Reduction Software
*
* Module name:  callocPhotometry
* Input/Output: See function arguments to avoid duplication
* Description:  Allocates memory for the photometry result
*
* History:
* 16-Jan-04     (csabet) Created
******************************************************************************/
PhotometryResult *callocPhotometry (void)
{
    /*  Local Declarations
    --------------------*/
    PhotometryResult *array=NULL;

    /*  Algorithm
    -----------*/
    array = (PhotometryResult *) calloc (1, sizeof (PhotometryResult));

    array->photomA = (float *) calloc (2, sizeof (float));
    array->photomAErr = (float *) calloc (2, sizeof (float));

    array->photomB = (float *) calloc (2, sizeof (float));
    array->photomBErr = (float *) calloc (2, sizeof (float));

    array->chop2ChopAErr = (float *) calloc (2, sizeof (float));
    array->chop2ChopBErr = (float *) calloc (2, sizeof (float));

    return (array);
}
/*****************************************************************************/


/******************************************************************************
*               European Southern Observatory
*               MIDI Data Reduction Software
*
* Module name:  freePhotometry
* Input/Output: See function arguments to avoid duplication
* Description:  Releases memory for the photometry result
*
* History:
* 16-Jan-04     (csabet) Created
******************************************************************************/
void freePhotometry (
    PhotometryResult *array)
{
    /*  Local Declarations
    --------------------*/

    /*  Algorithm
    -----------*/
    free (array->photomA);
    free (array->photomAErr);

    free (array->photomB);
    free (array->photomBErr);

    free (array->chop2ChopAErr);
    free (array->chop2ChopBErr);

    free (array);
    array = NULL;

    return;
}
/*****************************************************************************/


/******************************************************************************
*               European Southern Observatory
*               MIDI Data Reduction Software
*
* Module name:  callocRawVis
* Input/Output: See function arguments to avoid duplication
* Description:  Allocates memory for the interferometry result
*
* History:
* 15-Jan-04     (csabet) Created
******************************************************************************/
RawVisibility *callocRawVis (void)
{
    /*  Local Declarations
    --------------------*/
    RawVisibility *array=NULL;

    /*  Algorithm
    -----------*/
    array = (RawVisibility *) calloc (1, sizeof (RawVisibility));

    return (array);
}
/*****************************************************************************/



/******************************************************************************
*               European Southern Observatory
*               MIDI Data Reduction Software
*
* Module name:  freeRawVis
* Input/Output: See function arguments to avoid duplication
* Description:  Releases memory for the interferometry result data
*
* History:
* 15-Jan-04     (csabet) Created
******************************************************************************/
void freeRawVis (
    RawVisibility *array)
{
    /*  Local Declarations
    --------------------*/

    /*  Algorithm
    -----------*/
    free (array);
    array = NULL;

    return;
}
/*****************************************************************************/



/******************************************************************************
*               European Southern Observatory
*               MIDI Data Reduction Software
*
* Module name:  callocMidiCompressed
* Input/Output: See function arguments to avoid duplication
* Description:  Allocates memory for the compressed data
*
* History:
* 05-Nov-03     (csabet) Created
******************************************************************************/
CompressedData *callocMidiCompressed (
    ImageFormat            *format)
{
    //    Local Declarations
    //    ------------------
    CompressedData    *array=NULL;
    int                i, X;

    //    Algorithm
    //    ---------
    array = (CompressedData *) calloc (1, sizeof (CompressedData));

    array->iDispFringe = (float ***) calloc (format->numOfRegionsToProcess, sizeof(float **));
    for (i = 0; i < format->numOfRegionsToProcess; i++)
    {
        (array->iDispFringe)[i] = (float **) calloc (format->iXWidth, sizeof(float *));
        for (X = 0; X < format->iXWidth; X++)
            (array->iDispFringe)[i][X] = (float *) calloc (format->numOfFrames, sizeof(float));
    }

    array->rejectList = (int **) calloc (format->iXWidth, sizeof (int *));
    for (i = 0; i < format->iXWidth; i++)
        array->rejectList[i] = (int *) calloc (format->numOfFrames, sizeof (int));

    array->badScanList = (int *) calloc (format->numOfScans, sizeof (int));
    
    array->image = (float **) calloc (format->numOfRegionsToProcess, sizeof(float *));
    for (i = 0; i < format->numOfRegionsToProcess; i++)
        array->image[i] = (float *) calloc (format->iXWidth * format->iYWidth, sizeof(float));

    array->iFringe1 = (double *) calloc (format->numOfFrames, sizeof(double));
    array->iFringe2 = (double *) calloc (format->numOfFrames, sizeof(double));
    array->iFringe = (double *) calloc (format->numOfFrames, sizeof(double));
    array->allSpectrum = (float *) calloc (format->allSpectrumLength, sizeof(float));
    array->dcLevels = (float *) calloc (REGIONS_UNDISPERSED * format->numOfScans, sizeof (float));
    array->tarType = (char *) calloc(format->numOfFrames, sizeof(char));
    array->time = (float *) calloc(format->numOfFrames, sizeof(float));
    array->localOPD = (float *) calloc(format->numOfFrames, sizeof(float));
    array->bigDL = (float *) calloc(format->numOfFrames, sizeof(float));

    return (array);
}
/*****************************************************************************/


 
/******************************************************************************
*               European Southern Observatory
*               MIDI Data Reduction Software
*
* Module name:  freeCompressedData
* Input/Output: See function arguments to avoid duplication
* Description:  Releases all the memory related to the
*               compressed data
*
* History:
* 05-Nov-03     (csabet) Created
******************************************************************************/
void freeCompressedData (
    ImageFormat            *format,
    CompressedData        *array)
{
    //    Local Declarations
    //    ------------------
    int i, j;
 
    //    Algorithm
    //    ---------
    for (i = 0; i < format->numOfRegionsToProcess; i++)
    {
        for (j = 0; j < format->iXWidth; j++)
            free ((array->iDispFringe)[i][j]);
           
        free ((array->iDispFringe)[i]);
    }
    free (array->iDispFringe);

    for (i = 0; i < format->numOfRegionsToProcess; i++)
    {
        free (array->image[i]);
    }
    free (array->image);

    for (i = 0; i < format->iXWidth; i++)
        free (array->rejectList[i]);
    free (array->rejectList);

    free (array->badScanList);
    free (array->iFringe1);
    free (array->iFringe2);
    free (array->iFringe);
    free (array->allSpectrum);
    free (array->dcLevels);
    free (array->tarType);
    free (array->time);
    free (array->localOPD);
    free (array->bigDL);
 
    free (array);
    array = NULL;
    
    return;
}
/*****************************************************************************/




/******************************************************************************
*               European Southern Observatory
*               MIDI Data Reduction Software
*
* Module name:  callocMidiFiles
* Input/Output: See function arguments to avoid duplication
* Description:  Allocates memory for the MIDI file Structure
*
* History:
* 05-Nov-03     (csabet) Created
******************************************************************************/
MidiFiles *callocMidiFiles (
    int stringLength)
{
    /*  Local Declarations
    --------------------*/
    MidiFiles   *fileNames=NULL;

    /*  Algorithm
    -----------*/
    fileNames = (MidiFiles *)  calloc (1, sizeof (MidiFiles));
    fileNames->inFitsList = (char *) calloc (stringLength, sizeof (char));
    fileNames->inFitsClassified = (char *) calloc (stringLength, sizeof (char));
    fileNames->inFitsBatch = (char *) calloc (stringLength, sizeof (char));
    fileNames->inFileDir = (char *) calloc (stringLength, sizeof (char));
    fileNames->maskFileDir = (char *) calloc (stringLength, sizeof (char));
    fileNames->maskFileName = (char *) calloc (stringLength, sizeof (char));
    fileNames->inFitsName = (char *) calloc (stringLength, sizeof (char));
    fileNames->outFitsName = (char *) calloc (stringLength, sizeof (char));
    fileNames->outWaterfallName = (char *) calloc (stringLength, sizeof (char));
    fileNames->outWaterpowerName = (char *) calloc (stringLength, sizeof (char));
    fileNames->outAveImageName = (char *) calloc (stringLength, sizeof (char));
    fileNames->outChSpectrumName = (char *) calloc (stringLength, sizeof (char));
    fileNames->calibDbName = (char *) calloc (stringLength, sizeof (char));
    fileNames->grismFoilName = (char *) calloc (stringLength, sizeof (char));
    fileNames->prismFoilName = (char *) calloc (stringLength, sizeof (char));
    fileNames->calibDbDir = (char *) calloc (stringLength, sizeof (char));
    fileNames->trfNameRead = (char *) calloc (stringLength, sizeof (char));
    fileNames->trfNameWrite = (char *) calloc (stringLength, sizeof (char));
    fileNames->trfHistoryName = (char *) calloc (stringLength, sizeof (char));
    fileNames->trrNameRead = (char *) calloc (stringLength, sizeof (char));
    fileNames->trrNameWrite = (char *) calloc (stringLength, sizeof (char));
    fileNames->trrHistoryName = (char *) calloc (stringLength, sizeof (char));
    fileNames->wclNameRead = (char *) calloc (stringLength, sizeof (char));
    fileNames->wclNameWrite = (char *) calloc (stringLength, sizeof (char));
    fileNames->transferRatiosName = (char *) calloc (stringLength, sizeof (char));
    fileNames->wclHistoryName = (char *) calloc (stringLength, sizeof (char));
    fileNames->archFileName = (char *) calloc (stringLength, sizeof (char));
    fileNames->pipeFileName = (char *) calloc (stringLength, sizeof (char));
    fileNames->waveCalibName = (char *) calloc (stringLength, sizeof (char));

    return (fileNames);
}
/*****************************************************************************/



/******************************************************************************
*               European Southern Observatory
*               MIDI Data Reduction Software
*
* Module name:  freeMidiFiles
* Input/Output: See function arguments to avoid duplication
* Description:  Releases all MIDI file structures
*
* History:
* 05-Nov-03     (csabet) Created
******************************************************************************/
void freeMidiFiles (
    MidiFiles   *fileNames)
{
    /*  Local Declarations
    --------------------*/

    /*  Algorithm
    -----------*/
    free (fileNames->inFitsList);
    free (fileNames->inFitsClassified);
    free (fileNames->inFitsBatch);
    free (fileNames->inFileDir);
    free (fileNames->maskFileDir);
    free (fileNames->maskFileName);
    free (fileNames->inFitsName);
    free (fileNames->outFitsName);
    free (fileNames->outWaterfallName);
    free (fileNames->outWaterpowerName);
    free (fileNames->outAveImageName);
    free (fileNames->outChSpectrumName);
    free (fileNames->calibDbName);
    free (fileNames->grismFoilName);
    free (fileNames->prismFoilName);
    free (fileNames->calibDbDir);
    free (fileNames->trfNameRead);
    free (fileNames->trfNameWrite);
    free (fileNames->trfHistoryName);
    free (fileNames->trrNameRead);
    free (fileNames->trrNameWrite);
    free (fileNames->trrHistoryName);
    free (fileNames->wclNameRead);
    free (fileNames->wclNameWrite);
    free (fileNames->wclHistoryName);
    free (fileNames->archFileName);
    free (fileNames->transferRatiosName);
    free (fileNames->pipeFileName);
    free (fileNames->waveCalibName);
    free (fileNames);
    fileNames = NULL;

    return;
}
/*****************************************************************************/

/******************************************************************************
*                European Southern Observatory
*                MIDI Data Reduction Software
*
* Module name:    callocDspTrn
* Input/Output:    See function arguments to avoid duplication
* Description:    Allocates memory for the Dispersive Transmission data structure
*
* History:        
* 26-Aug-04        (csabet) Created
******************************************************************************/
DispersiveTrans *callocDspTrn (
    int            numOfFiles)
{
    /*    Local Declarations
    --------------------*/
    DispersiveTrans    *array=NULL;
    int                i;

    /*    Algorithm
    -----------*/
    array = (DispersiveTrans *) calloc (1, sizeof (DispersiveTrans));
    array->numOfFiles = numOfFiles;
    array->target = (MidiCoords *) calloc (numOfFiles, sizeof (MidiCoords));
    
    array->integTime = (float *) calloc (numOfFiles, sizeof (float));
    array->integFlux = (float *) calloc (numOfFiles, sizeof (float));
    array->meanFlux = (float *) calloc (numOfFiles, sizeof (float));
    array->transmission = (float *) calloc (numOfFiles/2, sizeof (float));

    array->beamCombiner = (char **) calloc (numOfFiles, sizeof (char *));
    array->shutterId = (char **) calloc (numOfFiles, sizeof (char *));
    array->cameraId = (char **) calloc (numOfFiles, sizeof (char *));
    array->grismId = (char **) calloc (numOfFiles, sizeof (char *));
    array->filterName = (char **) calloc (numOfFiles, sizeof (char *));
    for (i = 0; i < array->numOfFiles; i++)
    {
        array->beamCombiner[i] = (char *) calloc (MIN_STRING_LENGTH, sizeof (char));
        array->shutterId[i] = (char *) calloc (MIN_STRING_LENGTH, sizeof (char));
        array->cameraId[i] = (char *) calloc (MIN_STRING_LENGTH, sizeof (char));
        array->grismId[i] = (char *) calloc (MIN_STRING_LENGTH, sizeof (char));
        array->filterName[i] = (char *) calloc (MIN_STRING_LENGTH, sizeof (char));
    }

    return (array);
}
/*****************************************************************************/



/******************************************************************************
*                European Southern Observatory
*                MIDI Data Reduction Software
*
* Module name:    freeDspTrn
* Input/Output:    See function arguments to avoid duplication
* Description:    Releases all the memory related to the dspTrn data structure
*
* History:        
* 26-Aug-04        (csabet) Created
******************************************************************************/
void freeDspTrn (
    DispersiveTrans    *array)
{
    /*    Local Declarations
    --------------------*/
    int    i;
    
    /*    Algorithm
    -----------*/
    free (array->integTime);
    free (array->integFlux);
    free (array->meanFlux);
    free (array->transmission);
    free (array->target);
    for (i = 0; i < array->numOfFiles; i++)
    {
        free (array->beamCombiner[i]);
        free (array->shutterId[i]);
        free (array->cameraId[i]);
        free (array->grismId[i]);
        free (array->filterName[i]);
    }
    free (array->beamCombiner);
    free (array->shutterId);
    free (array->cameraId);
    free (array->grismId);
    free (array->filterName);

    free (array);
    array = NULL;
   
    return;
}
/*****************************************************************************/



/******************************************************************************
*                European Southern Observatory
*                MIDI Data Reduction Software
*
* Module name:    callocRefPix
* Input/Output:    See function arguments to avoid duplication
* Description:    Allocates memory for the Reference Pixel alignment data structure
*
* History:        
* 11-Aug-04        (csabet) Created
* 28-Aug-06        (csabet) Modified
******************************************************************************/
ReferencePixel *callocRefPix (
    int            numOfFiles,
    int            numOfBeams)
{
    //    Local Declarations
    //    ------------------
    ReferencePixel    *array=NULL;
    int             i;
    
    //    Algorithm
    //    ---------
    array = (ReferencePixel *) calloc (1, sizeof (ReferencePixel));
    array->numOfExposures = numOfFiles;
    array->exposure = (RefPixExposure *) calloc (array->numOfExposures, sizeof (RefPixExposure));
    
    for (i = 0; i < array->numOfExposures; i++)
    {
        array->exposure[i].shutterId = (char *) calloc (MIN_STRING_LENGTH, sizeof (char));
        array->exposure[i].beamCombiner = (char *) calloc (MIN_STRING_LENGTH, sizeof (char));
        array->exposure[i].numOfBeams = numOfBeams;
        array->exposure[i].centroid = (MidiCoords *) calloc (array->exposure[i].numOfBeams, sizeof (MidiCoords));
    }
        
    return (array);
}
/*****************************************************************************/



/******************************************************************************
*                European Southern Observatory
*                MIDI Data Reduction Software
*
* Module name:    freeRefPix
* Input/Output:    See function arguments to avoid duplication
* Description:    Releases all the memory related to the refPix data structure
*
* History:        
* 11-Aug-04        (csabet) Created
* 28-Aug-06        (csabet) Modified
******************************************************************************/
void freeRefPix (
    ReferencePixel    *array)
{
    //    Local Declarations
    //    ------------------
    int    i;
    
    //    Algorithm
    //    ---------
    for (i = 0; i < array->numOfExposures; i++)
    {
        free (array->exposure[i].beamCombiner);
        free (array->exposure[i].shutterId);
        free (array->exposure[i].centroid);
    }
    free (array->exposure);
    free (array);
    array = NULL;
       
    return;
}
/*****************************************************************************/



/******************************************************************************
*                European Southern Observatory
*                MIDI Data Reduction Software
*
* Module name:    callocDetLin
* Input/Output:    See function arguments to avoid duplication
* Description:    Allocates memory for the detector linearity data structure
*
* History:        
* 06-Aug-04        (csabet) Created
******************************************************************************/
DetLinearity *callocDetLin (
    int            numOfFiles,
    ImageFormat    *format)
{
    /*    Local Declarations
    --------------------*/
    DetLinearity    *array=NULL;
    int                i;

    /*    Algorithm
    -----------*/
    array = (DetLinearity *) calloc (1, sizeof (DetLinearity));
    array->size = numOfFiles;

    array->mean = (float *) calloc (array->size, sizeof(float));
    array->grismId = (char *) calloc (MIN_STRING_LENGTH, sizeof (char));
    array->saturated = (int *) calloc (array->size, sizeof (int));
    array->integTime = (float *) calloc (array->size, sizeof(float));
    array->deviation = (float *) calloc (format->iXWidth * format->iYWidth, sizeof (float));
    array->aveImage = (float **) calloc (array->size, sizeof (float *));
    for (i = 0; i < array->size; i++)
        array->aveImage[i] = (float *) calloc (format->iXWidth * format->iYWidth, sizeof (float));

    return (array);
}
/*****************************************************************************/



/******************************************************************************
*                European Southern Observatory
*                MIDI Data Reduction Software
*
* Module name:    freeDetLin
* Input/Output:    See function arguments to avoid duplication
* Description:    Releases all the memory related to the DetLinearity data structure
*
* History:        
* 06-Aug-04        (csabet) Created
******************************************************************************/
void freeDetLin (
    DetLinearity    *array)
{
    /*    Local Declarations
    --------------------*/
    int    i;

    /*    Algorithm
    -----------*/
    free (array->grismId);
    free (array->saturated);
    free (array->integTime);
    free (array->mean);
    free (array->deviation);

    for (i = 0; i < array->size; i++)
        free (array->aveImage[i]);
    free (array->aveImage);

    free (array);
    array = NULL;
       
    return;
}
/*****************************************************************************/



/******************************************************************************
*                European Southern Observatory
*                MIDI Data Reduction Software
*
* Module name:    callocDetRon
* Input/Output:    See function arguments to avoid duplication
* Description:    Allocates memory for the noise pattern data structure
*
* History:        
* 04-Aug-04        (csabet) Created
******************************************************************************/
DetRonNoise *callocDetRon (
    ImageFormat    *imageSize)
{
    /*    Local Declarations
    --------------------*/
    DetRonNoise    *array=NULL;

    /*    Algorithm
    -----------*/
    array = (DetRonNoise *) calloc (1, sizeof (DetRonNoise));

    array->pattern = (float *) calloc (imageSize->iXWidth * imageSize->iYWidth, sizeof(float));
    array->aveImage = (float *) calloc (imageSize->iXWidth * imageSize->iYWidth, sizeof(float));
    
    return (array);
}
/*****************************************************************************/



/******************************************************************************
*                European Southern Observatory
*                MIDI Data Reduction Software
*
* Module name:    freeDetRon
* Input/Output:    See function arguments to avoid duplication
* Description:    Releases all the memory related to the DetRonNoise data structure
*
* History:        
* 04-Aug-04        (csabet) Created
******************************************************************************/
void freeDetRon (
    DetRonNoise    *array)
{
    /*    Local Declarations
    --------------------*/

    /*    Algorithm
    -----------*/
    free (array->pattern);
    free (array->aveImage);
    free (array);
    array = NULL;
       
    return;
}
/*****************************************************************************/


/******************************************************************************
*                European Southern Observatory
*                MIDI Data Reduction Software
*
* Module name:    callocImageQuality
* Input/Output:    See function arguments to avoid duplication
* Description:    Allocates memory for the ImageQuality data structure
*
* History:        
* 26-Apr-05        (csabet) Created
******************************************************************************/
ImageQuality *callocImageQuality (
    ImageFormat    *format)
{
    //    Local Declarations
    //    ------------------
    ImageQuality    *image=NULL;
    int                i;

    //    Algorithm
    //    ---------
    image = (ImageQuality *) calloc (1, sizeof (ImageQuality));

    image->aveImage = (float **) calloc (format->numOfDetectorRegions, sizeof (float *));
    image->aveImageSky = (float **) calloc (format->numOfDetectorRegions, sizeof (float *));
    for (i = 0; i < format->numOfDetectorRegions; i++)
    {
        image->aveImage[i] = (float *) calloc (format->iXWidth * format->iYWidth, sizeof(float));
        image->aveImageSky[i] = (float *) calloc (format->iXWidth * format->iYWidth, sizeof(float));
    }

    image->targetFlux = (float *) calloc (format->numOfDetectorRegions, sizeof (float));
    image->targetPixelCount = (int *) calloc (format->numOfDetectorRegions, sizeof (int));
    image->coordX = (double *) calloc (format->numOfDetectorRegions, sizeof (double));
    image->coordY = (double *) calloc (format->numOfDetectorRegions, sizeof (double));
    image->sizeX = (double *) calloc (format->numOfDetectorRegions, sizeof (double));
    image->sizeY = (double *) calloc (format->numOfDetectorRegions, sizeof (double));
    image->tarType = (char *) calloc (format->numOfFrames, sizeof(char));
    
    return (image);
}
/*****************************************************************************/



/******************************************************************************
*                European Southern Observatory
*                MIDI Data Reduction Software
*
* Module name:    freeImageQuality
* Input/Output:    See function arguments to avoid duplication
* Description:    Releases all the memory related to the mageQuality data structure
*
* History:        
* 26-Apr-05        (csabet) Created
******************************************************************************/
void freeImageQuality (
    ImageFormat        *format,
    ImageQuality    *image)
{
    //    Local Declarations
    //    ------------------
    int    i;
    
    //    Algorithm
    //    ---------
    for (i = 0; i < format->numOfDetectorRegions; i++)
    {
        free (image->aveImage[i]);
        free (image->aveImageSky[i]);
    }

    free (image->tarType);
    free (image->aveImage);
    free (image->aveImageSky);
    free (image->targetFlux);
    free (image->targetPixelCount);
    free (image->coordX);
    free (image->coordY);
    free (image->sizeX);
    free (image->sizeY);

    free (image);
    image = NULL;
       
    return;
}
/*****************************************************************************/


/******************************************************************************
*                European Southern Observatory
*                MIDI Data Reduction Software
*
* Module name:    callocWavCal
* Input/Output:    See function arguments to avoid duplication
* Description:    Allocates memory for the WavCalibration data structure
*
* History:        
* 10-Jun-05        (csabet) Created
******************************************************************************/
WaveCalibration *callocWaveCal (
    int            numOfFiles,
    int            numOfRecords,
    ImageFormat    *format)
{
    //    Local Declarations
    //    ------------------
    WaveCalibration    *array=NULL;
    int                i;

    //    Algorithm
    //    ---------
    array = (WaveCalibration *) calloc (1, sizeof (WaveCalibration));

    array->numOfFiles = numOfFiles;
    array->tarType = (char *) calloc (format->numOfFrames, sizeof(char));
    array->variance = (float *) calloc (format->numOfDetectorRegions, sizeof (float));
    array->calibratedWave = (float **) calloc (format->numOfDetectorRegions, sizeof (float *));
    for (i = 0; i < format->numOfDetectorRegions; i++)
        array->calibratedWave[i] = (float *) calloc (format->iXWidth, sizeof(float));

    array->filterName = (char **) calloc (numOfFiles, sizeof (char *));
    array->shutterId = (char **) calloc (numOfFiles, sizeof (char *));
    for (i = 0; i < array->numOfFiles; i++)
    {
        array->filterName[i] = (char *) calloc (MIN_STRING_LENGTH, sizeof (char));
        array->shutterId[i] = (char *) calloc (MIN_STRING_LENGTH, sizeof (char));
    }

    //    ArIII memory
    array->ArIII = (NarrowBandInfo *) calloc (1, sizeof (NarrowBandInfo));
    array->ArIII->fileName = (char *) calloc (MAX_STRING_LENGTH, sizeof(char));
    array->ArIII->wavelength = (float *) calloc (format->numOfDetectorRegions, sizeof (float));
    array->ArIII->xCoord = (float *) calloc (format->numOfDetectorRegions, sizeof (float));
    array->ArIII->yCoord = (float *) calloc (format->numOfDetectorRegions, sizeof (float));
    array->ArIII->image = (float **) calloc (format->numOfDetectorRegions, sizeof (float *));
    for (i = 0; i < format->numOfDetectorRegions; i++)
        array->ArIII->image[i] = (float *) calloc (format->iXWidth * format->iYWidth, sizeof(float));

    //    NeII memory
    array->NeII = (NarrowBandInfo *) calloc (1, sizeof (NarrowBandInfo));
    array->NeII->fileName = (char *) calloc (MAX_STRING_LENGTH, sizeof(char));
    array->NeII->wavelength = (float *) calloc (format->numOfDetectorRegions, sizeof (float));
    array->NeII->xCoord = (float *) calloc (format->numOfDetectorRegions, sizeof (float));
    array->NeII->yCoord = (float *) calloc (format->numOfDetectorRegions, sizeof (float));
    array->NeII->image = (float **) calloc (format->numOfDetectorRegions, sizeof (float *));
    for (i = 0; i < format->numOfDetectorRegions; i++)
        array->NeII->image[i] = (float *) calloc (format->iXWidth * format->iYWidth, sizeof(float));

    //    SIV memory
    array->SIV = (NarrowBandInfo *) calloc (1, sizeof (NarrowBandInfo));
    array->SIV->fileName = (char *) calloc (MAX_STRING_LENGTH, sizeof(char));
    array->SIV->wavelength = (float *) calloc (format->numOfDetectorRegions, sizeof (float));
    array->SIV->xCoord = (float *) calloc (format->numOfDetectorRegions, sizeof (float));
    array->SIV->yCoord = (float *) calloc (format->numOfDetectorRegions, sizeof (float));
    array->SIV->image = (float **) calloc (format->numOfDetectorRegions, sizeof (float *));
    for (i = 0; i < format->numOfDetectorRegions; i++)
        array->SIV->image[i] = (float *) calloc (format->iXWidth * format->iYWidth, sizeof(float));

    //    Foil memory
    array->foil = (FoilInfo *) calloc (1, sizeof (FoilInfo));
    array->foil->actualWavelength = (float *) calloc (numOfRecords, sizeof (float));
    array->foil->actualTransmission = (float *) calloc (numOfRecords, sizeof (float));
    array->foil->fileName = (char *) calloc (MAX_STRING_LENGTH, sizeof(char));
    array->foil->image = (float **) calloc (format->numOfDetectorRegions, sizeof (float *));
    for (i = 0; i < format->numOfDetectorRegions; i++)
        array->foil->image[i] = (float *) calloc (format->iXWidth * format->iYWidth, sizeof(float));

    //    OPEN memory
    array->open = (OpenInfo *) calloc (1, sizeof (OpenInfo));
    array->open->fileName = (char *) calloc (MAX_STRING_LENGTH, sizeof(char));
    array->open->image = (float **) calloc (format->numOfDetectorRegions, sizeof (float *));
    for (i = 0; i < format->numOfDetectorRegions; i++)
        array->open->image[i] = (float *) calloc (format->iXWidth * format->iYWidth, sizeof(float));

    //    DARK memory
    array->dark = (DarkInfo *) calloc (1, sizeof (DarkInfo));
    array->dark->fileName = (char *) calloc (MAX_STRING_LENGTH, sizeof(char));
    array->dark->image = (float **) calloc (format->numOfDetectorRegions, sizeof (float *));
    for (i = 0; i < format->numOfDetectorRegions; i++)
        array->dark->image[i] = (float *) calloc (format->iXWidth * format->iYWidth, sizeof(float));
    
    return (array);
}
/*****************************************************************************/


/******************************************************************************
*                European Southern Observatory
*                MIDI Data Reduction Software
*
* Module name:    freeWavCal
* Input/Output:    See function arguments to avoid duplication
* Description:    Releases all the memory related to the WavCalibration data structure
*
* History:        
* 10-Jun-05        (csabet) Created
******************************************************************************/
void freeWaveCal (
    ImageFormat        *format,
    WaveCalibration    *array)
{
    //    Local Declarations
    //    ------------------
    int    i;

    //    Algorithm
    //    ---------
    for (i = 0; i < array->numOfFiles; i++)
    {
        free (array->filterName[i]);
        free (array->shutterId[i]);
    }
    free (array->filterName);
    free (array->shutterId);
    free (array->tarType);
    free (array->variance);

    for (i = 0; i < format->numOfDetectorRegions; i++) free (array->calibratedWave[i]);
    free (array->calibratedWave);

    for (i = 0; i < format->numOfDetectorRegions; i++) free (array->ArIII->image[i]);
    free (array->ArIII->image);
    free (array->ArIII->wavelength);
    free (array->ArIII->xCoord);
    free (array->ArIII->yCoord);
    free (array->ArIII->fileName);
    free (array->ArIII);
    
    for (i = 0; i < format->numOfDetectorRegions; i++) free (array->NeII->image[i]);
    free (array->NeII->image);
    free (array->NeII->wavelength);
    free (array->NeII->xCoord);
    free (array->NeII->yCoord);
    free (array->NeII->fileName);
    free (array->NeII);
    
    for (i = 0; i < format->numOfDetectorRegions; i++) free (array->SIV->image[i]);
    free (array->SIV->image);
    free (array->SIV->wavelength);
    free (array->SIV->xCoord);
    free (array->SIV->yCoord);
    free (array->SIV->fileName);
    free (array->SIV);
    
    for (i = 0; i < format->numOfDetectorRegions; i++) free (array->foil->image[i]);
    free (array->foil->image);
    free (array->foil->fileName);
    free (array->foil->actualWavelength);
    free (array->foil->actualTransmission);
    free (array->foil);
    
    for (i = 0; i < format->numOfDetectorRegions; i++) free (array->open->image[i]);
    free (array->open->image);
    free (array->open->fileName);
    free (array->open);
    
    for (i = 0; i < format->numOfDetectorRegions; i++) free (array->dark->image[i]);
    free (array->dark->image);
    free (array->dark->fileName);
    free (array->dark);
    
    free (array);
    array = NULL;
           
    return;
}
/*****************************************************************************/


