/******************************************************************************
*******************************************************************************
*               European Southern Observatory
*             VLTI MIDI Data Reduction Software
*
* Module name:  midiFitsClassification.c
* Description:  Contains routines for file classification
*
* History:
* 28-Apr-05     (csabet) Created
*******************************************************************************
******************************************************************************/

/******************************************************************************
*   Compiler directives
******************************************************************************/
//  Search for TO_DO_MIDI_DRS
#define DEBUG_FITS_FILES_x

/******************************************************************************
*   Include files
******************************************************************************/
#include <sys/types.h>
#include <sys/stat.h>
#include <unistd.h>
#include <stdio.h>
#include <math.h>
#include "midiGlobal.h"
#include "midiLib.h"
#include "memoryHandling.h"
#include "qfits.h"
#include "qfits_rw.h"
#include "diagnostics.h"
#include "errorHandling.h"
#include "fileHandling.h"
#include "midiFitsClassification.h"
#include "midiGlobal.h"
#include <cpl.h>
/**********************************************************
*   Constant definitions
**********************************************************/


/**********************************************************
*   Global Variables
**********************************************************/

/*============================ C O D E    A R E A ===========================*/



/******************************************************************************
*               European Southern Observatory
*             VLTI MIDI Data Reduction Software
*
* Module name:  classifyFitsFiles
* Input/Output: See function arguments to avoid duplication
* Description:  It loops through the given list of FITS files and puts the
*               keyword END_OF_BATCH at the end of a batch of related files. A batch
*               of files will contain a set of related split Interferometry files
*               as well as the associated photometry files.
*
* History:
* 10-Oct-03     (csabet) Created
******************************************************************************/
void classifyFitsFiles (
    UserOptions       *options,       // In: User options
    MidiFiles         *fileNames,     // IO: Pointer to midi files structure
    FILE              *filePtr,
    int               *error,         // Ou: Error status. 1=Info, 2=Error
    cpl_parameterlist *parlist,
    cpl_frameset      *frameset)

{

    //  Local Declarations
        //      ------------------
    const char  routine[] = "classifyFitsFiles";
    int                 stringLength, fileNumInList, batchNum = 0, fileCount, localError;
    FILE                *inFitsClassifiedPtr=NULL, *inFitsTempPtr=NULL;
    char                *inFitsTemp=NULL, *stringTemp=NULL, 
                                *fileNameStr=NULL, *classification=NULL;
    ImageFormat *format=NULL;

    //  Algorithm
    //  ---------
    if (diagnostic > 4) cpl_msg_info(cpl_func,"Invoking      routine   '%s' \n", routine);
    if (diagnostic > 4) fprintf (filePtr, "Invoking      routine   '%s' \n", routine);

    cpl_msg_info(cpl_func,"\nClassifying input FITS files \n");
    cpl_msg_info(cpl_func,"---------------------------- \n");
    fprintf (filePtr, "\nClassifying input FITS files \n");
    fprintf (filePtr, "---------------------------- \n");

        //      Reset status
        *error = 0;
        localError = 0;
        
        //      Allocate memory
    fileNameStr = (char *) calloc (MAX_STRING_LENGTH, sizeof (char));
    classification = (char *) calloc (MAX_STRING_LENGTH, sizeof (char));
    inFitsTemp = (char *) calloc (MAX_STRING_LENGTH, sizeof (char));
    stringTemp = (char *) calloc (MAX_STRING_LENGTH, sizeof (char));
        format = callocImageFormat ();

    //  Open the list of files for reading
    if ((inFitsClassifiedPtr = fopen (fileNames->inFitsClassified, "r")) == NULL)
    {
        midiReportWarning (filePtr, routine, __FILE__, __LINE__, "Cannot open input FITS file list");
                *error = 2;
                free (stringTemp);
                free (inFitsTemp);
                free (classification);
                free (fileNameStr);
                freeImageFormat (format);
                return;
        }

#ifdef DEBUG_FITS_FILES
    fileNumInList = 0;
    while (fgets (stringTemp, MAX_STRING_LENGTH, inFitsClassifiedPtr) != NULL)
    {
        fileNumInList++;
        sscanf (stringTemp, "%s%s", fileNameStr, classification);
        getFitsClassification (fileNumInList, filePtr, fileNameStr, format, error);
    }
    fclose (filePtr);
    exit (0);
#endif

    //  Open a main temporary file for writing
    stringLength = strlen (fileNames->inFitsClassified);
    strncpy (inFitsTemp, fileNames->inFitsClassified, stringLength-4);
    strcat (inFitsTemp, "Buffer.log");
    if ((inFitsTempPtr = fopen (inFitsTemp, "w")) == NULL)
    {
        midiReportWarning (filePtr, routine, __FILE__, __LINE__, "Cannot open a temporary batch file");
                *error = 2;
                fclose (inFitsClassifiedPtr);
                free (stringTemp);
                free (inFitsTemp);
                free (classification);
                free (fileNameStr);
                freeImageFormat (format);
                return;
        }

    //  Loop through the list of FITS files and classify
    fileNumInList = 0;
    while (fgets (stringTemp, MAX_STRING_LENGTH, inFitsClassifiedPtr) != NULL)
    {
                //      Extract name and classification of the input FITS file
        sprintf (classification, "%s", "");
        sscanf (stringTemp, "%s%s", fileNameStr, classification);

        //      Increment file number
        fileNumInList++;

        //      Get FITS file parameters
        getFitsClassification (fileNumInList, filePtr, fileNameStr, format, &localError);
                if (localError) continue;

                //      Check template of the subsequent first file
                if (((strcmp (format->obsCatg, "CALIB") == 0) || 
                        (strcmp (format->obsCatg, "SCIENCE") == 0)) &&
                        ((strcmp (format->obsTech, "INTERFEROMETRY") == 0) || 
                        (strcmp (format->obsTech, "IMAGE,WINDOW,CHOPNOD") == 0)) &&
                        ((strcmp (format->obsType, "TRACK,OBJECT,DISPERSED") == 0) || 
                        (strcmp (format->obsType, "TRACK,OBJECT,DISPERSED,SCIPHOT") == 0) || 

                        (strcmp (format->obsType, "FRINGE_TRACK,OBJECT,FOURIER") == 0) ||
                        (strcmp (format->obsType, "FRINGE_TRACK,OBJECT,FOURIER,SCI_PHOT") == 0) || 

                        (strcmp (format->obsType, "PHOTOMETRY,OBJECT,SCIPHOT") == 0) ||
                        (strcmp (format->obsType, "PHOTOMETRY,OBJECT") == 0))  &&
                        (format->fileNumInTemplate == 1))
                {
                        if (options->drsFrg) getFringeTrackBatch (format, inFitsTempPtr, inFitsClassifiedPtr, 
                                fileNameStr, classification, filePtr, &batchNum, &fileNumInList, error, parlist, frameset);
                }
                else if (((strcmp (format->obsCatg, "CALIB") == 0) || 
                        (strcmp (format->obsCatg, "SCIENCE") == 0)) &&
                        ((strcmp (format->obsTech, "IMAGE,WINDOW") == 0) || 
                        (strcmp (format->obsType, "COARSE,OBJECT") == 0)))
                {
                        if (options->drsAcq) getAcquisitionBatch (inFitsTempPtr, fileNameStr, classification, 
                                filePtr, &batchNum, error);
                }
                else if ((strcmp (format->obsCatg, "CALIB") == 0) &&
                        ((strcmp (format->obsTech, "IMAGE") == 0) || 
                        (strcmp (format->obsTech, "SPECTRUM") == 0)) &&
                        (strcmp (format->obsType, "FLAT") == 0) &&
                        (format->fileNumInTemplate == 1))
                {
                        if (options->tecLin) getDetLinBatch (format, inFitsTempPtr, inFitsClassifiedPtr, fileNameStr,
                                classification, filePtr, &batchNum, &fileNumInList, error);
                }
                else if ((strcmp (format->obsCatg, "CALIB") == 0) &&
                        (strcmp (format->obsTech, "IMAGE") == 0) &&
                        (strcmp (format->obsType, "BIAS") == 0) &&
                        (format->fileNumInTemplate == 1))
                {
                        if (options->tecRon) getDetRonBatch (format, inFitsTempPtr, inFitsClassifiedPtr, fileNameStr,
                                classification, filePtr, &batchNum, &fileNumInList, error);
                }
                else if ((strcmp (format->obsCatg, "CALIB") == 0) &&
                        (strcmp (format->obsTech, "SPECTRUM") == 0) &&
                        (strcmp (format->obsType, "WAVE") == 0) &&
                        (format->fileNumInTemplate == 1))
                {
                        if (options->tecTrn) getDspTrnBatch (format, inFitsTempPtr, inFitsClassifiedPtr, fileNameStr,
                                classification, filePtr, &batchNum, &fileNumInList, error);
                }
                else if ((strcmp (format->obsCatg, "CALIB") == 0) &&
                        (strcmp (format->obsTech, "IMAGE") == 0) &&
                        (strcmp (format->obsType, "FMTCHCK") == 0) &&
                        (format->fileNumInTemplate == 1))
                {
                        if (options->tecPix) getRefPixBatch (format, inFitsTempPtr, inFitsClassifiedPtr, fileNameStr,
                                classification, filePtr, &batchNum, &fileNumInList, error);
                }
                else if ((strcmp (format->obsCatg, "CALIB") == 0) &&
                        (strcmp (format->obsTech, "SPECTRUM") == 0) &&
                        (strcmp (format->obsType, "WAVE,SPECTEMPL") == 0) &&
                        (format->fileNumInTemplate == 1))
                {
                        if (options->tecWav) getWavCalBatch (format, inFitsTempPtr, inFitsClassifiedPtr, fileNameStr,
                                classification, filePtr, &batchNum, &fileNumInList, error);
                }
                else
                {       
                continue;
                }
        }

    //  Copy the content of the main temporary file into the main list
    if (inFitsTempPtr) fclose (inFitsTempPtr);
    if (inFitsClassifiedPtr) 
    {
                fclose (inFitsClassifiedPtr);
                inFitsClassifiedPtr = NULL;
        }
    if ((inFitsTempPtr = fopen (inFitsTemp, "r")) == NULL)
    {
        midiReportWarning (filePtr, routine, __FILE__, __LINE__, "Cannot open input FITS file list for reading");
        *error = 2;
        }
        else
        {
                //      Check if the file is empty
                if (fscanf (inFitsTempPtr, "%s\n", stringTemp) == EOF)
            {
                        if (options->drsFrg || options->drsAcq || options->tecRon || options->tecLin || 
                                options->tecTrn || options->tecPix || options->tecWav)
                        {
                                *error = 2;
                                sprintf (midiMessage, "%s\n\t     %s\n\t\t     %s\n\t\t     %s", 
                                        "Cannot find a complete set of FITS files in the given list or directory.",
                                        "Check the following log files:", fileNames->inFitsClassified, inFitsTemp);
                        midiReportWarning (filePtr, routine, __FILE__, __LINE__, midiMessage);
                        }
                        else
                        {
                                *error = 1;
                                midiReportInfo (midiReportPtr, routine, __FILE__, __LINE__, "No Batch Classification was requested");
                        }
                }
                else
                {
                        rewind (inFitsTempPtr);
                        if ((inFitsClassifiedPtr = fopen (fileNames->inFitsClassified, "w")) == NULL)
                        {
                                *error = 2;
                                midiReportWarning (filePtr, routine, __FILE__, __LINE__, "Cannot open input FITS file list for writing");
                        }
                        else
                        {
                                fileCount = 0;
                                while (fgets (stringTemp, MAX_STRING_LENGTH, inFitsTempPtr) != NULL)
                                {
                                        //      Get template name first
                                        fileCount++;
                                        if (fileCount == 1)
                                        {
                                                fprintf (inFitsClassifiedPtr, "%s", stringTemp);
                                                continue;
                                        }
                                        if (strcmp (stringTemp, "END_OF_BATCH\n") == 0)
                                                fileCount = 0;
                                        sprintf (classification, "%s", "");
                                        sscanf (stringTemp, "%s%s", fileNameStr, classification);
                                        fprintf (inFitsClassifiedPtr, "%s %s\n", fileNameStr, classification);
                                }
                        }
                }
        }

    //  Close all files
    if (inFitsClassifiedPtr) fclose (inFitsClassifiedPtr);
    if (inFitsTempPtr) fclose (inFitsTempPtr);
    if (*error < 2) remove (inFitsTemp);

    //  Release memory
    if (stringTemp) free (stringTemp);
    if (inFitsTemp) free (inFitsTemp);
    if (classification) free (classification);
    if (fileNameStr) free (fileNameStr);
        if (format) freeImageFormat (format);

    return;
}
/******************************************************************************/


/******************************************************************************
*               European Southern Observatory
*            VLTI MIDI Data Reduction Software
*
* Module name:  getFitsClassification
* Input/Output: See function arguments to avoid duplication
* Description:  A relatively quicker routine to get the fits file classification
*
* History:
* 08-Mar-05     (csabet) Created
******************************************************************************/
void getFitsClassification (
    int                 fileNum,        // In: File counter
    FILE                *reportPtr,     // In: Pointer to a temporary report file
    char                *fileName,      // In: Name of the input FITS file
    ImageFormat *format,        // Ou: Classification
    int                 *error)         // Ou: Error status

{

    /*  Local Declarations
    --------------------*/
    const char  routine[] = "getFitsClassification";
    char                *stringQfits, *emptyString, *cleanString;
    int                 unknownNumber;
    struct stat buf;

    /*  Algorithm
    -----------*/
    if (diagnostic > 4) cpl_msg_info(cpl_func,"Invoking      routine   '%s' \n", routine);
    if (diagnostic > 4) fprintf (reportPtr, "Invoking      routine   '%s' \n", routine);

        //      Reset status
    *error = 0;
    unknownNumber = - 1;

    //  Check if the file exist
    if (stat (fileName, &buf) == -1)
    {
                sprintf (midiMessage, "Cannot find file ... %s", fileName);
        midiReportWarning (reportPtr, routine, __FILE__, __LINE__, midiMessage);
        *error = 1;
        return;
    }
    else if ((qfits_is_fits (fileName)) != 1)  // Check if this is a FITS file
    {
                sprintf (midiMessage, "Not a FITS file ... %s", fileName);
        midiReportWarning (reportPtr, routine, __FILE__, __LINE__, midiMessage);
        *error = 1;
        return;
    }

        //      Allocate memory
    cleanString = (char *) calloc (MAX_STRING_LENGTH, sizeof (char));
    emptyString = (char *) calloc (MAX_STRING_LENGTH, sizeof (char));
    strcpy (emptyString, "UNKNOWN");

        //      Get data Observation Category
        stringQfits = qfits_query_hdr (fileName, "HIERARCH ESO DPR CATG");
        if (stringQfits == NULL) sprintf (format->obsCatg, "%s", emptyString);
        else
        {
                cleanUpString (stringQfits, cleanString);
                sprintf (format->obsCatg, "%s", cleanString);
        }

        //      Get data Observation Technique
        stringQfits = qfits_query_hdr (fileName, "HIERARCH ESO DPR TECH");
        if (stringQfits == NULL) sprintf (format->obsTech, "%s", emptyString);
        else
        {
                cleanUpString (stringQfits, cleanString);
                sprintf (format->obsTech, "%s", cleanString);
        }

        //      Get data Observation Type
        stringQfits = qfits_query_hdr (fileName, "HIERARCH ESO DPR TYPE");
        if (stringQfits == NULL) sprintf (format->obsType, "%s", emptyString);
        else
        {
                cleanUpString (stringQfits, cleanString);
                sprintf (format->obsType, "%s", cleanString);
        }

        //      Get target name
        stringQfits = qfits_query_hdr (fileName, "HIERARCH ESO OBS TARG NAME");
        if (stringQfits == NULL) sprintf (format->targetName, "%s", emptyString);
        else
        {
                cleanUpString (stringQfits, cleanString);
                sprintf (format->targetName, "%s", cleanString);
        }

    //  Get TPL Start
    stringQfits = qfits_query_hdr (fileName, "HIERARCH ESO TPL START");
    if (stringQfits == NULL) sprintf (format->tplStart, "%s", emptyString);
    else
    {
        cleanUpString (stringQfits, cleanString);
                sprintf (format->tplStart, "%s", cleanString);
    }

    //  Get number of templates
    stringQfits = qfits_query_hdr (fileName, "HIERARCH ESO OBS TPLNO");
    if (stringQfits == NULL) format->numOfTemplates = unknownNumber;
    else sscanf (stringQfits, "%d", &(format->numOfTemplates));

    //  Get template's number of files
    stringQfits = qfits_query_hdr (fileName, "HIERARCH ESO OCS TPL NFILE");
    if (stringQfits == NULL) format->numOfFilesInTemplate = unknownNumber;
    else sscanf (stringQfits, "%d", &(format->numOfFilesInTemplate));

    //  Get template's sequential file number
    stringQfits = qfits_query_hdr (fileName, "HIERARCH ESO OCS TPL FILENO");
    if (stringQfits == NULL) format->fileNumInTemplate = unknownNumber;
    else sscanf (stringQfits, "%d", &(format->fileNumInTemplate));

    //  Get template's Exposure number
    stringQfits = qfits_query_hdr (fileName, "HIERARCH ESO TPL EXPNO");
    if (stringQfits == NULL) format->exposureNumInTemplate = unknownNumber;
    else sscanf (stringQfits, "%d", &(format->exposureNumInTemplate));

    //  Get template's number of exposures
    stringQfits = qfits_query_hdr (fileName, "HIERARCH ESO TPL NEXP");
    if (stringQfits == NULL) format->numOfExposures = unknownNumber;
    else sscanf (stringQfits, "%d", &(format->numOfExposures));

    //  Get template's file number within exposures
    stringQfits = qfits_query_hdr (fileName, "HIERARCH ESO OCS EXPO FILENO");
    if (stringQfits == NULL) format->fileNumInExposure = unknownNumber;
    else sscanf (stringQfits, "%d", &(format->fileNumInExposure));

        //      Get template name
    stringQfits = qfits_query_hdr (fileName, "HIERARCH ESO TPL NAME");
        if (stringQfits == NULL) sprintf (format->tplName, "%s", emptyString);
        else
        {
                cleanUpString (stringQfits, cleanString);
                sprintf (format->tplName, "%s", cleanString);
        }
 
        //      Get beam combiner
        stringQfits = qfits_query_hdr (fileName, "HIERARCH ESO INS OPT1 ID");
        if (stringQfits == NULL) sprintf (format->beamCombiner, "%s", emptyString);
        else
        {
                cleanUpString (stringQfits, cleanString);
                sprintf (format->beamCombiner, "%s", cleanString);
        }

        //      Get Shutter ID
        stringQfits = qfits_query_ext (fileName, "HIERARCH ESO INS SHUT ID", 0);
        if (stringQfits == NULL) sprintf (format->shutterId, "%s", emptyString);
        else
        {
                cleanUpString (stringQfits, cleanString);
                sprintf (format->shutterId, "%s", cleanString);
        }

    if (diagnostic)
    {
                cpl_msg_info(cpl_func,"\nfile: %4d\n", fileNum);
                cpl_msg_info(cpl_func,"fileName: %s\n", fileName);
                cpl_msg_info(cpl_func,"tplName                (TPL NAME): %s\n", format->tplName);
                cpl_msg_info(cpl_func,"tplStart              (TPL START): %s\n", format->tplStart);
                if (format->fileNumInTemplate == -1) 
                        cpl_msg_info(cpl_func,"fileNumInTemplate    (TPL FILENO): UNKNOWN\n");
                else
                        cpl_msg_info(cpl_func,"fileNumInTemplate    (TPL FILENO): %d\n", format->fileNumInTemplate);
                
                if (format->numOfFilesInTemplate == -1)
                        cpl_msg_info(cpl_func,"numOfFilesInTemplate  (TPL NFILE): UNKNOWN\n");
                else
                        cpl_msg_info(cpl_func,"numOfFilesInTemplate  (TPL NFILE): %d\n", format->numOfFilesInTemplate);
                
                if (format->numOfTemplates)
                        cpl_msg_info(cpl_func,"numOfTemplates        (OBS TPLNO): UNKNOWN\n");
                else
                        cpl_msg_info(cpl_func,"numOfTemplates        (OBS TPLNO): %d\n", format->numOfTemplates);
                
                if (format->numOfExposures)
                        cpl_msg_info(cpl_func,"numOfExposures         (TPL NEXP): UNKNOWN\n");
                else
                        cpl_msg_info(cpl_func,"numOfExposures         (TPL NEXP): %d\n", format->numOfExposures);
                
                if (format->fileNumInExposure)
                        cpl_msg_info(cpl_func,"fileNumInExposure   (EXPO FILENO): UNKNOWN\n");
                else
                        cpl_msg_info(cpl_func,"fileNumInExposure   (EXPO FILENO): %d\n", format->fileNumInExposure);

                if (format->exposureNumInTemplate)
                        cpl_msg_info(cpl_func,"exposureNumInTemplate (TPL EXPNO): UNKNOWN\n");
                else
                        cpl_msg_info(cpl_func,"exposureNumInTemplate (TPL EXPNO): %d\n", format->exposureNumInTemplate);
                cpl_msg_info(cpl_func,"targetName            (TARG NAME): %s\n", format->targetName);
                cpl_msg_info(cpl_func,"beamCombiner        (INS OPT1 ID): %s\n", format->beamCombiner);
                cpl_msg_info(cpl_func,"shutterId           (INS SHUT ID): %s\n", format->shutterId);
                cpl_msg_info(cpl_func,"observationCategory    (DPR CATG): %s\n", format->obsCatg);
                cpl_msg_info(cpl_func,"observationType        (DPR TYPE): %s\n", format->obsType);
                cpl_msg_info(cpl_func,"observationTechnique   (DPR TECH): %s\n", format->obsTech);
    }

        fprintf (reportPtr, "\nfile: %4d\n", fileNum);
        fprintf (reportPtr, "fileName: %s\n", fileName);
        fprintf (reportPtr, "tplName                (TPL NAME): %s\n", format->tplName);
        fprintf (reportPtr, "tplStart              (TPL START): %s\n", format->tplStart);
        if (format->fileNumInTemplate == -1) 
                fprintf (reportPtr, "fileNumInTemplate    (TPL FILENO): UNKNOWN\n");
        else
                fprintf (reportPtr, "fileNumInTemplate    (TPL FILENO): %d\n", format->fileNumInTemplate);
        
        if (format->numOfFilesInTemplate == -1)
                fprintf (reportPtr, "numOfFilesInTemplate  (TPL NFILE): UNKNOWN\n");
        else
                fprintf (reportPtr, "numOfFilesInTemplate  (TPL NFILE): %d\n", format->numOfFilesInTemplate);
                
        if (format->numOfTemplates)
                fprintf (reportPtr, "numOfTemplates        (OBS TPLNO): UNKNOWN\n");
        else
                fprintf (reportPtr, "numOfTemplates        (OBS TPLNO): %d\n", format->numOfTemplates);
        
        if (format->numOfExposures)
                fprintf (reportPtr, "numOfExposures         (TPL NEXP): UNKNOWN\n");
        else
                fprintf (reportPtr, "numOfExposures         (TPL NEXP): %d\n", format->numOfExposures);
                
        if (format->fileNumInExposure)
                fprintf (reportPtr, "fileNumInExposure   (EXPO FILENO): UNKNOWN\n");
        else
                fprintf (reportPtr, "fileNumInExposure   (EXPO FILENO): %d\n", format->fileNumInExposure);

        if (format->exposureNumInTemplate)
                fprintf (reportPtr, "exposureNumInTemplate (TPL EXPNO): UNKNOWN\n");
        else
                fprintf (reportPtr, "exposureNumInTemplate (TPL EXPNO): %d\n", format->exposureNumInTemplate);
        fprintf (reportPtr, "targetName            (TARG NAME): %s\n", format->targetName);
        fprintf (reportPtr, "beamCombiner        (INS OPT1 ID): %s\n", format->beamCombiner);
        fprintf (reportPtr, "shutterId           (INS SHUT ID): %s\n", format->shutterId);
        fprintf (reportPtr, "observationCategory    (DPR CATG): %s\n", format->obsCatg);
        fprintf (reportPtr, "observationType        (DPR TYPE): %s\n", format->obsType);
        fprintf (reportPtr, "observationTechnique   (DPR TECH): %s\n", format->obsTech);

    /*  Release memory */
    free (emptyString);
    free (cleanString);

    return;
}
/*****************************************************************************/


/******************************************************************************
*               European Southern Observatory
*            VLTI MIDI Data Reduction Software
*
* Module name:  getFringeTrackBatch
* Input/Output: See function arguments to avoid duplication
* Description:  We know the first file in the given list is a TRACK. We
*                               check to see if we can find a complete batch. If not the file
*                               pointer is returned unchanged.
*
* History:
* 28-Apr-05             (csabet) Created
******************************************************************************/
void getFringeTrackBatch (
    ImageFormat *format,                // In: Format structure
    FILE              *inFitsTempPtr, // In: Pointer to the main buffer
    FILE              *inFitsListPtr, // In: Pointer to the list of FITS files
    char              *fileNameStr,   // In: Name of the first FITS file
    char              *dummyClassif,  // In: Dummy classification
    FILE              *filePtr,               // In: Pointer to the log file
    int               *localBatchNum, // IO: Local batch number
    int               *fileNumInList, // IO: pointer to file number
    int               *error,                 // Ou: Error status
	cpl_parameterlist *parlist,
	cpl_frameset      *frameset)

{

    //  Local Declarations
    //  ------------------
    const char  routine[] = "getFringeTrackBatch";
        char            *inFitsBatchTemp=NULL, *localTplStart=NULL, *stringTemp=NULL, *localTplName=NULL,
                                *localBeamCombiner, *localObsCatg, *localObsTech, *localObsType;
        FILE            *inFitsBatchTempPtr=NULL;
        int                     i, numOfInterfFiles, numOfPhotAFiles, numOfPhotBFiles, fileNumInTemplate, numOfEmptyFiles,
                                localFileNumInList, numOfFiles;
        long            inFitsListPosition;
    	int    checkSof=0;
    	cpl_parameter * cur_param;
    //  Algorithm
    //  ---------
    if (diagnostic > 4) cpl_msg_info(cpl_func,"Invoking      routine   '%s' \n", routine);
    if (diagnostic > 4) fprintf(filePtr, "Invoking      routine   '%s' \n", routine);

        //      Reset status
        *error = 0;
        numOfFiles = 0;
        numOfInterfFiles = 0;
        numOfPhotAFiles = 0;
        numOfPhotBFiles = 0;
        numOfEmptyFiles = 0;

        /*Get the recipe parameter*/
    	cur_param=cpl_parameterlist_find(parlist,
    			"midi.midi_fringe_all.checkSof");
    	checkSof=cpl_parameter_get_bool(cur_param);

        //      Allocate memory
    inFitsBatchTemp = (char *) calloc (MAX_STRING_LENGTH, sizeof (char));
    stringTemp = (char *) calloc (MAX_STRING_LENGTH, sizeof (char));
    localTplStart = (char *) calloc (MAX_STRING_LENGTH, sizeof (char));
    localTplName = (char *) calloc (MAX_STRING_LENGTH, sizeof (char));
    localBeamCombiner = (char *) calloc (MAX_STRING_LENGTH, sizeof (char));
    localObsCatg = (char *) calloc (MAX_STRING_LENGTH, sizeof (char));
    localObsTech = (char *) calloc (MAX_STRING_LENGTH, sizeof (char));
    localObsType = (char *) calloc (MAX_STRING_LENGTH, sizeof (char));

        //      Open a batch temporary file for writing
        sprintf (inFitsBatchTemp, "localBuffer.log");
        if ((inFitsBatchTempPtr = fopen (inFitsBatchTemp, "w")) == NULL)
        {
                midiReportWarning (filePtr, routine, __FILE__, __LINE__, "Cannot open a batch temporary file");
                *error = 1;
                free (inFitsBatchTemp);
                free (localTplStart);
                free (localTplName);
                free (stringTemp);
                free (localBeamCombiner);
                free (localObsCatg);
                free (localObsTech);
                free (localObsType);
                return;
        }
                                
        //      Save the first file and its parameters
        fprintf (inFitsBatchTempPtr, "%s %s\n", fileNameStr, dummyClassif);
        inFitsListPosition = ftell (inFitsListPtr);
        localFileNumInList = *fileNumInList;

        if (strcmp (format->obsTech, "INTERFEROMETRY") == 0) 
                numOfInterfFiles++;
        else if (strcmp (format->shutterId, "AOPEN") == 0) 
                numOfPhotAFiles++;
        else if (strcmp (format->shutterId, "BOPEN") == 0) 
                numOfPhotBFiles++;

        fileNumInTemplate = format->fileNumInTemplate;

        sprintf (localTplStart, "%s", format->tplStart);
        sprintf (localTplName, "%s", format->tplName);
        sprintf (localBeamCombiner, "%s", format->beamCombiner);
        sprintf (localObsCatg, "%s", format->obsCatg);
        sprintf (localObsTech, "%s", format->obsTech);
        sprintf (localObsType, "%s", format->obsType);

        sprintf (batchTemplate, "%s", format->tplName);

        //      Get the rest of the batch files from the list
        while (fgets (stringTemp, MAX_STRING_LENGTH, inFitsListPtr) != NULL)
        {
                sprintf (dummyClassif, "%s", "");
                sscanf (stringTemp, "%s%s", fileNameStr, dummyClassif);

                localFileNumInList++;
                fileNumInTemplate++;

                //      Get FITS file parameters. If ok copy into the batch temporary file
                getFitsClassification (localFileNumInList, filePtr, fileNameStr, format, error);

                      /*Check template name consistency. For the
                       * calibrator database the tpl Name is "UNKNOWN"*/
                //if (strcmp (format->tplName, localTplName) != 0) *error = 1;
                if (strcmp (format->tplName, localTplName) != 0 && strcmp (format->tplName, "UNKNOWN") != 0 )
                {
                	*error = 1;
                }

                /*Checking for TPL START and OCS TPL FILENO (de)activating*/
                if(checkSof==FALSE)
                {
                	cpl_msg_warning(cpl_func,"Overruling TPL START homogeneity check");
                	cpl_msg_warning(cpl_func,"Overruling OCS TPL FILENO homogeneity check");
                	format->fileNumInTemplate=fileNumInTemplate;
                	sprintf (localTplStart, "%s", format->tplStart);
                }

                //      We expect possibly more split interferometry and photometry files and an empty file
                if (!(*error) && ((strcmp (format->obsCatg, "CALIB") == 0) || 
                        (strcmp (format->obsCatg, "SCIENCE") == 0)) &&
                        (strcmp (format->obsTech, "INTERFEROMETRY") == 0) && 
                        ((strcmp (format->obsType, "TRACK,OBJECT,DISPERSED") == 0) || 
                        (strcmp (format->obsType, "TRACK,OBJECT,DISPERSED,SCIPHOT") == 0) ||

                        (strcmp (format->obsType, "FRINGE_TRACK,OBJECT,FOURIER") == 0) ||
                        (strcmp (format->obsType, "FRINGE_TRACK,OBJECT,FOURIER,SCI_PHOT") == 0) || 
 
                        (strcmp (format->obsType, "TRACK,OBJECT,DISPERSED") == 0)) &&
                        (strcmp (format->tplStart, localTplStart) == 0) && 
                        (format->fileNumInTemplate == fileNumInTemplate))
                {
                        fprintf (inFitsBatchTempPtr, "%s %s\n", fileNameStr, dummyClassif);
                        numOfInterfFiles++;
                }
                else if (!(*error) && ((strcmp (format->obsCatg, "CALIB") == 0) || 
                        (strcmp (format->obsCatg, "SCIENCE") == 0)) &&
                        (strcmp (format->obsTech, "IMAGE,WINDOW,CHOPNOD") == 0) &&
                        ((strcmp (format->obsType, "PHOTOMETRY,OBJECT") == 0)  ||
                        (strcmp (format->obsType, "PHOTOMETRY,OBJECT,SCIPHOT") == 0)) &&
                        (strcmp (format->tplStart, localTplStart) == 0) && 
                        (strcmp (format->shutterId, "AOPEN") == 0) &&
                        (format->fileNumInTemplate == fileNumInTemplate))
                {
                        fprintf (inFitsBatchTempPtr, "%s %s\n", fileNameStr, dummyClassif);
                        numOfPhotAFiles++;
                }
                else if (!(*error) && ((strcmp (format->obsCatg, "CALIB") == 0) || 
                        (strcmp (format->obsCatg, "SCIENCE") == 0)) &&
                        (strcmp (format->obsTech, "IMAGE,WINDOW,CHOPNOD") == 0) &&
                        ((strcmp (format->obsType, "PHOTOMETRY,OBJECT") == 0)  ||
                        (strcmp (format->obsType, "PHOTOMETRY,OBJECT,SCIPHOT") == 0)) &&
                        (strcmp (format->tplStart, localTplStart) == 0) && 
                        (strcmp (format->shutterId, "BOPEN") == 0) &&
                        (format->fileNumInTemplate == fileNumInTemplate))
                {
                        fprintf (inFitsBatchTempPtr, "%s %s\n", fileNameStr, dummyClassif);
                        numOfPhotBFiles++;
                }
                else if (!(*error) && ((strcmp (format->obsCatg, "CALIB") == 0) || 
                        (strcmp (format->obsCatg, "SCIENCE") == 0)) &&
                        (strcmp (format->obsTech, "OTHER") == 0) && 
                        (strcmp (format->obsType, "OTHER") == 0) &&
                        (strcmp (format->tplStart, localTplStart) == 0) &&
                        (format->fileNumInTemplate == fileNumInTemplate) && 
                        (numOfEmptyFiles < 1))
                {
                        fprintf (inFitsBatchTempPtr, "%s %s\n", fileNameStr, dummyClassif);
                        numOfEmptyFiles++;
                        break;
                }
                else
                {
                        //      We have not found a complete set. Hence reset the file position
                        fseek (inFitsListPtr, inFitsListPosition, SEEK_SET);
                        localFileNumInList--;
                        break;
                }
                inFitsListPosition = ftell (inFitsListPtr);
        }
        *fileNumInList = localFileNumInList;
        
        //      If END_OF_TEMPLATE is deactivated then we do not check the numOfEmptyFiles
        if (!endOfTemplate) numOfEmptyFiles = 1;
        
        /*      Now finalise batchTemplate name. If batchTemplate 
        is valid then copy all files into the main temporary file */
        if (((strcmp (localTplName, "MIDI STARINTF OBS FRINGE") == 0) ||
                (strcmp (localTplName, "MIDI STARINTF OBS FRINGE CAL") == 0)) &&
                (strcmp (localBeamCombiner, "HIGH_SENS") == 0) && 
                (strcmp (localObsCatg, "CALIB") == 0) &&
                (numOfInterfFiles > 0) && (numOfPhotAFiles > 0) && (numOfPhotBFiles > 0) && 
                (numOfEmptyFiles == 1) && (numOfPhotAFiles == numOfPhotBFiles))
                sprintf (batchTemplate, "%s", "HIGH_SENS_CALIB");
        else if (((strcmp (localTplName, "MIDI STARINTF OBS FRINGE") == 0) ||
                (strcmp (localTplName, "MIDI STARINTF OBS FRINGE SCI") == 0)) &&
                (strcmp (localBeamCombiner, "HIGH_SENS") == 0) && 
                (strcmp (localObsCatg, "SCIENCE") == 0) && 
                (numOfInterfFiles > 0) && (numOfPhotAFiles > 0) && (numOfPhotBFiles > 0) && 
                (numOfEmptyFiles == 1) && (numOfPhotAFiles == numOfPhotBFiles))
                sprintf (batchTemplate, "%s", "HIGH_SENS_SCIENCE");
        else if (((strcmp (localTplName, "MIDI STARINTF OBS FRINGE") == 0) ||
                (strcmp (localTplName, "MIDI STARINTF OBS FRINGE CAL") == 0)) &&
                (strcmp (localBeamCombiner, "SCI_PHOT") == 0) && 
                (strcmp (localObsCatg, "CALIB") == 0) && 
                (numOfInterfFiles > 0) && (numOfPhotAFiles+numOfPhotBFiles == 0) && (numOfEmptyFiles == 1))
                sprintf (batchTemplate, "%s", "SCI_PHOT_CALIB");
        else if (((strcmp (localTplName, "MIDI STARINTF OBS FRINGE") == 0) ||
                (strcmp (localTplName, "MIDI STARINTF OBS FRINGE SCI") == 0)) &&
                (strcmp (localBeamCombiner, "SCI_PHOT") == 0) && 
                (strcmp (localObsCatg, "SCIENCE") == 0) && 
                (numOfInterfFiles > 0) && (numOfPhotAFiles+numOfPhotBFiles == 0) && (numOfEmptyFiles == 1))
                sprintf (batchTemplate, "%s", "SCI_PHOT_SCIENCE");
        else if (((strcmp (localTplName, "MIDI STARINTF OBS FRINGE") == 0) ||
                (strcmp (localTplName, "MIDI STARINTF OBS FRINGE CAL") == 0)) &&
                (strcmp (localBeamCombiner, "SCI_PHOT") == 0) && 
                (numOfInterfFiles == 0) && (numOfPhotAFiles > 0) && (numOfPhotBFiles > 0) && 
                (numOfEmptyFiles == 1) && (numOfPhotAFiles == numOfPhotBFiles))
                sprintf (batchTemplate, "%s", "SCI_PHOT_KAPPA");
        else if (((strcmp (localTplName, "MIDI STARINTF OBS FRINGE") == 0) ||
                (strcmp (localTplName, "MIDI STARINTF OBS FRINGE CAL") == 0)) &&
                (strcmp (localBeamCombiner, "SCI_PHOT") == 0) && 
                (numOfInterfFiles > 0) && (numOfPhotAFiles > 0) && (numOfPhotBFiles > 0) && 
                (numOfEmptyFiles == 1) && (numOfPhotAFiles == numOfPhotBFiles))
        {
                numOfFiles = numOfInterfFiles + numOfPhotAFiles + numOfPhotBFiles + numOfEmptyFiles;
                sprintf (batchTemplate, "%s", "SCI_PHOT_KAPPA");
        }
        else
        {
                if (diagnostic > 1)
                {
                        midiReportWarning (filePtr, routine, __FILE__, __LINE__, "Unknown Template Configuration:");
                        cpl_msg_info(cpl_func,"     Template Name            = %s \n", localTplName);
                        cpl_msg_info(cpl_func,"     Beam Combiner            = %s \n", localBeamCombiner);
                        cpl_msg_info(cpl_func,"     Obs. Category            = %s \n", localObsCatg);
                        cpl_msg_info(cpl_func,"     Number of Interf files   = %d \n", numOfInterfFiles);
                        cpl_msg_info(cpl_func,"     Number of Photom A files = %d \n", numOfPhotAFiles);
                        cpl_msg_info(cpl_func,"     Number of Photom B files = %d \n", numOfPhotBFiles);
                        fprintf (filePtr, "     Template Name            = %s \n", localTplName);
                        fprintf (filePtr, "     Beam Combiner            = %s \n", localBeamCombiner);
                        fprintf (filePtr, "     Obs. Category            = %s \n", localObsCatg);
                        fprintf (filePtr, "     Number of Interf files   = %d \n", numOfInterfFiles);
                        fprintf (filePtr, "     Number of Photom A files = %d \n", numOfPhotAFiles);
                        fprintf (filePtr, "     Number of Photom B files = %d \n", numOfPhotBFiles);
                }
                free (inFitsBatchTemp);
                free (localTplStart);
                free (localTplName);
                free (stringTemp);
                free (localBeamCombiner);
                free (localObsCatg);
                free (localObsTech);
                free (localObsType);
                if (inFitsBatchTempPtr)
                {
                        fclose (inFitsBatchTempPtr);
                        inFitsBatchTempPtr = NULL;
                }
                return;
        }
        
        //      Reopen the batch temporary file for reading
        fclose (inFitsBatchTempPtr);
        inFitsBatchTempPtr = NULL;
        if ((inFitsBatchTempPtr = fopen (inFitsBatchTemp, "r")) == NULL)
        {
                midiReportWarning (filePtr, routine, __FILE__, __LINE__, "Cannot open a batch temporary file");
                *error = 1;
                free (inFitsBatchTemp);
                free (localTplStart);
                free (localTplName);
                free (stringTemp);
                free (localBeamCombiner);
                free (localObsCatg);
                free (localObsTech);
                free (localObsType);
                return;
        }

        //      If it is a SCI_PHOT_KAPPA batch with interferometry files, it must be split into two batches
        if ((strcmp (batchTemplate, "SCI_PHOT_KAPPA") == 0) && (numOfInterfFiles > 0))
        {
                //      First extract the Photometry files for kappa calculation, only if it is a CALIB template
                if (strcmp (localObsCatg, "CALIB") == 0)
                {
                        (*localBatchNum)++;
                        cpl_msg_info(cpl_func,"\nSelected batch  %d  is  %s:\n", (*localBatchNum), batchTemplate);
                        fprintf (filePtr, "\nSelected batch  %d:  is  %s\n", (*localBatchNum), batchTemplate);
                        fprintf (inFitsTempPtr, "%s \n", batchTemplate);
                        for (i = 1; i <= numOfFiles; i++)
                        {
                                fgets (stringTemp, MAX_STRING_LENGTH, inFitsBatchTempPtr);
                                if (i > numOfInterfFiles)
                                {
                                        sprintf (dummyClassif, "%s", "");
                                        sscanf (stringTemp, "%s%s", fileNameStr, dummyClassif);
                                        cpl_msg_info(cpl_func,"%s %s\n", fileNameStr, dummyClassif);
                                        fprintf (filePtr, "%s %s\n", fileNameStr, dummyClassif);
                                        fprintf (inFitsTempPtr, "%s %s\n", fileNameStr, dummyClassif);
                                }
                        }
                        cpl_msg_info(cpl_func,"\n");
                        fprintf (filePtr, "\n");

                        /*      We should now have a complete batch of files 
                        in the temporary file list. Add an appending keyword */
                        fprintf (inFitsTempPtr, "%s\n", "END_OF_BATCH");
                }

                //      Now rewind the batch list and extract the Interferometry files          
                if (inFitsBatchTempPtr) rewind (inFitsBatchTempPtr);
                (*localBatchNum)++;
                if (strcmp (localObsCatg, "CALIB") == 0)
                {
                        cpl_msg_info(cpl_func,"\nSelected batch  %d  is  SCI_PHOT_CALIB:\n", (*localBatchNum));
                        fprintf (filePtr, "\nSelected batch  %d:  is  SCI_PHOT_CALIB\n", (*localBatchNum));
                        fprintf (inFitsTempPtr, "%s \n", "SCI_PHOT_CALIB");
                }
                else
                {
                        cpl_msg_info(cpl_func,"\nSelected batch  %d  is  SCI_PHOT_SCIENCE:\n", (*localBatchNum));
                        fprintf (filePtr, "\nSelected batch  %d:  is  SCI_PHOT_SCIENCE\n", (*localBatchNum));
                        fprintf (inFitsTempPtr, "%s \n", "SCI_PHOT_SCIENCE");
                }
                for (i = 1; i <= numOfFiles; i++)
                {
                        fgets (stringTemp, MAX_STRING_LENGTH, inFitsBatchTempPtr);
                        if (i <= numOfInterfFiles)
                        {
                                sprintf (dummyClassif, "%s", "");
                                sscanf (stringTemp, "%s%s", fileNameStr, dummyClassif);
                                cpl_msg_info(cpl_func,"%s %s\n", fileNameStr, dummyClassif);
                                fprintf (filePtr, "%s %s\n", fileNameStr, dummyClassif);
                                fprintf (inFitsTempPtr, "%s %s\n", fileNameStr, dummyClassif);
                        }
                }
                cpl_msg_info(cpl_func,"\n");
                fprintf (filePtr, "\n");

                /*      We should now have a complete batch of files 
                in the temporary file list. Add an appending keyword */
                fprintf (inFitsTempPtr, "%s\n", "END_OF_BATCH");
                if (inFitsBatchTempPtr)
                {
                        fclose (inFitsBatchTempPtr);
                        inFitsBatchTempPtr = NULL;
                }
                remove (inFitsBatchTemp);
        }
        else
        {
                (*localBatchNum)++;
                cpl_msg_info(cpl_func,"\nSelected batch  %d  is  %s:\n", (*localBatchNum), batchTemplate);
                fprintf (filePtr, "\nSelected batch  %d:  is  %s\n", (*localBatchNum), batchTemplate);
                fprintf (inFitsTempPtr, "%s \n", batchTemplate);
                while (fgets (stringTemp, MAX_STRING_LENGTH, inFitsBatchTempPtr) != NULL)
                {
                        sprintf (dummyClassif, "%s", "");
                        sscanf (stringTemp, "%s%s", fileNameStr, dummyClassif);
                        cpl_msg_info(cpl_func,"%s %s\n", fileNameStr, dummyClassif);
                        fprintf (filePtr, "%s %s\n", fileNameStr, dummyClassif);
                        fprintf (inFitsTempPtr, "%s %s\n", fileNameStr, dummyClassif);
                }
                cpl_msg_info(cpl_func,"\n");
                fprintf (filePtr, "\n");

                /*      We should now have a complete batch of files 
                in the temporary file list. Add an appending keyword */
                fprintf (inFitsTempPtr, "%s\n", "END_OF_BATCH");
                if (inFitsBatchTempPtr) 
                {       
                        fclose (inFitsBatchTempPtr);
                        inFitsBatchTempPtr = NULL;
                }
                remove (inFitsBatchTemp);
        }

    //  Release memory
        if (inFitsBatchTempPtr)
        {
                fclose (inFitsBatchTempPtr);
                inFitsBatchTempPtr = NULL;
        }
    if (inFitsBatchTemp) free (inFitsBatchTemp);
        if (localTplStart) free (localTplStart);
        if (localTplName) free (localTplName);
        if (stringTemp) free (stringTemp);
        if (localBeamCombiner) free (localBeamCombiner);
        if (localObsCatg) free (localObsCatg);
        if (localObsTech) free (localObsTech);
        if (localObsType) free (localObsType);

        return;
}
/*****************************************************************************/


/******************************************************************************
*               European Southern Observatory
*            VLTI MIDI Data Reduction Software
*
* Module name:  getAcquisitionBatch
* Input/Output: See function arguments to avoid duplication
* Description:  Acquisitio files are individual
*
* History:
* 28-Apr-05             (csabet) Created
******************************************************************************/
void getAcquisitionBatch (
    FILE                *inFitsTempPtr, // In: Pointer to the main buffer
        char            *fileNameStr,   // In: Name of the first FITS file
        char            *dummyClassif,  // In: Dummy classification
        FILE            *filePtr,               // In: Pointer to the log file
        int                     *localBatchNum, // IO: Local batch number
    int                 *error)                 // Ou: Error status
{

    //  Local Declarations
    //  ------------------
    const char  routine[] = "getAcquisitionBatch";
        
    //  Algorithm
    //  ---------
    if (diagnostic > 4) cpl_msg_info(cpl_func,"Invoking      routine   '%s' \n", routine);
    if (diagnostic > 4) fprintf(filePtr, "Invoking      routine   '%s' \n", routine);

        //      Reset status
        *error = 0;

        (*localBatchNum)++;
        sprintf (batchTemplate, "%s", "ACQ");
        cpl_msg_info(cpl_func,"\nSelected batch  %d  is  %s:\n", (*localBatchNum), batchTemplate);
        fprintf (filePtr, "\nSelected batch  %d  is  %s:\n", (*localBatchNum), batchTemplate);
        sprintf (dummyClassif, "%s", "");
        cpl_msg_info(cpl_func,"%s %s\n", fileNameStr, dummyClassif);
        fprintf (filePtr, "%s %s\n", fileNameStr, dummyClassif);
        fprintf (inFitsTempPtr, "%s \n", batchTemplate);
        fprintf (inFitsTempPtr, "%s %s\n", fileNameStr, dummyClassif);
        fprintf (inFitsTempPtr, "%s\n", "END_OF_BATCH");
        cpl_msg_info(cpl_func,"\n");
        fprintf (filePtr, "\n");

        return;
}
/*****************************************************************************/



 
/******************************************************************************
*               European Southern Observatory
*          VLTI MIDI Maintenance Templates Software
*
* Module name:  getDetLinBatch
* Input/Output: See function arguments to avoid duplication
* Description:  It loops through the given list of FITS files and searches for files
*                               that relate to Detector Linearity template. It then creates
*                               a batch file with the following format:
*                               
*                               <TEMPLATE_NAME> e.g. MIDI AUTOTEST TEC DETLIN
*                               <FITS File 1>
*                               <FITS File 2>
*                               <.>
*                               <.> 
*                               <FITS File n>
*                               END_OF_BATCH
*
* History:      
* 14-Jun-04     (csabet) created
******************************************************************************/
void getDetLinBatch (
    ImageFormat *format,                // In: Format structure
    FILE                *inFitsTempPtr, // In: Pointer to the main buffer
    FILE                *inFitsListPtr, // In: Pointer to the list of FITS files
        char            *fileNameStr,   // In: Name of the first FITS file
        char            *dummyClassif,  // In: Dummy classification
        FILE            *filePtr,               // In: Pointer to the log file
        int                     *localBatchNum, // IO: Local batch number
        int                     *fileNumInList, // IO: pointer to file number
    int                 *error)                 // Ou: Error status
{

    //  Local Declarations
    //  ------------------
    const char  routine[] = "getDetLinBatch";
        char            *inFitsBatchTemp=NULL, *localTplStart=NULL, *stringTemp=NULL;
        FILE            *inFitsBatchTempPtr=NULL;
        int                     numOfDataFiles, fileNumInTemplate, numOfEmptyFiles,
                                localFileNumInList;
        long            inFitsListPosition;
        
    //  Algorithm
    //  ---------
    if (diagnostic > 4) cpl_msg_info(cpl_func,"Invoking      routine   '%s' \n", routine);
    if (diagnostic > 4) fprintf(filePtr, "Invoking      routine   '%s' \n", routine);

        //      Reset status
        *error = 0;
        numOfDataFiles = 0;
        numOfEmptyFiles = 0;

        //      Allocate memory
    inFitsBatchTemp = (char *) calloc (MAX_STRING_LENGTH, sizeof (char));
    localTplStart = (char *) calloc (MAX_STRING_LENGTH, sizeof (char));
    stringTemp = (char *) calloc (MAX_STRING_LENGTH, sizeof (char));

        //      Open a batch temporary file for writing
        sprintf (inFitsBatchTemp, "localBuffer.log");
        if ((inFitsBatchTempPtr = fopen (inFitsBatchTemp, "w")) == NULL)
        {
                midiReportWarning (filePtr, routine, __FILE__, __LINE__, "Cannot open a batch temporary file");
                *error = 1;
                free (inFitsBatchTemp);
                free (localTplStart);
                free (stringTemp);
                return;
        }
                                
        //      Save the first file and its parameters
        fprintf (inFitsBatchTempPtr, "%s %s\n", fileNameStr, dummyClassif);
        fileNumInTemplate = format->fileNumInTemplate;
        sscanf (format->tplStart, "%s", localTplStart);
        numOfDataFiles++;
        localFileNumInList = *fileNumInList;
        inFitsListPosition = ftell (inFitsListPtr);
        sprintf (batchTemplate, "%s", "DETLIN");

        //      Get the rest of the batch files from the list
        while (fgets (stringTemp, MAX_STRING_LENGTH, inFitsListPtr) != NULL)
        {
                sprintf (dummyClassif, "%s", "");
                sscanf (stringTemp, "%s%s", fileNameStr, dummyClassif);

                localFileNumInList++;
                fileNumInTemplate++;

                //      Get FITS file parameters. If ok copy into the batch temporary file
                getFitsClassification (localFileNumInList, filePtr, fileNameStr, format, error);

                //      We expect possibly more split files
                if (!(*error) && (strcmp (format->obsCatg, "CALIB") == 0) &&
                        ((strcmp (format->obsTech, "IMAGE") == 0) || 
                        (strcmp (format->obsTech, "SPECTRUM") == 0)) &&
                        (strcmp (format->obsType, "FLAT") == 0) &&
                        (strcmp (format->tplStart, localTplStart) == 0) &&
                        (format->fileNumInTemplate == fileNumInTemplate))
                {
                        fprintf (inFitsBatchTempPtr, "%s %s\n", fileNameStr, dummyClassif);
                        numOfDataFiles++;
                }
                else if (!(*error) && (strcmp (format->obsCatg, "CALIB") == 0) &&
                        (strcmp (format->obsTech, "OTHER") == 0) &&
                        (strcmp (format->obsType, "OTHER") == 0) &&
                        (strcmp (format->tplStart, localTplStart) == 0) &&
                        (format->fileNumInTemplate == fileNumInTemplate) && 
                        (numOfEmptyFiles < 1))
                {
                        fprintf (inFitsBatchTempPtr, "%s %s\n", fileNameStr, dummyClassif);
                        numOfEmptyFiles++;
                        break;
                }
                else
                {
                        //      We have not found a complete set. Hence reset the file position
                        fseek (inFitsListPtr, inFitsListPosition, SEEK_SET);
                        localFileNumInList--;
                        break;
                }
                inFitsListPosition = ftell (inFitsListPtr);
        }

        //      If END_OF_TEMPLATE is deactivated then we do not check the numOfEmptyFiles
        if (!endOfTemplate) numOfEmptyFiles = 1;

        //      We expect at least one data file and one empty file. If there has been 
        //      a complete set of files then copy all files into the main temporary file
        if ((numOfDataFiles > 0) && (numOfEmptyFiles == 1))
        {
                //      Reopen the batch temporary file for reading
                if (inFitsBatchTempPtr) 
                {
                        fclose (inFitsBatchTempPtr);
                        inFitsBatchTempPtr = NULL;
                }
                if ((inFitsBatchTempPtr = fopen (inFitsBatchTemp, "r")) == NULL)
                {
                        midiReportWarning (filePtr, routine, __FILE__, __LINE__, "Cannot open a batch temporary file");
                        *error = 1;
                        free (inFitsBatchTemp);
                        free (localTplStart);
                        free (stringTemp);
                        return;
                }

                (*localBatchNum)++;
                cpl_msg_info(cpl_func,"\nSelected batch  %d  is  %s:\n", (*localBatchNum), batchTemplate);
                fprintf (filePtr, "\nSelected batch  %d:  is  %s\n", (*localBatchNum), batchTemplate);
                fprintf (inFitsTempPtr, "%s \n", batchTemplate);
                while (fgets (stringTemp, MAX_STRING_LENGTH, inFitsBatchTempPtr) != NULL)
                {
                        sprintf (dummyClassif, "%s", "");
                        sscanf (stringTemp, "%s%s", fileNameStr, dummyClassif);
                        cpl_msg_info(cpl_func,"%s %s\n", fileNameStr, dummyClassif);
                        fprintf (filePtr, "%s %s\n", fileNameStr, dummyClassif);
                        fprintf (inFitsTempPtr, "%s %s\n", fileNameStr, dummyClassif);
                }
                cpl_msg_info(cpl_func,"\n");
                fprintf (filePtr, "\n");
        
                //      We should now have a complete set of files
                //      in the temporary file list. Add an appending keyword
                fprintf (inFitsTempPtr, "%s\n", "END_OF_BATCH");
        }
        *fileNumInList = localFileNumInList;
        if (inFitsBatchTempPtr) 
        {
                fclose (inFitsBatchTempPtr);
                inFitsBatchTempPtr = NULL;
        }
        remove (inFitsBatchTemp);

    //  Release memory
    if (inFitsBatchTemp) free (inFitsBatchTemp);
        if (localTplStart) free (localTplStart);
        if (stringTemp) free (stringTemp);

        return;
}
/*****************************************************************************/
 

/******************************************************************************
*               European Southern Observatory
*          VLTI MIDI Maintenance Templates Software
*
* Module name:  getDetRonBatch
* Input/Output: See function arguments to avoid duplication
* Description:  It loops through the given list of FITS files and searches for files
*                               that relate to Detector Readout Noise template. It then creates
*                               a batch file with the following format:
*                               
*                               <TEMPLATE_NAME> e.g. MIDI AUTOTEST TEC DETRON
*                               <FITS File 1>
*                               <FITS File 2>
*                               <.>
*                               <.> 
*                               <FITS File n>
*                               END_OF_BATCH
*
* History:      
* 14-Jun-04     (csabet) created
******************************************************************************/
void getDetRonBatch (
    ImageFormat *format,                // In: Format structure
    FILE                *inFitsTempPtr, // In: Pointer to the main buffer
    FILE                *inFitsListPtr, // In: Pointer to the list of FITS files
        char            *fileNameStr,   // In: Name of the first FITS file
        char            *dummyClassif,  // In: Dummy classification
        FILE            *filePtr,               // In: Pointer to the log file
        int                     *localBatchNum, // IO: Local batch number
        int                     *fileNumInList, // IO: pointer to file number
    int                 *error)                 // Ou: Error status
{

    //  Local Declarations
    //  ------------------
    const char  routine[] = "getDetRonBatch";
        char            *inFitsBatchTemp=NULL, *localTplStart=NULL, *stringTemp=NULL;
        FILE            *inFitsBatchTempPtr=NULL;
        int                     numOfDataFiles, fileNumInTemplate, numOfEmptyFiles,
                                localFileNumInList;
        long            inFitsListPosition;
        
    //  Algorithm
    //  ---------
    if (diagnostic > 4) cpl_msg_info(cpl_func,"Invoking      routine   '%s' \n", routine);
    if (diagnostic > 4) fprintf(filePtr, "Invoking      routine   '%s' \n", routine);

        //      Reset status
        *error = 0;
        numOfDataFiles = 0;
        numOfEmptyFiles = 0;

        //      Allocate memory
    inFitsBatchTemp = (char *) calloc (MAX_STRING_LENGTH, sizeof (char));
    localTplStart = (char *) calloc (MAX_STRING_LENGTH, sizeof (char));
    stringTemp = (char *) calloc (MAX_STRING_LENGTH, sizeof (char));

        //      Open a batch temporary file for writing
        sprintf (inFitsBatchTemp, "localBuffer.log");
        if ((inFitsBatchTempPtr = fopen (inFitsBatchTemp, "w")) == NULL)
        {
                midiReportWarning (filePtr, routine, __FILE__, __LINE__, "Cannot open a batch temporary file");
                *error = 1;
                free (inFitsBatchTemp);
                free (localTplStart);
                free (stringTemp);
                return;
        }
                                
        //      Save the first file and its parameters
        fprintf (inFitsBatchTempPtr, "%s %s\n", fileNameStr, dummyClassif);
        fileNumInTemplate = format->fileNumInTemplate;
        sscanf (format->tplStart, "%s", localTplStart);
        numOfDataFiles++;
        localFileNumInList = *fileNumInList;
        inFitsListPosition = ftell (inFitsListPtr);
        sprintf (batchTemplate, "%s", "DETRON");

        //      Get the rest of the batch files from the list
        while (fgets (stringTemp, MAX_STRING_LENGTH, inFitsListPtr) != NULL)
        {
                sprintf (dummyClassif, "%s", "");
                sscanf (stringTemp, "%s%s", fileNameStr, dummyClassif);

                localFileNumInList++;
                fileNumInTemplate++;

                //      Get FITS file parameters. If ok copy into the batch temporary file
                getFitsClassification (localFileNumInList, filePtr, fileNameStr, format, error);

                //      We expect possibly more split files
                if (!(*error) && (strcmp (format->obsCatg, "CALIB") == 0) &&
                        (strcmp (format->obsTech, "IMAGE") == 0) &&
                        (strcmp (format->obsType, "BIAS") == 0) &&
                        (strcmp (format->tplStart, localTplStart) == 0) &&
                        (format->fileNumInTemplate == fileNumInTemplate))
                {
                        fprintf (inFitsBatchTempPtr, "%s %s\n", fileNameStr, dummyClassif);
                        numOfDataFiles++;
                }
                else if (!(*error) && (strcmp (format->obsCatg, "CALIB") == 0) &&
                        (strcmp (format->obsTech, "OTHER") == 0) &&
                        (strcmp (format->obsType, "OTHER") == 0) &&
                        (strcmp (format->tplStart, localTplStart) == 0) &&
                        (format->fileNumInTemplate == fileNumInTemplate) && 
                        (numOfEmptyFiles < 1))
                {
                        fprintf (inFitsBatchTempPtr, "%s %s\n", fileNameStr, dummyClassif);
                        numOfEmptyFiles++;
                        break;
                }
                else
                {
                        //      We have not found a complete set. Hence reset the file position
                        fseek (inFitsListPtr, inFitsListPosition, SEEK_SET);
                        localFileNumInList--;
                        break;
                }
                inFitsListPosition = ftell (inFitsListPtr);
        }

        //      If END_OF_TEMPLATE is deactivated then we do not check the numOfEmptyFiles
        if (!endOfTemplate) numOfEmptyFiles = 1;

        //      We expect at least one data file and one empty file. If there has been 
        //      a complete set of files then copy all files into the main temporary file
        if ((numOfDataFiles > 0) && (numOfEmptyFiles == 1))
        {
                //      Reopen the batch temporary file for reading
                if (inFitsBatchTempPtr) 
                {
                        fclose (inFitsBatchTempPtr);
                        inFitsBatchTempPtr = NULL;
                }
                if ((inFitsBatchTempPtr = fopen (inFitsBatchTemp, "r")) == NULL)
                {
                        midiReportWarning (filePtr, routine, __FILE__, __LINE__, "Cannot open a batch temporary file");
                        *error = 1;
                        free (inFitsBatchTemp);
                        free (localTplStart);
                        free (stringTemp);
                        return;
                }

                (*localBatchNum)++;
                cpl_msg_info(cpl_func,"\nSelected batch  %d  is  %s:\n", (*localBatchNum), batchTemplate);
                fprintf (filePtr, "\nSelected batch  %d:  is  %s\n", (*localBatchNum), batchTemplate);
                fprintf (inFitsTempPtr, "%s \n", batchTemplate);
                while (fgets (stringTemp, MAX_STRING_LENGTH, inFitsBatchTempPtr) != NULL)
                {
                        sprintf (dummyClassif, "%s", "");
                        sscanf (stringTemp, "%s%s", fileNameStr, dummyClassif);
                        cpl_msg_info(cpl_func,"%s %s\n", fileNameStr, dummyClassif);
                        fprintf (filePtr, "%s %s\n", fileNameStr, dummyClassif);
                        fprintf (inFitsTempPtr, "%s %s\n", fileNameStr, dummyClassif);
                }
                cpl_msg_info(cpl_func,"\n");
                fprintf (filePtr, "\n");
        
                //      We should now have a complete set of files in the 
                //      temporary file list. Add an appending keyword
                fprintf (inFitsTempPtr, "%s\n", "END_OF_BATCH");
        }
        *fileNumInList = localFileNumInList;
        if (inFitsBatchTempPtr) 
        {
                fclose (inFitsBatchTempPtr);
                inFitsBatchTempPtr = NULL;
        }
        remove (inFitsBatchTemp);

    //  Release memory
    if (inFitsBatchTemp) free (inFitsBatchTemp);
        if (localTplStart) free (localTplStart);
        if (stringTemp) free (stringTemp);

        return;
}
/*****************************************************************************/



/******************************************************************************
*               European Southern Observatory
*          VLTI MIDI Maintenance Templates Software
*
* Module name:  getDspTrnBatch
* Input/Output: See function arguments to avoid duplication
* Description:  It loops through the given list of FITS files and searches for files
*                               that relate to Detector Readout Noise template. It then creates
*                               a batch file with the following format:
*                               
*                               <TEMPLATE_NAME> e.g. MIDI AUTOTEST TEC DSPTRN
*                               <FITS File 1>
*                               <FITS File 2>
*                               <.>
*                               <.> 
*                               <FITS File n>
*                               END_OF_BATCH
*
* History:      
* 14-Jun-04     (csabet) created
******************************************************************************/
void getDspTrnBatch (
    ImageFormat *format,                // In: Format structure
    FILE                *inFitsTempPtr, // In: Pointer to the main buffer
    FILE                *inFitsListPtr, // In: Pointer to the list of FITS files
        char            *fileNameStr,   // In: Name of the first FITS file
        char            *dummyClassif,  // In: Dummy classification
        FILE            *filePtr,               // In: Pointer to the log file
        int                     *localBatchNum, // IO: Local batch number
        int                     *fileNumInList, // IO: pointer to file number
    int                 *error)                 // Ou: Error status
{

    //  Local Declarations
    //  ------------------
    const char  routine[] = "getDspTrnBatch";
        char            *inFitsBatchTemp=NULL, *localTplStart=NULL, *stringTemp=NULL;
        FILE            *inFitsBatchTempPtr=NULL;
        int                     numOfDataFiles, fileNumInTemplate, numOfEmptyFiles,
                                localFileNumInList;
        long            inFitsListPosition;
        
    //  Algorithm
    //  ---------
    if (diagnostic > 4) cpl_msg_info(cpl_func,"Invoking      routine   '%s' \n", routine);
    if (diagnostic > 4) fprintf(filePtr, "Invoking      routine   '%s' \n", routine);

        //      Reset status
        *error = 0;
        numOfDataFiles = 0;
        numOfEmptyFiles = 0;

        //      Allocate memory
    inFitsBatchTemp = (char *) calloc (MAX_STRING_LENGTH, sizeof (char));
    localTplStart = (char *) calloc (MAX_STRING_LENGTH, sizeof (char));
    stringTemp = (char *) calloc (MAX_STRING_LENGTH, sizeof (char));

        //      Open a batch temporary file for writing
        sprintf (inFitsBatchTemp, "localBuffer.log");
        if ((inFitsBatchTempPtr = fopen (inFitsBatchTemp, "w")) == NULL)
        {
                midiReportWarning (filePtr, routine, __FILE__, __LINE__, "Cannot open a batch temporary file");
                *error = 1;
                free (inFitsBatchTemp);
                free (localTplStart);
                free (stringTemp);
                return;
        }
                                
        //      Save the first file and its parameters
        fprintf (inFitsBatchTempPtr, "%s %s\n", fileNameStr, dummyClassif);
        fileNumInTemplate = format->fileNumInTemplate;
        sscanf (format->tplStart, "%s", localTplStart);
        numOfDataFiles++;
        localFileNumInList = *fileNumInList;
        inFitsListPosition = ftell (inFitsListPtr);
        sprintf (batchTemplate, "%s", "DSPTRN");

        //      Get the rest of the batch files from the list
        while (fgets (stringTemp, MAX_STRING_LENGTH, inFitsListPtr) != NULL)
        {
                sprintf (dummyClassif, "%s", "");
                sscanf (stringTemp, "%s%s", fileNameStr, dummyClassif);

                localFileNumInList++;
                fileNumInTemplate++;

                //      Get FITS file parameters. If ok copy into the batch temporary file
                getFitsClassification (localFileNumInList, filePtr, fileNameStr, format, error);

                //      We expect possibly more split files
                if (!(*error) && (strcmp (format->obsCatg, "CALIB") == 0) &&
                        (strcmp (format->obsTech, "SPECTRUM") == 0) &&
                        (strcmp (format->obsType, "WAVE") == 0) &&
                        (strcmp (format->tplStart, localTplStart) == 0) &&
                        (format->fileNumInTemplate == fileNumInTemplate))
                {
                        fprintf (inFitsBatchTempPtr, "%s %s\n", fileNameStr, dummyClassif);
                        numOfDataFiles++;
                }
                else if (!(*error) && (strcmp (format->obsCatg, "CALIB") == 0) &&
                        (strcmp (format->obsTech, "OTHER") == 0) &&
                        (strcmp (format->obsType, "OTHER") == 0) &&
                        (strcmp (format->tplStart, localTplStart) == 0) &&
                        (format->fileNumInTemplate == fileNumInTemplate) && 
                        (numOfEmptyFiles < 1))
                {
                        fprintf (inFitsBatchTempPtr, "%s %s\n", fileNameStr, dummyClassif);
                        numOfEmptyFiles++;
                        break;
                }
                else
                {
                        //      We have not found a complete set. Hence reset the file position
                        fseek (inFitsListPtr, inFitsListPosition, SEEK_SET);
                        localFileNumInList--;
                        break;
                }
                inFitsListPosition = ftell (inFitsListPtr);
        }

        //      If END_OF_TEMPLATE is deactivated then we do not check the numOfEmptyFiles
        if (!endOfTemplate) numOfEmptyFiles = 1;

        //      We expect at least one data file and one empty file. If there has been 
        //      a complete set of files then copy all files into the main temporary file
        if ((numOfDataFiles > 0) && (numOfEmptyFiles == 1))
        {
                //      Reopen the batch temporary file for reading
                if (inFitsBatchTempPtr) 
                {
                        fclose (inFitsBatchTempPtr);
                        inFitsBatchTempPtr = NULL;
                }
                if ((inFitsBatchTempPtr = fopen (inFitsBatchTemp, "r")) == NULL)
                {
                        midiReportWarning (filePtr, routine, __FILE__, __LINE__, "Cannot open a batch temporary file");
                        *error = 1;
                        free (inFitsBatchTemp);
                        free (localTplStart);
                        free (stringTemp);
                        return;
                }

                (*localBatchNum)++;
                cpl_msg_info(cpl_func,"\nSelected batch  %d  is  %s:\n", (*localBatchNum), batchTemplate);
                fprintf (filePtr, "\nSelected batch  %d:  is  %s\n", (*localBatchNum), batchTemplate);
                fprintf (inFitsTempPtr, "%s \n", batchTemplate);
                while (fgets (stringTemp, MAX_STRING_LENGTH, inFitsBatchTempPtr) != NULL)
                {
                        sprintf (dummyClassif, "%s", "");
                        sscanf (stringTemp, "%s%s", fileNameStr, dummyClassif);
                        cpl_msg_info(cpl_func,"%s %s\n", fileNameStr, dummyClassif);
                        fprintf (filePtr, "%s %s\n", fileNameStr, dummyClassif);
                        fprintf (inFitsTempPtr, "%s %s\n", fileNameStr, dummyClassif);
                }
                cpl_msg_info(cpl_func,"\n");
                fprintf (filePtr, "\n");
        
                //      We should now have a complete set of files
                //      in the temporary file list. Add an appending keyword
                fprintf (inFitsTempPtr, "%s\n", "END_OF_BATCH");
        }
        *fileNumInList = localFileNumInList;
        if (inFitsBatchTempPtr) 
        {
                fclose (inFitsBatchTempPtr);
                inFitsBatchTempPtr = NULL;
        }
        remove (inFitsBatchTemp);

    //  Release memory
    if (inFitsBatchTemp) free (inFitsBatchTemp);
        if (localTplStart) free (localTplStart);
        if (stringTemp) free (stringTemp);

        return;
}
/*****************************************************************************/
 
 
/******************************************************************************
*               European Southern Observatory
*          VLTI MIDI Maintenance Templates Software
*
* Module name:  getRefPixBatch
* Input/Output: See function arguments to avoid duplication
* Description:  It loops through the given list of FITS files and searches for files
*                               that relate to the Reference Pixels Alignment template. It then creates
*                               a batch file with the following format:
*                               
*                               <TEMPLATE_NAME> e.g. MIDI AUTOTEST TEC REFPIX
*                               <FITS File 1>
*                               <FITS File 2>
*                               <.>
*                               <.> 
*                               <FITS File n>
*                               END_OF_BATCH
*
* History:      
* 14-Jun-04     (csabet) created
******************************************************************************/
void getRefPixBatch (
    ImageFormat *format,                // In: Format structure
    FILE                *inFitsTempPtr, // In: Pointer to the main buffer
    FILE                *inFitsListPtr, // In: Pointer to the list of FITS files
        char            *fileNameStr,   // In: Name of the first FITS file
        char            *dummyClassif,  // In: Dummy classification
        FILE            *filePtr,               // In: Pointer to the log file
        int                     *localBatchNum, // IO: Local batch number
        int                     *fileNumInList, // IO: pointer to file number
    int                 *error)                 // Ou: Error status
{

    //  Local Declarations
    //  ------------------
    const char  routine[] = "getRefPixBatch";
        char            *inFitsBatchTemp=NULL, *localTplStart=NULL, *stringTemp=NULL;
        FILE            *inFitsBatchTempPtr=NULL;
        int                     numOfDataFiles, fileNumInTemplate, numOfEmptyFiles,
                                localFileNumInList;
        long            inFitsListPosition;
        
    //  Algorithm
    //  ---------
    if (diagnostic > 4) cpl_msg_info(cpl_func,"Invoking      routine   '%s' \n", routine);
    if (diagnostic > 4) fprintf(filePtr, "Invoking      routine   '%s' \n", routine);

        //      Reset status
        *error = 0;
        numOfDataFiles = 0;
        numOfEmptyFiles = 0;

        //      Allocate memory
    inFitsBatchTemp = (char *) calloc (MAX_STRING_LENGTH, sizeof (char));
    localTplStart = (char *) calloc (MAX_STRING_LENGTH, sizeof (char));
    stringTemp = (char *) calloc (MAX_STRING_LENGTH, sizeof (char));

        //      Open a batch temporary file for writing
        sprintf (inFitsBatchTemp, "localBuffer.log");
        if ((inFitsBatchTempPtr = fopen (inFitsBatchTemp, "w")) == NULL)
        {
                midiReportWarning (filePtr, routine, __FILE__, __LINE__, "Cannot open a batch temporary file");
                *error = 1;
                free (inFitsBatchTemp);
                free (localTplStart);
                free (stringTemp);
                return;
        }
                                
        //      Save the first file and its parameters
        fprintf (inFitsBatchTempPtr, "%s %s\n", fileNameStr, dummyClassif);
        fileNumInTemplate = format->fileNumInTemplate;
        sscanf (format->tplStart, "%s", localTplStart);
        numOfDataFiles++;
        localFileNumInList = *fileNumInList;
        inFitsListPosition = ftell (inFitsListPtr);
        sprintf (batchTemplate, "%s", "REFPIX");

        //      Get the rest of the batch files from the list
        while (fgets (stringTemp, MAX_STRING_LENGTH, inFitsListPtr) != NULL)
        {
                sprintf (dummyClassif, "%s", "");
                sscanf (stringTemp, "%s%s", fileNameStr, dummyClassif);

                localFileNumInList++;
                fileNumInTemplate++;

                //      Get FITS file parameters. If ok copy into the batch temporary file
                getFitsClassification (localFileNumInList, filePtr, fileNameStr, format, error);

                //      We expect possibly more split files
                if (!(*error) && (strcmp (format->obsCatg, "CALIB") == 0) &&
                        (strcmp (format->obsTech, "IMAGE") == 0) &&
                        (strcmp (format->obsType, "FMTCHCK") == 0) &&
                        (strcmp (format->tplStart, localTplStart) == 0) &&
                        (format->fileNumInTemplate == fileNumInTemplate))
                {
                        fprintf (inFitsBatchTempPtr, "%s %s\n", fileNameStr, dummyClassif);
                        numOfDataFiles++;
                }
                else if (!(*error) && (strcmp (format->obsCatg, "CALIB") == 0) &&
                        (strcmp (format->obsTech, "OTHER") == 0) &&
                        (strcmp (format->obsType, "OTHER") == 0) &&
                        (strcmp (format->tplStart, localTplStart) == 0) &&
                        (format->fileNumInTemplate == fileNumInTemplate) && 
                        (numOfEmptyFiles < 1))
                {
                        fprintf (inFitsBatchTempPtr, "%s %s\n", fileNameStr, dummyClassif);
                        numOfEmptyFiles++;
                        break;
                }
                else
                {
                        //      We have not found a complete set. Hence reset the file position
                        fseek (inFitsListPtr, inFitsListPosition, SEEK_SET);
                        localFileNumInList--;
                        break;
                }
                inFitsListPosition = ftell (inFitsListPtr);
        }

        //      If END_OF_TEMPLATE is deactivated then we do not check the numOfEmptyFiles
        if (!endOfTemplate) numOfEmptyFiles = 1;

        //      We expect at least one data file and one empty file. If there has been 
        //      a complete set of files then copy all files into the main temporary file
        if ((numOfDataFiles > 0) && (numOfEmptyFiles == 1))
        {
                //      Reopen the batch temporary file for reading
                if (inFitsBatchTempPtr) 
                {
                        fclose (inFitsBatchTempPtr);
                        inFitsBatchTempPtr = NULL;
                }
                if ((inFitsBatchTempPtr = fopen (inFitsBatchTemp, "r")) == NULL)
                {
                        midiReportWarning (filePtr, routine, __FILE__, __LINE__, "Cannot open a batch temporary file");
                        *error = 1;
                        free (inFitsBatchTemp);
                        free (localTplStart);
                        free (stringTemp);
                        return;
                }

                (*localBatchNum)++;
                cpl_msg_info(cpl_func,"\nSelected batch  %d  is  %s:\n", (*localBatchNum), batchTemplate);
                fprintf (filePtr, "\nSelected batch  %d:  is  %s\n", (*localBatchNum), batchTemplate);
                fprintf (inFitsTempPtr, "%s \n", batchTemplate);
                while (fgets (stringTemp, MAX_STRING_LENGTH, inFitsBatchTempPtr) != NULL)
                {
                        sprintf (dummyClassif, "%s", "");
                        sscanf (stringTemp, "%s%s", fileNameStr, dummyClassif);
                        cpl_msg_info(cpl_func,"%s %s\n", fileNameStr, dummyClassif);
                        fprintf (filePtr, "%s %s\n", fileNameStr, dummyClassif);
                        fprintf (inFitsTempPtr, "%s %s\n", fileNameStr, dummyClassif);
                }
                cpl_msg_info(cpl_func,"\n");
                fprintf (filePtr, "\n");
        
                //      We should now have a complete set of files
                //      in the temporary file list. Add an appending keyword
                fprintf (inFitsTempPtr, "%s\n", "END_OF_BATCH");
        }
        *fileNumInList = localFileNumInList;
        if (inFitsBatchTempPtr) 
        {
                fclose (inFitsBatchTempPtr);
                inFitsBatchTempPtr = NULL;
        }
        remove (inFitsBatchTemp);

    //  Release memory
    if (inFitsBatchTemp) free (inFitsBatchTemp);
        if (localTplStart) free (localTplStart);
        if (stringTemp) free (stringTemp);

        return;
}
/*****************************************************************************/


/******************************************************************************
*               European Southern Observatory
*          VLTI MIDI Maintenance Templates Software
*
* Module name:  getWavCalBatch
* Input/Output: See function arguments to avoid duplication
* Description:  It loops through the given list of FITS files and searches for files
*                               that relate to the Wavelength Calibration template. It then creates
*                               a batch file with the following format:
*                               
*                               <TEMPLATE_NAME> e.g. MIDI AUTOTEST TEC WAVECAL
*                               <FITS File 1>
*                               <FITS File 2>
*                               <.>
*                               <.> 
*                               <FITS File n>
*                               END_OF_BATCH
*
* History:      
* 02-Jun-05     (csabet) created
******************************************************************************/
void getWavCalBatch (
    ImageFormat *format,                // In: Format structure
    FILE                *inFitsTempPtr, // In: Pointer to the main buffer
    FILE                *inFitsListPtr, // In: Pointer to the list of FITS files
        char            *fileNameStr,   // In: Name of the first FITS file
        char            *dummyClassif,  // In: Dummy classification
        FILE            *filePtr,               // In: Pointer to the log file
        int                     *localBatchNum, // IO: Local batch number
        int                     *fileNumInList, // IO: pointer to file number
    int                 *error)                 // Ou: Error status
{

    //  Local Declarations
    //  ------------------
    const char  routine[] = "getWavCalBatch";
        char            *inFitsBatchTemp=NULL, *localTplStart=NULL, *stringTemp=NULL;
        FILE            *inFitsBatchTempPtr=NULL;
        int                     numOfDataFiles, fileNumInTemplate, numOfEmptyFiles,
                                localFileNumInList;
        long            inFitsListPosition;
        
    //  Algorithm
    //  ---------
    if (diagnostic > 4) cpl_msg_info(cpl_func,"Invoking      routine   '%s' \n", routine);
    if (diagnostic > 4) fprintf(filePtr, "Invoking      routine   '%s' \n", routine);

        //      Reset status
        *error = 0;
        numOfDataFiles = 0;
        numOfEmptyFiles = 0;

        //      Allocate memory
    inFitsBatchTemp = (char *) calloc (MAX_STRING_LENGTH, sizeof (char));
    localTplStart = (char *) calloc (MAX_STRING_LENGTH, sizeof (char));
    stringTemp = (char *) calloc (MAX_STRING_LENGTH, sizeof (char));

        //      Open a batch temporary file for writing
        sprintf (inFitsBatchTemp, "localBuffer.log");
        if ((inFitsBatchTempPtr = fopen (inFitsBatchTemp, "w")) == NULL)
        {
                midiReportWarning (filePtr, routine, __FILE__, __LINE__, "Cannot open a batch temporary file");
                *error = 1;
                free (inFitsBatchTemp);
                free (localTplStart);
                free (stringTemp);
                return;
        }
                                
        //      Save the first file and its parameters
        fprintf (inFitsBatchTempPtr, "%s %s\n", fileNameStr, dummyClassif);
        fileNumInTemplate = format->fileNumInTemplate;
        sscanf (format->tplStart, "%s", localTplStart);
        numOfDataFiles++;
        localFileNumInList = *fileNumInList;
        inFitsListPosition = ftell (inFitsListPtr);
        sprintf (batchTemplate, "%s", "WAVECAL");

        //      Get the rest of the batch files from the list
        while (fgets (stringTemp, MAX_STRING_LENGTH, inFitsListPtr) != NULL)
        {
                sprintf (dummyClassif, "%s", "");
                sscanf (stringTemp, "%s%s", fileNameStr, dummyClassif);

                localFileNumInList++;
                fileNumInTemplate++;

                //      Get FITS file parameters. If ok copy into the batch temporary file
                getFitsClassification (localFileNumInList, filePtr, fileNameStr, format, error);

                //      We expect possibly more split files
                if (!(*error) && (strcmp (format->obsCatg, "CALIB") == 0) &&
                        (strcmp (format->obsTech, "SPECTRUM") == 0) &&
                        (strcmp (format->obsType, "WAVE,SPECTEMPL") == 0) &&
                        (strcmp (format->tplStart, localTplStart) == 0) &&
                        (format->fileNumInTemplate == fileNumInTemplate))
                {
                        fprintf (inFitsBatchTempPtr, "%s %s\n", fileNameStr, dummyClassif);
                        numOfDataFiles++;
                }
                else if (!(*error) && (strcmp (format->obsCatg, "CALIB") == 0) &&
                        (strcmp (format->obsTech, "OTHER") == 0) &&
                        (strcmp (format->obsType, "OTHER") == 0) &&
                        (strcmp (format->tplStart, localTplStart) == 0) &&
                        (format->fileNumInTemplate == fileNumInTemplate) && 
                        (numOfEmptyFiles < 1))
                {
                        fprintf (inFitsBatchTempPtr, "%s %s\n", fileNameStr, dummyClassif);
                        numOfEmptyFiles++;
                        break;
                }
                else
                {
                        //      We have not found a complete set. Hence reset the file position
                        fseek (inFitsListPtr, inFitsListPosition, SEEK_SET);
                        localFileNumInList--;
                        break;
                }
                inFitsListPosition = ftell (inFitsListPtr);
        }

        //      If END_OF_TEMPLATE is deactivated then we do not check the numOfEmptyFiles
        if (!endOfTemplate) numOfEmptyFiles = 1;

        //      We expect at least one data file and one empty file. If there has been 
        //      a complete set of files then copy all files into the main temporary file
        if ((numOfDataFiles > 0) && (numOfEmptyFiles == 1))
        {
                //      Reopen the batch temporary file for reading
                if (inFitsBatchTempPtr) 
                {
                        fclose (inFitsBatchTempPtr);
                        inFitsBatchTempPtr = NULL;
                }
                if ((inFitsBatchTempPtr = fopen (inFitsBatchTemp, "r")) == NULL)
                {
                        midiReportWarning (filePtr, routine, __FILE__, __LINE__, "Cannot open a batch temporary file");
                        *error = 1;
                        free (inFitsBatchTemp);
                        free (localTplStart);
                        free (stringTemp);
                        return;
                }

                (*localBatchNum)++;
                cpl_msg_info(cpl_func,"\nSelected batch  %d  is  %s:\n", (*localBatchNum), batchTemplate);
                fprintf (filePtr, "\nSelected batch  %d:  is  %s\n", (*localBatchNum), batchTemplate);
                fprintf (inFitsTempPtr, "%s \n", batchTemplate);
                while (fgets (stringTemp, MAX_STRING_LENGTH, inFitsBatchTempPtr) != NULL)
                {
                        sprintf (dummyClassif, "%s", "");
                        sscanf (stringTemp, "%s%s", fileNameStr, dummyClassif);
                        cpl_msg_info(cpl_func,"%s %s\n", fileNameStr, dummyClassif);
                        fprintf (filePtr, "%s %s\n", fileNameStr, dummyClassif);
                        fprintf (inFitsTempPtr, "%s %s\n", fileNameStr, dummyClassif);
                }
                cpl_msg_info(cpl_func,"\n");
                fprintf (filePtr, "\n");
        
                //      We should now have a complete set of files
                //      in the temporary file list. Add an appending keyword
                fprintf (inFitsTempPtr, "%s\n", "END_OF_BATCH");
        }
        *fileNumInList = localFileNumInList;
        if (inFitsBatchTempPtr) 
        {
                fclose (inFitsBatchTempPtr);
                inFitsBatchTempPtr = NULL;
        }
        remove (inFitsBatchTemp);

    //  Release memory
    if (inFitsBatchTemp) free (inFitsBatchTemp);
        if (localTplStart) free (localTplStart);
        if (stringTemp) free (stringTemp);

        return;
}
/*****************************************************************************/
