/******************************************************************************
*******************************************************************************
*               European Southern Observatory
*             VLTI MIDI Data Reduction Software
*
* Module name:  visibility.c
* Description:  Contains routines for all visibility computation routines
*
* History:
* 21-Jul-03     (csabet) Created
*******************************************************************************
******************************************************************************/

/******************************************************************************
*   Compiler directives
******************************************************************************/

/******************************************************************************
*   Include files
******************************************************************************/
#include <stdio.h>
#include <cpl.h>
#include <stdlib.h>
#include <string.h>
#include <math.h>
#include "midiGlobal.h"
#include "midiLib.h"
#include "visibility.h"
#include "errorHandling.h"
#include "diagnostics.h"
#include "midi_dfs.h"

/*============================ C O D E    A R E A ===========================*/




/******************************************************************************
*               European Southern Observatory
*            VLTI MIDI Data Reduction Software
*
* Module name:  computeCalibVis
* Input/Output: See function arguments to avoid duplication
* Description:  This routine computes the final visibility on SCIENCE targets. It
*               requires the transfer function as well as the uncalibrated
*               visibilities.
*
* History:
* 08-Feb-05     (csabet) Created
******************************************************************************/
void computeCalibVis (
    MidiFiles        *fileNames,                // In: Pointer to MIDI file structure
    ImageFormat        *format,                // In: Image structure
    DispersedResult    *dispResult,            // In: Dispersed result structure
    int                *error,                    // Ou: Status
    cpl_parameterlist  *parlist,
    cpl_frameset       *frameset)
{

    //    Local Declarations
    //    ------------------
    const char  routine[] = "computeCalibVis";
    int         X;
/*
    FILE        *trfPtr=NULL;
    int			numOfRecords;
    char        *givenGrismId;
    char             * tag=NULL;
*/
    char             * softag=NULL;
    cpl_frame        * cur_frame=NULL;
    cpl_table        * table=NULL;
    cpl_errorstate     prestate = cpl_errorstate_get();   
    int                ext_trf=0;
    int                dimentrf=0;
    cpl_frame        * local_trf_frame=NULL;
    cpl_propertylist * dummy_propertylist = NULL;    
    cpl_propertylist * qclist=NULL;    
    char             * dataname=NULL;
    //    Algorithm
    //    ---------
    if (diagnostic > 4)cpl_msg_info(cpl_func,"Invoking      routine   '%s' \n", routine);
    if (diagnostic > 4) fprintf(midiReportPtr, "Invoking      routine   '%s' \n", routine);

   cpl_msg_info(cpl_func,"\nComputing Calibrated Visibility for batch  %d \n", batchNumber);
   cpl_msg_info(cpl_func,"----------------------------------------- \n");
    fprintf (midiReportPtr, "\nComputing Calibrated Visibility for batch  %d \n", batchNumber);
    fprintf (midiReportPtr, "----------------------------------------- \n");

    //    Reset parameters
    *error = 0;
    dispResult->trfExists = 0;
    dispResult->calibVisExists = 0;

/* -------------------------------------------------------------------------- */
    prestate = cpl_errorstate_get();   

/*       Grism or Prism? */
    if (strcmp(format->grismId,"GRISM")==0){
       softag = cpl_sprintf("%s", MIDI_TRF_GRISM);
    }
    else if (strcmp(format->grismId,"PRISM")==0){
       softag = cpl_sprintf("%s", MIDI_TRF_PRISM);
    }
    else {
       cpl_msg_warning(cpl_func, "No GRISM or PRISM found");   
       cpl_msg_warning(cpl_func, "No transfer function applied");
       cpl_errorstate_set(prestate); 
       *error = 1;
       return;
    }
    
    cur_frame = cpl_frameset_find( frameset, softag );    

    /* 
       In case no transfer function file was sent (i.e. during online pipeline run) then
       check the tmp dir
    */   
    if( !cur_frame )	  
    {
       /* PIPELINE will not send the trf. Try to load it from tmp */
       dummy_propertylist=cpl_propertylist_load("/tmp/MIDI_trf.fits", 0);	 
       if (dummy_propertylist != NULL)
       {
          local_trf_frame=cpl_frame_new();
          cpl_frame_set_filename (local_trf_frame,"/tmp/MIDI_trf.fits");
          cpl_frame_set_group (local_trf_frame, CPL_FRAME_GROUP_CALIB);
          cpl_frame_set_level (local_trf_frame, CPL_FRAME_LEVEL_FINAL);
          cpl_frame_set_tag (local_trf_frame, cpl_propertylist_get_string(dummy_propertylist, "ESO PRO CATG"));
          cpl_frame_set_type (local_trf_frame, CPL_FRAME_TYPE_TABLE);
          cpl_frameset_insert(frameset, local_trf_frame);
          cpl_propertylist_delete(dummy_propertylist);          
        }
    }

    cur_frame = cpl_frameset_find( frameset, softag );    
    
    if (cur_frame)
    {
       cpl_msg_info(cpl_func, "Transfer function found in the SOF: %s\n",cpl_frame_get_filename(cur_frame));   
    }

    
    if (!cur_frame) {
       cpl_msg_warning(cpl_func, "No siutable transfer function found!");   
       cpl_msg_warning(cpl_func, "No transfer function applied to the data");
       cpl_free(softag);
       cpl_errorstate_set(prestate); 
       *error = 1;
       return;
    }

    cpl_free(softag);
    
    /* Load extension  TRANSFER_FUNCTION */ 
    
    prestate = cpl_errorstate_get();   

    ext_trf=cpl_fits_find_extension(cpl_frame_get_filename(cur_frame),"TRANSFER_FUNCTION");

    if((table = cpl_table_load(cpl_frame_get_filename(cur_frame), ext_trf, 1))==NULL)
    {
       cpl_msg_info(cpl_func, "No siutable table found in the file %s\n",cpl_frame_get_filename(cur_frame));   
       cpl_msg_warning(cpl_func, "No transfer function applied");
       cpl_errorstate_set(prestate);  
       *error = 1;
       return;
    }

        //    Check if number of records is consistent
    dimentrf=cpl_table_get_nrow(table);
    if ( dimentrf != format->iXWidth)
    {
       cpl_msg_warning(cpl_func, "Inconsistent Number of Records found");   
       cpl_msg_warning(cpl_func, "No transfer function applied");
       cpl_errorstate_set(prestate);  
       cpl_table_delete(table);
       *error = 1;
       return;
    }
    
    /* If all is Ok read the values */
    
    if (cpl_table_has_column(table, "TRF") && cpl_table_has_column(table, "TRF_ERROR") )
    {

       for (X=0; X<dimentrf;X++){
          dispResult->trf[X]=cpl_table_get_float(table, "TRF", X, NULL);
          dispResult->trfErr[X]=cpl_table_get_float(table, "TRF_ERROR", X, NULL);
       }
    }
    else{
       cpl_msg_info(cpl_func, "No siutable table column found in the file %s\n",cpl_frame_get_filename(cur_frame));   
       cpl_msg_warning(cpl_func,"No transfer function applied");
       cpl_table_delete(table);
       *error = 1;
       return;
    }


/* Save the fits table  into a fits file */
    qclist=cpl_propertylist_new();    

    /* Save QC parameters from the original file to the new product */ 
    dummy_propertylist = cpl_propertylist_load(cpl_frame_get_filename(cur_frame), 0);
    cpl_propertylist_copy_property_regexp(qclist, dummy_propertylist, "^ESO QC",0);    
    cpl_propertylist_delete(dummy_propertylist);
    
    if (strcmp(format->grismId,"GRISM")==0) {
       cpl_propertylist_update_string(qclist, CPL_DFS_PRO_CATG, "TRF_GRISM");
       dataname=cpl_sprintf("MIDI_trf_grism.fits");
    }
    if (strcmp(format->grismId,"PRISM")==0) {
       cpl_propertylist_update_string(qclist, CPL_DFS_PRO_CATG, "TRF_PRISM");
       dataname=cpl_sprintf("MIDI_trf_prism.fits");
    }
    cpl_propertylist_append_string(qclist, "EXTNAME", "TRANSFER_FUNCTION");
    
    
    cpl_dfs_save_table(frameset, NULL, parlist, frameset, NULL, table, 
                       qclist, "midi_fringe_all",
                       qclist, NULL, 
                       PACKAGE "/" PACKAGE_VERSION,
                       dataname);

    cpl_propertylist_delete(qclist);        
    cpl_free(dataname); 
    cpl_table_delete(table);



/* -------------------------------------------------------------------------- */
    //    Allicate memory
/*     givenGrismId = (char *) calloc (MIN_STRING_LENGTH, sizeof (char)); */

/*     //    Check if Transfer Function is available. Try to open the file for reading */
/*     if (diagnostic)cpl_msg_info(cpl_func,"Trying to extract Transfer Functions from %s \n", fileNames->trfNameRead); */
/*     fprintf (midiReportPtr, "Trying to extract Transfer Functions from %s \n", fileNames->trfNameRead); */
/*     if ((trfPtr = fopen (fileNames->trfNameRead, "r")) == NULL) */
/*     { */
/*         *error = 1; */
/*         midiReportWarning (midiReportPtr, routine, __FILE__, __LINE__, "Cannot open Transfer Function file"); */
/*         free (givenGrismId); */
/*         return; */
/*     } */
    
/*     //    Read Grism ID */
/*     if (fscanf (trfPtr, "%s \n", givenGrismId) == EOF) */
/*     { */
/*         *error = 1; */
/*         midiReportWarning (midiReportPtr, routine, __FILE__, __LINE__, "Cannot read Grism ID"); */
/*         if (trfPtr) fclose (trfPtr); */
/*         free (givenGrismId); */
/*         if (strcmp (format->obsCatg, "SCIENCE") == 0) remove (fileNames->trfNameRead); */
/*         return; */
/*     } */
/*     else */
/*     { */
/*         //    Check if the given Grism ID is the same as that of the current batch */
/*         if (strcmp (format->grismId, givenGrismId) != 0) */
/*         { */
/*             midiReportWarning (midiReportPtr, routine, __FILE__, __LINE__, "Inconsistent Grism ID"); */
/*             if (diagnostic)cpl_msg_info(cpl_func,"Given grism ID          = %s. Expected %s \n", givenGrismId, format->grismId); */
/*             fprintf (midiReportPtr, "Given grism ID          = %s. Expected %s \n", givenGrismId, format->grismId); */
/*             if (trfPtr) fclose (trfPtr); */
/*             free (givenGrismId); */
/*             if (strcmp (format->obsCatg, "SCIENCE") == 0) remove (fileNames->trfNameRead); */
/*             return; */
/*         } */
/*         if (diagnostic)cpl_msg_info(cpl_func,"Given grism ID          = %s. Expected %s \n", givenGrismId, format->grismId); */
/*         fprintf (midiReportPtr, "Given grism ID          = %s. Expected %s \n", givenGrismId, format->grismId); */
/*     } */
    
/*     //    Get number of records */
/*     if (fscanf (trfPtr, "%d \n", &numOfRecords) == EOF) */
/*     { */
/*         *error = 1; */
/*         midiReportWarning (midiReportPtr, routine, __FILE__, __LINE__, "Cannot read Number of Records"); */
/*         if (trfPtr) fclose (trfPtr); */
/*         free (givenGrismId); */
/*         if (strcmp (format->obsCatg, "SCIENCE") == 0) remove (fileNames->trfNameRead); */
/*         return; */
/*     } */
/*     else  */
/*     { */
/*         //    Check if number of records is consistent */
/*         if (numOfRecords != format->iXWidth) */
/*         { */
/*             *error = 1; */
/*             sprintf (midiMessage, "Inconsistent Number of Records from %s. Expected %d",  */
/*                 fileNames->trfNameRead, format->iXWidth);  */
/*             midiReportWarning (midiReportPtr, routine, __FILE__, __LINE__, midiMessage); */
/*             if (diagnostic)cpl_msg_info(cpl_func,"Given Number of Records = %d. Expected %d \n", numOfRecords, format->iXWidth); */
/*             fprintf (midiReportPtr, "Given Number of Records = %d. Expected %d \n", numOfRecords, format->iXWidth); */
/*             if (trfPtr) fclose (trfPtr); */
/*             free (givenGrismId); */
/*             if (strcmp (format->obsCatg, "SCIENCE") == 0) remove (fileNames->trfNameRead); */
/*             return; */
/*         } */
/*         if (diagnostic)cpl_msg_info(cpl_func,"Given Number of Records = %d. Expected %d \n", numOfRecords, format->iXWidth); */
/*         fprintf (midiReportPtr, "Given Number of Records = %d. Expected %d \n", numOfRecords, format->iXWidth); */
/*     } */

/*     //    If all is well then try to load the data into the array */
/*     for (X = 0; X < format->iXWidth; X++) */
/*     { */
/*         if (fscanf (trfPtr, "%f %f", &(dispResult->trf[X]), &(dispResult->trfErr[X])) == EOF) */
/*         { */
/*             *error = 1; */
/*             midiReportWarning (midiReportPtr, routine, __FILE__, __LINE__, "Cannot read Transfer Functions"); */
/*             if (trfPtr) fclose (trfPtr); */
/*             free (givenGrismId); */
/*             if (strcmp (format->obsCatg, "SCIENCE") == 0) remove (fileNames->trfNameRead); */
/*             return; */
/*         } */
/*         fscanf (trfPtr, "\n"); */
/*         if (diagnostic)cpl_msg_info(cpl_func,"Channel %3d = %f %f \n", X, (dispResult->trf)[X], (dispResult->trfErr)[X]); */
/*         fprintf (midiReportPtr, "Channel %3d = %f %f \n", X, (dispResult->trf)[X], (dispResult->trfErr)[X]); */
/*     } */
/*     if (diagnostic)cpl_msg_info(cpl_func,"\nRead %d sets of transfer function values from %s \n", format->iXWidth, fileNames->trfNameRead); */
/*     fprintf (midiReportPtr, "\nRead %d sets of transfer function values from %s \n", format->iXWidth, fileNames->trfNameRead); */
/*     if (trfPtr) fclose (trfPtr); */
/*     free (givenGrismId); */
/*     if (strcmp (format->obsCatg, "SCIENCE") == 0) remove (fileNames->trfNameRead); */


    //    Compute calibrated visibilities
    if (diagnostic)cpl_msg_info(cpl_func,"\nChannel   Calibrated Visibility   Calibrated Visibility Error \n");
    if (diagnostic)cpl_msg_info(cpl_func,"------------------------------------------------------------- \n");
    fprintf (midiReportPtr, "\nChannel   Calibrated Visibility   Calibrated Visibility Error QCLOG \n");
    fprintf (midiReportPtr, "-------------------------------------------------------------   QCLOG\n");
    for (X = 0; X < format->iXWidth; X++)
    {
        if (badChannelList[X] || (dispResult->trf[X] <= 0.0))
        {
            badChannelList[X] |= BSL_DATA_ERROR;
            if (diagnostic)cpl_msg_info(cpl_func,"%3d                  <-- %s --> \n", X, UNAV);
            fprintf (midiReportPtr, "%3d                  <-- %s --> \n", X, UNAV);
            continue;
        }

        (dispResult->calibVis2)[X] = (dispResult->normVis2)[X] / (dispResult->trf)[X];
        (dispResult->calibVis2Err)[X] = (dispResult->normVis2Err)[X] / (fabs ((dispResult->trf)[X])) +
            (dispResult->trfErr)[X] * (fabs ((dispResult->normVis2)[X]) / ((dispResult->trf)[X] * (dispResult->trf)[X]));

        if (diagnostic)cpl_msg_info(cpl_func,"%3d         %5.4f                   %5.4f \n", 
            X, (dispResult->calibVis2)[X], (dispResult->calibVis2Err)[X]);
        fprintf (midiReportPtr, "%3d         %5.4f                   %5.4f QCLOG \n", 
            X, (dispResult->calibVis2)[X], (dispResult->calibVis2Err)[X]);
    }

    if (plotFile)
    {
        midiCreatePlotFile2D ("CalibVisCombined", "Combined Calibrated Visibility", 
            "Channale", "Visibility", 0, dispResult->calibVis2, 0, format->iXWidth, 1, 0);
            
        midiCreatePlotFile2D ("CalibVisErrCombined", "Combined Calibrated Visibility Error", 
            "Channel", "Visibility Error", 0, dispResult->calibVis2Err, 0, format->iXWidth, 1, 0);

        midiCreatePlotFile2D ("NormalizedUncalibVisCombined", "Combined Calibrated Visibility", 
            "Channale", "Visibility", 0, dispResult->calibVis2, 0, format->iXWidth, 1, 0);
            
        midiCreatePlotFile2D ("NormalizedUncalibVisErrCombined", "Combined Calibrated Visibility Error", 
            "Channel", "Visibility Error", 0, dispResult->calibVis2Err, 0, format->iXWidth, 1, 0);
    }
    
    //    Close files
    dispResult->trfExists = 1;
    dispResult->calibVisExists = 1;

    return;
}
/*****************************************************************************/
 
/******************************************************************************
*               European Southern Observatory
*            VLTI MIDI Data Reduction Software
*
* Module name:  computeUncalibVisHSUndisp
* Input/Output: See function arguments to avoid duplication
* Description:  Uses the results of the estimateFringe() together with
*               any other parameters (TBD) and errors and computes
*               an estimate of the visibility. This routine applies to both
*               CALIB and SCIENCE targets.
*
* History:
* 02-Dec-03     (csabet) Created
******************************************************************************/
void computeUncalibVisHSUndisp (
    RawVisibility    *rawVis,
    PhotometryResult *photom)
{

    /*  Local Declarations
    --------------------*/
    const char  routine[] = "computeUncalibVisHSUndisp";
    
    /*  Algorithm
    -----------*/
    if (diagnostic > 4)cpl_msg_info(cpl_func,"Invoking      routine   '%s' \n", routine);
    if (diagnostic > 4) fprintf(midiReportPtr, "Invoking      routine   '%s' \n", routine);

   cpl_msg_info(cpl_func,"\nComputing Uncalibrated Visibilities for batch  %d \n", batchNumber);
   cpl_msg_info(cpl_func,"--------------------------------------------- \n");
    fprintf (midiReportPtr, "\nComputing Uncalibrated Visibilities for batch  %d \n", batchNumber);
    fprintf (midiReportPtr, "--------------------------------------------- \n");

    rawVis->visErr = fabs (rawVis->vis * (photom->chop2ChopAErr[0] + photom->chop2ChopAErr[1] +
                    photom->chop2ChopBErr[0] + photom->chop2ChopBErr[1]) / (8.0 * sqrt(2.0)));
    rawVis->vis1Err = fabs (rawVis->vis1 * (photom->chop2ChopAErr[0] + photom->chop2ChopAErr[1] +
                    photom->chop2ChopBErr[0] + photom->chop2ChopBErr[1]) / 8.0);
    rawVis->vis2Err = fabs (rawVis->vis2 * (photom->chop2ChopAErr[0] + photom->chop2ChopAErr[1] +
                    photom->chop2ChopBErr[0] + photom->chop2ChopBErr[1]) / 8.0);
    rawVis->visSqrdErr = 2.0 * (rawVis->visErr * fabs (rawVis->vis));
    rawVis->visSqrd1Err = 2.0 * (rawVis->vis1Err * fabs (rawVis->vis1));
    rawVis->visSqrd2Err = 2.0 * (rawVis->vis2Err * fabs (rawVis->vis2));

   cpl_msg_info(cpl_func,"Uncalibrated Visibility         region 0       = %f \n", rawVis->vis);
   cpl_msg_info(cpl_func,"Uncalibrated Visibility         region 0 Error = %f \n", rawVis->visErr);
   cpl_msg_info(cpl_func,"Uncalibrated Visibility         region 1       = %f \n", rawVis->vis1);
   cpl_msg_info(cpl_func,"Uncalibrated Visibility         region 1 Error = %f \n", rawVis->vis1Err);
   cpl_msg_info(cpl_func,"Uncalibrated Visibility         region 2       = %f \n", rawVis->vis2);
   cpl_msg_info(cpl_func,"Uncalibrated Visibility         region 2 Error = %f \n", rawVis->vis2Err);
   cpl_msg_info(cpl_func,"Uncalibrated Squared Visibility region 0       = %f \n", rawVis->visSqrd);
   cpl_msg_info(cpl_func,"Uncalibrated Squared Visibility region 0 Error = %f \n", rawVis->visSqrdErr);
   cpl_msg_info(cpl_func,"Uncalibrated Squared Visibility region 1       = %f \n", rawVis->visSqrd1);
   cpl_msg_info(cpl_func,"Uncalibrated Squared Visibility region 1 Error = %f \n", rawVis->visSqrd1Err);
   cpl_msg_info(cpl_func,"Uncalibrated Squared Visibility region 2       = %f \n", rawVis->visSqrd2);
   cpl_msg_info(cpl_func,"Uncalibrated Squared Visibility region 2 Error = %f \n", rawVis->visSqrd2Err);

    fprintf (midiReportPtr, "Uncalibrated Visibility         region 0       = %f \n", rawVis->vis);
    fprintf (midiReportPtr, "Uncalibrated Visibility         region 0 Error = %f \n", rawVis->visErr);
    fprintf (midiReportPtr, "Uncalibrated Visibility         region 1       = %f \n", rawVis->vis1);
    fprintf (midiReportPtr, "Uncalibrated Visibility         region 1 Error = %f \n", rawVis->vis1Err);
    fprintf (midiReportPtr, "Uncalibrated Visibility         region 2       = %f \n", rawVis->vis2);
    fprintf (midiReportPtr, "Uncalibrated Visibility         region 2 Error = %f \n", rawVis->vis2Err);
    fprintf (midiReportPtr, "Uncalibrated Squared Visibility region 0       = %f \n", rawVis->visSqrd);
    fprintf (midiReportPtr, "Uncalibrated Squared Visibility region 0 Error = %f \n", rawVis->visSqrdErr);
    fprintf (midiReportPtr, "Uncalibrated Squared Visibility region 1       = %f \n", rawVis->visSqrd1);
    fprintf (midiReportPtr, "Uncalibrated Squared Visibility region 1 Error = %f \n", rawVis->visSqrd1Err);
    fprintf (midiReportPtr, "Uncalibrated Squared Visibility region 2       = %f \n", rawVis->visSqrd2);
    fprintf (midiReportPtr, "Uncalibrated Squared Visibility region 2 Error = %f \n", rawVis->visSqrd2Err);

    return;
}
/*****************************************************************************/


/******************************************************************************
*               European Southern Observatory
*            VLTI MIDI Data Reduction Software
*
* Module name:  computeCalibVisHSUndisp
* Input/Output: See function arguments to avoid duplication
* Description:  This routine computes the final visibility on SCIENCE targets. It
*               requires the transfer function as well as the uncalibrated
*               visibilities.
*
* History:
* 21-Jul-03     (csabet) Created
******************************************************************************/
void computeCalibVisHSUndisp (
    MidiFiles                *fileNames,    // In: Pointer to MIDI file structure
    RawVisibility            *rawVis,    // In: Raw visibilities
    TransferFunction        *trf,        // In: Transfer Functions
    CalibratedVisibility    *calibVis,    // Ou: Calibrated visibilities
    int                        *error)        // Ou: Status
{

    /*  Local Declarations
    --------------------*/
    const char  routine[] = "computeCalibVisHSUndisp";
    FILE        *trfPtr;
    int         i;

    /*  Algorithm
    -----------*/
    if (diagnostic > 4)cpl_msg_info(cpl_func,"Invoking      routine   '%s' \n", routine);
    if (diagnostic > 4) fprintf(midiReportPtr, "Invoking      routine   '%s' \n", routine);

   cpl_msg_info(cpl_func,"\nComputing Calibrated Visibility for batch  %d \n", batchNumber);
   cpl_msg_info(cpl_func,"----------------------------------------- \n");
    fprintf (midiReportPtr, "\nComputing Calibrated Visibility for batch  %d \n", batchNumber);
    fprintf (midiReportPtr, "----------------------------------------- \n");

    /* Reset parameters */
    *error = 0;
    trf->exists = 0;
    calibVis->exists = 0;

    /* Open the transfer function file for reading */
    if ((trfPtr = fopen (fileNames->trfNameRead, "r")) == NULL)
    {
        midiReportWarning (midiReportPtr, routine, __FILE__, __LINE__, "Cannot open Transfer Functions file");
        *error = 1;
        return;
    }

    /* Read the transfer fuctions from the file */
    for (i = 0; i < 6; i++)
    {
        if (fscanf (trfPtr, "%f\n", &(trf->trf[i])) == EOF)
        {
            midiReportWarning (midiReportPtr, routine, __FILE__, __LINE__, "Cannot read Transfer Functions");
            *error = 1;
            return;
        }
        if (fscanf (trfPtr, "%f\n", &(trf->trfErr[i])) == EOF)
        {
            midiReportWarning (midiReportPtr, routine, __FILE__, __LINE__, "Cannot read Transfer Functions");
            *error = 1;
            return;
        }
    }

    /* Compute calibrated visibilities */
    if (trf->trf[0] == 0 || trf->trfErr[0] == 0 ||
        trf->trf[1] == 0 || trf->trfErr[1] == 0 ||
        trf->trf[2] == 0 || trf->trfErr[2] == 0 ||
        trf->trf[3] == 0 || trf->trfErr[3] == 0 ||
        trf->trf[4] == 0 || trf->trfErr[4] == 0 ||
        trf->trf[5] == 0 || trf->trfErr[5] == 0)
    {
        midiReportWarning (midiReportPtr, routine, __FILE__, __LINE__, 
            "A singularity encountered. Cannot compute Calibrated Visibilities");
        *error = 1;
        return;
    }
    else
    {
        calibVis->vis[0] = rawVis->vis / trf->trf[0];
        calibVis->visErr[0] = rawVis->visErr / (fabs (trf->trf[0])) +
            trf->trfErr[0] * (fabs (rawVis->vis) / (trf->trf[0] * trf->trf[0]));

        calibVis->vis[1] = rawVis->vis1 / trf->trf[1];
        calibVis->visErr[1] = rawVis->vis1Err / (fabs (trf->trf[1])) +
            trf->trfErr[1] * (fabs (rawVis->vis1) / (trf->trf[1] * trf->trf[1]));

        calibVis->vis[2] = rawVis->vis2 / trf->trf[2];
        calibVis->visErr[2] = rawVis->vis2Err / (fabs (trf->trf[2])) +
            trf->trfErr[2] * (fabs (rawVis->vis2) / (trf->trf[2] * trf->trf[2]));

        calibVis->visSqrd[0] = rawVis->visSqrd / trf->trf[3];
        calibVis->visSqrdErr[0] = rawVis->visSqrdErr / (fabs (trf->trf[3])) +
            trf->trfErr[3] * (fabs (rawVis->visSqrd) / (trf->trf[3] * trf->trf[3]));

        calibVis->visSqrd[1] = rawVis->visSqrd1 / trf->trf[4];
        calibVis->visSqrdErr[1] = rawVis->visSqrd1Err / (fabs (trf->trf[4])) +
            trf->trfErr[4] * (fabs (rawVis->visSqrd1) / (trf->trf[4] * trf->trf[4]));

        calibVis->visSqrd[2] = rawVis->visSqrd2 / trf->trf[5];
        calibVis->visSqrdErr[2] = rawVis->visSqrd2Err / (fabs (trf->trf[5])) +
            trf->trfErr[5] * (fabs (rawVis->visSqrd2) / (trf->trf[5] * trf->trf[5]));
    }

    /* Report results and close file */
   cpl_msg_info(cpl_func,"Calibrated Visibility         region 0       = %f \n", calibVis->vis[0]);
   cpl_msg_info(cpl_func,"Calibrated Visibility         region 0 Error = %f \n", calibVis->visErr[0]);
   cpl_msg_info(cpl_func,"Calibrated Visibility         region 1       = %f \n", calibVis->vis[1]);
   cpl_msg_info(cpl_func,"Calibrated Visibility         region 1 Error = %f \n", calibVis->visErr[1]);
   cpl_msg_info(cpl_func,"Calibrated Visibility         region 2       = %f \n", calibVis->vis[2]);
   cpl_msg_info(cpl_func,"Calibrated Visibility         region 2 Error = %f \n", calibVis->visErr[2]);
   cpl_msg_info(cpl_func,"Calibrated Squared Visibility region 0       = %f \n", calibVis->visSqrd[0]);
   cpl_msg_info(cpl_func,"Calibrated Squared Visibility region 0 Error = %f \n", calibVis->visSqrdErr[0]);
   cpl_msg_info(cpl_func,"Calibrated Squared Visibility region 1       = %f \n", calibVis->visSqrd[1]);
   cpl_msg_info(cpl_func,"Calibrated Squared Visibility region 1 Error = %f \n", calibVis->visSqrdErr[1]);
   cpl_msg_info(cpl_func,"Calibrated Squared Visibility region 2       = %f \n", calibVis->visSqrd[2]);
   cpl_msg_info(cpl_func,"Calibrated Squared Visibility region 2 Error = %f \n", calibVis->visSqrdErr[2]);

    fprintf (midiReportPtr, "Calibrated Visibility         region 0       = %f (QCLOG)\n", calibVis->vis[0]);
    fprintf (midiReportPtr, "Calibrated Visibility         region 0 Error = %f (QCLOG)\n", calibVis->visErr[0]);
    fprintf (midiReportPtr, "Calibrated Visibility         region 1       = %f (QCLOG)\n", calibVis->vis[1]);
    fprintf (midiReportPtr, "Calibrated Visibility         region 1 Error = %f (QCLOG)\n", calibVis->visErr[1]);
    fprintf (midiReportPtr, "Calibrated Visibility         region 2       = %f (QCLOG)\n", calibVis->vis[2]);
    fprintf (midiReportPtr, "Calibrated Visibility         region 2 Error = %f (QCLOG)\n", calibVis->visErr[2]);
    fprintf (midiReportPtr, "Calibrated Squared Visibility region 0       = %f (QCLOG)\n", calibVis->visSqrd[0]);
    fprintf (midiReportPtr, "Calibrated Squared Visibility region 0 Error = %f (QCLOG)\n", calibVis->visSqrdErr[0]);
    fprintf (midiReportPtr, "Calibrated Squared Visibility region 1       = %f (QCLOG)\n", calibVis->visSqrd[1]);
    fprintf (midiReportPtr, "Calibrated Squared Visibility region 1 Error = %f (QCLOG)\n", calibVis->visSqrdErr[1]);
    fprintf (midiReportPtr, "Calibrated Squared Visibility region 2       = %f (QCLOG)\n", calibVis->visSqrd[2]);
    fprintf (midiReportPtr, "Calibrated Squared Visibility region 2 Error = %f (QCLOG)\n", calibVis->visSqrdErr[2]);

    fclose (trfPtr);
    remove (fileNames->trfNameRead);
    trf->exists = 1;
    calibVis->exists = 1;

    return;
}
/*****************************************************************************/

 
/******************************************************************************
*               European Southern Observatory
*            VLTI MIDI Data Reduction Software
*
* Module name:  computeUncalibVis
* Input/Output: See function arguments to avoid duplication
* Description:  Computes DISPERSED Uncalibrated Normalized Visibility
*
* History:
* 21-Oct-04     (JM) 
* 21-Jan-05     (csabet) Cleaned up and Integrated
******************************************************************************/
void computeUncalibVis (
    ImageFormat        *format,    // In: General data format
    float            *freqFound,    // IO: Wavelength calibration
    DispersedResult    *dispResult,// In: Dispersed result structure
    int                *error)        // Ou: Error status
{
 
    /*  Local Declarations
    --------------------*/
    const char  routine[] = "computeUncalibVis";
    int         X, n;      
    float         errfromvis, neterr;
    double         phot, photA, photB;
    
    /*  Algorithm
    -----------*/
    if (diagnostic > 4)cpl_msg_info(cpl_func,"Invoking      routine   '%s' \n", routine);
    if (diagnostic > 4) fprintf(midiReportPtr, "Invoking      routine   '%s' \n", routine);
   cpl_msg_info(cpl_func,"\nComputing Normalized Uncalibrated Visibility for batch  %d \n", batchNumber);
   cpl_msg_info(cpl_func,"------------------------------------------------------ \n");
    fprintf (midiReportPtr, "\nComputing Normalized Uncalibrated Visibility for batch  %d \n", batchNumber);
    fprintf (midiReportPtr, "------------------------------------------------------ \n");

    //    Reset
    *error = 0;
        
    //    First for photometry. For SCI_PHOT each photometry has one region to process
    for (X = 0; X < format->iXWidth; X++)
    {
        if (badChannelList[X])
            continue;
            
        if (strcmp (format->beamCombiner, "SCI_PHOT") == 0)
        {
            phot = 4.F * (dispResult->photomA)[0][X] * (dispResult->photomB)[0][X];
            (dispResult->photomNet2Err)[X] = sqrt(sqr((dispResult->photomAErr)[0][X] /
                (dispResult->photomA)[0][X]) + sqr((dispResult->photomBErr)[0][X] /
                (dispResult->photomB)[0][X]));
        }
        else
        {
            photA = (dispResult->photomA)[0][X] + (dispResult->photomA)[1][X];
            photB = (dispResult->photomB)[0][X] + (dispResult->photomB)[1][X];
            phot = 4.F*photA*photB;
            (dispResult->photomNet2Err)[X] = sqrt((sqr((dispResult->photomAErr)[0][X]) +
                sqr((dispResult->photomAErr)[1][X])) / sqr(photA) +
                (sqr((dispResult->photomBErr)[0][X]) +
                sqr((dispResult->photomBErr)[1][X])) / sqr(photB));
        }

        if (phot > 0.F) 
            dispResult->photomNet2[X] = phot;    // do not ever insert a negative value here!!
        else
            dispResult->photomNet2Err[X] = 0.F;// set the err to zero when the photom itself is invalid.        
    }

    //    Now compute the visibility^2
    n = 0;
    for (X = 0; X < format->iXWidth; X++)
    {
        if (badChannelList[X])
            continue;
            
        if ((dispResult->photomNet2)[X] > 0.F)
            (dispResult->normVis2)[X] = (dispResult->rawVis2)[X] / (dispResult->photomNet2)[X];
        else
            (dispResult->normVis2)[X] = 0.F;
            
        if ((dispResult->normVis2)[X] <= 0.0)
        {
            (dispResult->normVis2)[X] = 0.F;
            badChannelList[X] |= BSL_DATA_ERROR;
        }
        else
            n++;
    }

    if (!n)
    {
        midiReportWarning (midiReportPtr, routine, __FILE__, __LINE__, "Cannot find any good visibility range");
        *error = 1;
        return;
    }

    if (diagnostic)cpl_msg_info(cpl_func,"\nChannel   Freq / Wave    Net photometry   Visibility^2    RMS Photometry     Errors percent \n");
    fprintf (midiReportPtr, "\nChannel   Freq / Wave    Net photometry   Visibility^2    RMS Photometry     Errors percent       QCLOG \n");
    if (diagnostic)cpl_msg_info(cpl_func,"         THz  micron       ADU/frame       normalized      Error percent     from Vis     Net \n");
    fprintf (midiReportPtr, "         THz  micron       ADU/frame       normalized      Error percent     from Vis     Net     QCLOG \n");

    for (X = 0; X < format->iXWidth; X++)
    {
        if (badChannelList[X])
        {
            if (diagnostic) cpl_msg_info(cpl_func,"%3d    %6.3f   %6.3f               <-- %s --> \n", 
                X, 299.8F/freqFound[X], freqFound[X], UNAV);
            fprintf (midiReportPtr, "%3d    %6.3f   %6.3f               <-- %s -->                       QCLOG \n", 
                X, 299.8F/freqFound[X], freqFound[X], UNAV);
            continue;
        }

        if (diagnostic) cpl_msg_info(cpl_func,"%3d ", X);    // Tell the column and region....
        fprintf (midiReportPtr, "%3d ", X);        // Tell the column and region....
 
                
        if (diagnostic)cpl_msg_info(cpl_func,"   %6.3f   %6.3f", 299.8F/freqFound[X], freqFound[X]);
        if (diagnostic)cpl_msg_info(cpl_func,"   %9.3f", sqrt((dispResult->photomNet2)[X]));
        if (diagnostic)cpl_msg_info(cpl_func,"           %7.4f", (dispResult->normVis2)[X]);
        if (diagnostic)cpl_msg_info(cpl_func,"           %7.2f", 100.F*(dispResult->photomNet2Err)[X]);
        fprintf (midiReportPtr, "   %6.3f   %6.3f", 299.8F/freqFound[X], freqFound[X]);
        fprintf (midiReportPtr, "   %9.3f", sqrt((dispResult->photomNet2)[X]));
        fprintf (midiReportPtr, "           %7.4f", (dispResult->normVis2)[X]);
        fprintf (midiReportPtr, "           %7.2f", 100.F*(dispResult->photomNet2Err)[X]);
        errfromvis = (dispResult->rawVis2Err)[X] / (dispResult->rawVis2)[X]; // This must be improved. csabet 02-02-05
        neterr = sqrt(sqr(errfromvis) + sqr((dispResult->photomNet2Err)[X]));
        (dispResult->normVis2Err)[X] = neterr;     // This must be improved. csabet 07-02-05
        if (diagnostic)cpl_msg_info(cpl_func,"       %8.2f", 100.F*errfromvis);
        if (diagnostic)cpl_msg_info(cpl_func,"       %8.2f", 100.F*neterr);
        fprintf (midiReportPtr, "       %8.2f", 100.F*errfromvis);
        fprintf (midiReportPtr, "       %8.2f", 100.F*neterr);

        if (diagnostic)cpl_msg_info(cpl_func,"\n");
        fprintf (midiReportPtr, "     QCLOG \n");
    }

    if (plotFile)
    {
        midiCreatePlotFile2D ("PhotomNetCombined", "Combined Net Photometry", 
            "Channel", "Photometry", 0, dispResult->photomNet2, 0, format->iXWidth, 1, 0);
        midiCreatePlotFile2D ("PhotomNetErrCombined", "Combined Net Photometry Error", 
            "Channel", "Photometry Error", 0, dispResult->photomNet2Err, 0, format->iXWidth, 1, 0);
        midiCreatePlotFile2D ("NormalizedUncalibVisCombined", "Combined Normalized Uncalibrated Visibility", 
            "Channel", "Visibility", 0, dispResult->normVis2, 0, format->iXWidth, 1, 0);
        midiCreatePlotFile2D ("NormalizedUncalibVisErrCombined", "Combined Normalized Uncalibrated Visibility Error", 
            "Channel", "Visibility Error", 0, dispResult->normVis2Err, 0, format->iXWidth, 1, 0);
    }

    //    Compute Binned Uncalibrated Visibilities
    computeBinnedUncalibVis (format, dispResult, error);
    if (*error)
    {
        midiReportWarning (midiReportPtr, routine, __FILE__, __LINE__, "Cannot compute Binned Uncalibrated visibilities");
        return;
    }
    
    return;
}
/*****************************************************************************/


/******************************************************************************
*               European Southern Observatory
*            VLTI MIDI Data Reduction Software
*
* Module name:  normalizeVis
* Input/Output: See function arguments to avoid duplication
* Description:
*
*
* History:
* 21-Jul-03     (csabet) Created routine template
******************************************************************************/
void normalizeVis (
    RawVisibility    *rawVis,    // UNnormalized visibilities, get changed to normalized thru pointer
    float           photomA[],    // Photometry of A telescope, each region
    float           photomB[])    // Photometry of B telescope, each region
    
{    /*  Local Declarations
    --------------------*/
    const char  routine[] = "normalizeVis";

    /*  Algorithm
    -----------*/
    if (diagnostic > 4)cpl_msg_info(cpl_func,"Invoking      routine   '%s'  \n", routine);
    if (diagnostic > 4) fprintf(midiReportPtr, "Invoking      routine   '%s'  \n", routine);
    
// Assumes scanmix is equal between regions
    if (((photomA[0] + photomA[1]) * (photomB[0] + photomB[1])) != 0) /* Singularity check */ /* Added csabet 09-Jan-04 */
        rawVis->visSqrd /= (photomA[0] + photomA[1])*(photomB[0] + photomB[1]) ;
    else
        rawVis->visSqrd = NOT_A_NUMBER;

    if ((photomA[0] * photomB[0]) != 0) /* Singularity check */ /* Added csabet 09-Jan-04 */
        rawVis->visSqrd1 /= (photomA[0]*photomB[0]) ;
    else
        rawVis->visSqrd1 = NOT_A_NUMBER;

    if ((photomA[1] * photomB[1]) != 0)  /* Singularity check */ /* Added csabet 09-Jan-04 */
        rawVis->visSqrd2 /= (photomA[1]*photomB[1]) ;
    else
        rawVis->visSqrd2 = NOT_A_NUMBER;

    if (((photomA[0] + photomA[1]) * (photomB[0] + photomB[1])) > 0)  /* Singularity check */ /* Added csabet 09-Jan-04 */
        rawVis->vis /= (float) sqrt((photomA[0] + photomA[1])*(photomB[0] + photomB[1])) ;
    else
        rawVis->vis = NOT_A_NUMBER;

    if ((photomA[0] * photomB[0]) > 0)  /* Singularity check */ /* Added csabet 09-Jan-04 */
        rawVis->vis1 /= (float) sqrt(photomA[0]*photomB[0])  ;
    else
        rawVis->vis1 = NOT_A_NUMBER;

    if ((photomA[1] * photomB[1]) > 0)  /* Singularity check */ /* Added csabet 09-Jan-04 */
        rawVis->vis2 /= (float) sqrt(photomA[1]*photomB[1]) ;
    else
        rawVis->vis2 = NOT_A_NUMBER;

    /* Added csabet 09-Jan-04 */
   cpl_msg_info(cpl_func,
        "Visibilities       combined    I1          I2 \n"
        "--------------------------------------------- \n"
        "Vis amplitude:     %5.4f      %5.4f     %5.4f \n"
        "Vis^2:             %5.4f      %5.4f     %5.4f \n"
        "sqrt(Vis^2):       %5.4f      %5.4f     %5.4f \n",
        rawVis->vis, rawVis->vis1, rawVis->vis2,
        rawVis->visSqrd, rawVis->visSqrd1, rawVis->visSqrd2,
        sqrtp(rawVis->visSqrd), sqrtp(rawVis->visSqrd1), sqrtp(rawVis->visSqrd2));

    fprintf (midiReportPtr,
        "Visibilities       combined    I1          I2 QCLOG \n"
        "--------------------------------------------- \n"
        "Vis amplitude:     %5.4f      %5.4f     %5.4f QCLOG \n"
        "Vis^2:             %5.4f      %5.4f     %5.4f QCLOG \n"
        "sqrt(Vis^2):       %5.4f      %5.4f     %5.4f QCLOG \n",
        rawVis->vis, rawVis->vis1, rawVis->vis2,
        rawVis->visSqrd, rawVis->visSqrd1, rawVis->visSqrd2,
        sqrtp(rawVis->visSqrd), sqrtp(rawVis->visSqrd1), sqrtp(rawVis->visSqrd2));

    if ((rawVis->vis == NOT_A_NUMBER) || (rawVis->vis1 == NOT_A_NUMBER) || (rawVis->vis2 == NOT_A_NUMBER) ||
        (rawVis->visSqrd == NOT_A_NUMBER) || (rawVis->visSqrd1 == NOT_A_NUMBER) || (rawVis->visSqrd2 == NOT_A_NUMBER) )
    {
        sprintf (midiMessage, "A value of  %.4f  is a reject result. QCLOG", (float)NOT_A_NUMBER);
        midiReportWarning (midiReportPtr, routine, __FILE__, __LINE__, midiMessage);
    }
    
    return;
}
/*****************************************************************************/


/******************************************************************************
*                European Southern Observatory
*              VLTI MIDI Data Reduction Software 
*
* Module name:  computeBinnedUncalibVis
* Input/Output: See function arguments to avoid duplication
* Description:  This routine computes the uncalibrated visibilities at 5 spectral
*                bands as follows:
*
*                1)  8.64 microns (Channels 120 -> 126) PRISM
*                                 (Channels  38 ->  50) GRISM
*                2)  9.00 microns (Channels 113 -> 121) PRISM
*                                 (Channels  52 ->  68) GRISM
*                3) 10.46 microns (Channels  88 ->  99) PRISM
*                                 (Channels 116 -> 136) GRISM
*                4) 11.79 microns (Channels  60 ->  72) PRISM
*                                 (Channels 176 -> 200) GRISM
*                5) 12.80 microns (Channels  36 ->  50) PRISM
*                                 (Channels 217 -> 245) GRISM
*
* History:        
* 21-Sep-06        (csabet) Created
******************************************************************************/
void computeBinnedUncalibVis (
    ImageFormat        *format,        // In: General data format
    DispersedResult    *dispResult,    // In: Dispersed result structure
    int                *error)            // Ou: Error status
{

    //    Local Declarations
    //    ------------------
    const char    routine[] = "computeBinnedUncalibVis";
    int            i, n;

    //    Algorithm
    //    ---------
   cpl_msg_info(cpl_func,"\nBinned Uncalibrated Visibilities for batch  %d \n", batchNumber);
   cpl_msg_info(cpl_func,"------------------------------------------ \n");
    fprintf (midiReportPtr, "\nBinned Uncalibrated Visibilities for batch  %d QCLOG \n", batchNumber);
    fprintf (midiReportPtr, "------------------------------------------ QCLOG \n");

    //    Reset parameters
    *error = 0;
    n = 0;

    dispResult->uncalVisBinned1 = 0.0;
    dispResult->uncalVisBinned2 = 0.0;
    dispResult->uncalVisBinned3 = 0.0;
    dispResult->uncalVisBinned4 = 0.0;
    dispResult->uncalVisBinned5 = 0.0;

    if (strcmp(format->grismId, "PRISM") == 0)
    {
        n = 0;
        for (i = 36; i < 51; i++)
        {
            if (badChannelList[i])
                continue;
                
            dispResult->uncalVisBinned5 += (dispResult->normVis2)[i];
            n++;
        }
        dispResult->uncalVisBinned5 /= n;
        n = 0;
        for (i = 60; i < 73; i++)
        {
            if (badChannelList[i])
                continue;
                
            dispResult->uncalVisBinned4 += (dispResult->normVis2)[i];
            n++;
        }
        dispResult->uncalVisBinned4 /= n;
        n = 0;
        for (i = 88; i < 99; i++)
        {
            if (badChannelList[i])
                continue;
                
            dispResult->uncalVisBinned3 += (dispResult->normVis2)[i];
            n++;
        }
        dispResult->uncalVisBinned3 /= n;
        n = 0;
        for (i = 113; i < 122; i++)
        {
            if (badChannelList[i])
                continue;
                
            dispResult->uncalVisBinned2 += (dispResult->normVis2)[i];
            n++;
        }
        dispResult->uncalVisBinned2 /= n;
        n = 0;
        for (i = 120; i < 127; i++)
        {
            if (badChannelList[i])
                continue;
                
            dispResult->uncalVisBinned1 += (dispResult->normVis2)[i];
            n++;
        }
        dispResult->uncalVisBinned1 /= n;
    }
    else if (strcmp(format->grismId, "GRISM") == 0)
    {
        n = 0;
        for (i = 38; i < 51; i++)
        {
            if (badChannelList[i])
                continue;
                
            dispResult->uncalVisBinned1 += (dispResult->normVis2)[i];
            n++;
        }
        dispResult->uncalVisBinned1 /= n;
        n = 0;
        for (i = 52; i < 69; i++)
        {
            if (badChannelList[i])
                continue;
                
            dispResult->uncalVisBinned2 += (dispResult->normVis2)[i];
            n++;
        }
        dispResult->uncalVisBinned2 /= n;
        n = 0;
        for (i = 116; i < 137; i++)
        {
            if (badChannelList[i])
                continue;
                
            dispResult->uncalVisBinned3 += (dispResult->normVis2)[i];
            n++;
        }
        dispResult->uncalVisBinned3 /= n;
        n = 0;
        for (i = 176; i < 201; i++)
        {
            if (badChannelList[i])
                continue;
                
            dispResult->uncalVisBinned4 += (dispResult->normVis2)[i];
            n++;
        }
        dispResult->uncalVisBinned4 /= n;
        n = 0;
        for (i = 217; i < 246; i++)
        {
            if (badChannelList[i])
                continue;
                
            dispResult->uncalVisBinned5 += (dispResult->normVis2)[i];
            n++;
        }
        dispResult->uncalVisBinned5 /= n;
    }
    else
    {
        sprintf (midiMessage, "Unknown GRISM ID ... %s", format->grismId);
        midiReportWarning (midiReportPtr, routine, __FILE__, __LINE__, midiMessage);
        *error = 1;
        return;
    }

    //    Report
   cpl_msg_info(cpl_func,"Grism ID: %s \n", format->grismId);
    fprintf (midiReportPtr, "Grism ID: %s QCLOG \n", format->grismId);
    if (strcmp(format->grismId, "PRISM") == 0)
    {
       cpl_msg_info(cpl_func," 8.64 micron (Channel 120 -> 126) = %0.4f \n", dispResult->uncalVisBinned1);
       cpl_msg_info(cpl_func," 9.00 micron (Channel 113 -> 121) = %0.4f \n", dispResult->uncalVisBinned2);
       cpl_msg_info(cpl_func,"10.46 micron (Channel  88 ->  98) = %0.4f \n", dispResult->uncalVisBinned3);
       cpl_msg_info(cpl_func,"11.79 micron (Channel  60 ->  72) = %0.4f \n", dispResult->uncalVisBinned4);
       cpl_msg_info(cpl_func,"12.80 micron (Channel  36 ->  50) = %0.4f \n", dispResult->uncalVisBinned5);

        fprintf (midiReportPtr, " 8.64 micron (Channel 120 -> 126) = %0.4f QCLOG \n", dispResult->uncalVisBinned1);
        fprintf (midiReportPtr, " 9.00 micron (Channel 113 -> 121) = %0.4f QCLOG \n", dispResult->uncalVisBinned2);
        fprintf (midiReportPtr, "10.46 micron (Channel  88 ->  98) = %0.4f QCLOG \n", dispResult->uncalVisBinned3);
        fprintf (midiReportPtr, "11.79 micron (Channel  60 ->  72) = %0.4f QCLOG \n", dispResult->uncalVisBinned4);
        fprintf (midiReportPtr, "12.80 micron (Channel  36 ->  50) = %0.4f QCLOG \n", dispResult->uncalVisBinned5);
    }
    else
    {
       cpl_msg_info(cpl_func," 8.64 micron (Channel  38 ->  50) = %0.4f \n", dispResult->uncalVisBinned1);
       cpl_msg_info(cpl_func," 9.00 micron (Channel  52 ->  68) = %0.4f \n", dispResult->uncalVisBinned2);
       cpl_msg_info(cpl_func,"10.46 micron (Channel 116 -> 136) = %0.4f \n", dispResult->uncalVisBinned3);
       cpl_msg_info(cpl_func,"11.79 micron (Channel 176 -> 200) = %0.4f \n", dispResult->uncalVisBinned4);
       cpl_msg_info(cpl_func,"12.80 micron (Channel 217 -> 245) = %0.4f \n", dispResult->uncalVisBinned5);

        fprintf (midiReportPtr, " 8.64 micron (Channel  38 ->  50) = %0.4f QCLOG \n", dispResult->uncalVisBinned1);
        fprintf (midiReportPtr, " 9.00 micron (Channel  52 ->  68) = %0.4f QCLOG \n", dispResult->uncalVisBinned2);
        fprintf (midiReportPtr, "10.46 micron (Channel 116 -> 136) = %0.4f QCLOG \n", dispResult->uncalVisBinned3);
        fprintf (midiReportPtr, "11.79 micron (Channel 176 -> 200) = %0.4f QCLOG \n", dispResult->uncalVisBinned4);
        fprintf (midiReportPtr, "12.80 micron (Channel 217 -> 245) = %0.4f QCLOG \n", dispResult->uncalVisBinned5);
    }
    
    return;
}
/*****************************************************************************/

