/* $Id: midi_kappamatrix.c,v 1.31 2012-04-10 14:01:39 agabasch Exp $
 *
 * This file is part of the MIDI Pipeline
 * Copyright (C) 2002,2003 European Southern Observatory
 *
 * This program is free software; you can redistribute it and/or modify
 * it under the terms of the GNU General Public License as published by
 * the Free Software Foundation; either version 2 of the License, or
 * (at your option) any later version.
 *
 * This program is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 * GNU General Public License for more details.
 *
 * You should have received a copy of the GNU General Public License
 * along with this program; if not, write to the Free Software
 * Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA  02111-1307  USA
 */

/*
 * $Author: agabasch $
 * $Date: 2012-04-10 14:01:39 $
 * $Revision: 1.31 $
 * $Name: not supported by cvs2svn $
 */

#ifdef HAVE_CONFIG_H
#include <config.h>
#endif

/*-----------------------------------------------------------------------------
                                Includes
 -----------------------------------------------------------------------------*/

#include <cpl.h>

#include "midi_utils.h"
#include "midi_pfits.h"
#include "midi_dfs.h"
#include "string.h"
#include "midiTableToFits.h"
#include "midi_cplupgrade.h"

/*-----------------------------------------------------------------------------
                            Private function prototypes
 -----------------------------------------------------------------------------*/

static int midi_kappamatrix_create(cpl_plugin *);
static int midi_kappamatrix_exec(cpl_plugin *);
static int midi_kappamatrix_destroy(cpl_plugin *);
static int midi_kappamatrix(cpl_frameset *, const cpl_parameterlist *);
static int table_to_imglst_sky_target(const char * ,
                           const char * ,
                           cpl_imagelist * ,
                           cpl_imagelist * ,
                           cpl_table *);
static int table_to_imglst_mask(const char *,
                                cpl_imagelist *,
                                cpl_table *);
static void kappa_to_ascii(const cpl_image * , 
                          const cpl_image * ,
                          const cpl_image * ,
                          const cpl_image * ,
                          const char * );




/*-----------------------------------------------------------------------------
                            Static variables
 -----------------------------------------------------------------------------*/

static char midi_kappamatrix_description[] =
"This recipe derives the Kappa Matrix in the SCI_PHOT mode following\n"
"the definition in A&A 425 1161. It uses a set of AOPEN and BOPEN\n"
"photometry files as well as a spatial profile of the spectrum. The\n"
"spectrum is extracted using the Optimal Extraction algorithm\n" 
"(Horne 1986, PASP 98 209) adapted to NIR data, i.e. background dominated\n"
"images. The errors of the kappa matrix are not calculated but set a\n"
"priori to 5 per cent. "
"The main output is a fits table with the PRO\n"
"CATG keyword set to MIDI_KAPPAMATRIX_(G|P)RISM. For diagnostic reasons the\n"
"different kappa matrixes are also saved as single fits images\n\n"


"Input files:\n\n"
"  DO category:                Type:       Explanation:         Required:\n"
"  PHOTOM_SP_CALIB             Raw         Raw data frame            \n"
"  or                                                               Y\n"
"  PHOTOM_SP_SCIENCE           Raw         Raw data frame          \n\n"
"  KAPPA_SP_MASK_PRISM         Calib       Spectral profile          \n"
"  or                                                               Y\n"
"  KAPPA_SP_MASK_GRISM         Calib       Spectral profile        \n\n"
"Output files:\n\n"
"  DO category:                Data type:  Explanation:\n"
"  MIDI_KAPPAMATRIX_PRISM      FITS table  Kappa matrix: Main product  \n"
"  or                                                                  \n"
"  MIDI_KAPPAMATRIX_GRISM      FITS table  Kappa matrix: Main product \n\n"
"  MIDI_KAPPAMATRIX11          FITS image  Kappa matrix: For diagnostics\n"
"  MIDI_KAPPAMATRIX11_FILTERED FITS image  Kappa matrix: For diagnostics\n"
"  MIDI_KAPPAMATRIX11_NOMASK   FITS image  Kappa matrix: For diagnostics\n"
"  MIDI_KAPPAMATRIX12          FITS image  Kappa matrix: For diagnostics\n"
"  MIDI_KAPPAMATRIX12_FILTERED FITS image  Kappa matrix: For diagnostics\n"
"  MIDI_KAPPAMATRIX12_NOMASK   FITS image  Kappa matrix: For diagnostics\n"
"  MIDI_KAPPAMATRIX21          FITS image  Kappa matrix: For diagnostics\n"
"  MIDI_KAPPAMATRIX21_FILTERED FITS image  Kappa matrix: For diagnostics\n"
"  MIDI_KAPPAMATRIX21_NOMASK   FITS image  Kappa matrix: For diagnostics\n"
"  MIDI_KAPPAMATRIX22          FITS image  Kappa matrix: For diagnostics\n"
"  MIDI_KAPPAMATRIX22_FILTERED FITS image  Kappa matrix: For diagnostics\n"
"  MIDI_KAPPAMATRIX22_NOMASK   FITS image  Kappa matrix: For diagnostics\n\n";


/*-----------------------------------------------------------------------------
                                Function code
 -----------------------------------------------------------------------------*/


/*----------------------------------------------------------------------------*/
/**
 * @defgroup midi_kappamatrix    Midi Kappamatrix
 *
 */
/*----------------------------------------------------------------------------*/

/**@{*/

/*----------------------------------------------------------------------------*/
/**
  @brief    Build the list of available plugins, for this module. 
  @param    list    the plugin list
  @return   0 if everything is ok, 1 otherwise
  @note     Only this function is exported

  Create the recipe instance and make it available to the application using the 
  interface. 
 */
/*----------------------------------------------------------------------------*/
int cpl_plugin_get_info(cpl_pluginlist * list)
{
    cpl_recipe  *   recipe = cpl_calloc(1, sizeof *recipe );
    cpl_plugin  *   plugin = &recipe->interface;

    if (cpl_plugin_init(plugin,
                    CPL_PLUGIN_API,
                    MIDI_BINARY_VERSION,
                    CPL_PLUGIN_TYPE_RECIPE,
                    "midi_kappamatrix",
                    "Derives the Kappamatrix for the SCI_PHOT mode",
                    midi_kappamatrix_description,
                    "Armin Gabasch",
                    PACKAGE_BUGREPORT,
                    midi_get_license(),
                    midi_kappamatrix_create,
                    midi_kappamatrix_exec,
                    midi_kappamatrix_destroy)) {    
        cpl_msg_error(cpl_func, "Plugin initialization failed");
        (void)cpl_error_set_where(cpl_func);                          
        return 1;                                               
    }                                                    

    if (cpl_pluginlist_append(list, plugin)) {                 
        cpl_msg_error(cpl_func, "Error adding plugin to list");
        (void)cpl_error_set_where(cpl_func);                         
        return 1;                                              
    }                                                          
    
    return 0;
}

/*----------------------------------------------------------------------------*/
/**
  @brief    Setup the recipe options    
  @param    plugin  the plugin
  @return   0 if everything is ok

  Defining the command-line/configuration parameters for the recipe.
 */
/*----------------------------------------------------------------------------*/
static int midi_kappamatrix_create(cpl_plugin * plugin)
{
    cpl_recipe    * recipe;                                               
    cpl_parameter * p;
                                                                       
    /* Do not create the recipe if an error code is already set */     
    if (cpl_error_get_code() != CPL_ERROR_NONE) {                      
        cpl_msg_error(cpl_func, "%s():%d: An error is already set: %s",
                      cpl_func, __LINE__, cpl_error_get_where());      
        return (int)cpl_error_get_code();                              
    }                                                                  
                                                                       
    if (plugin == NULL) {                                              
        cpl_msg_error(cpl_func, "Null plugin");                        
        cpl_ensure_code(0, (int)CPL_ERROR_NULL_INPUT);                 
    }                                                                  
                                                                       
    /* Verify plugin type */                                           
    if (cpl_plugin_get_type(plugin) != CPL_PLUGIN_TYPE_RECIPE) {       
        cpl_msg_error(cpl_func, "Plugin is not a recipe");             
        cpl_ensure_code(0, (int)CPL_ERROR_TYPE_MISMATCH);              
    }                                                                  
                                                                       
    /* Get the recipe */                                               
    recipe = (cpl_recipe *)plugin;                                     
                                                                       
    /* Create the parameters list in the cpl_recipe object */          
    recipe->parameters = cpl_parameterlist_new();                      
    if (recipe->parameters == NULL) {                                  
        cpl_msg_error(cpl_func, "Parameter list allocation failed");   
        cpl_ensure_code(0, (int)CPL_ERROR_ILLEGAL_OUTPUT);             
    }                                                                  

    /* Fill the parameters list */

    /* --stropt */
/*     p = cpl_parameter_new_value("midi.midi_kappamatrix.outputfilename",  */
/*             CPL_TYPE_STRING, "Output Filename (placeholder; not used for the moment!!)", "midi.midi_kappamatrix","kappamatrix.fits"); */
/*     cpl_parameter_set_alias(p, CPL_PARAMETER_MODE_CLI, "outfile"); */
/*     cpl_parameter_disable(p, CPL_PARAMETER_MODE_ENV); */
/*     cpl_parameterlist_append(recipe->parameters, p); */

    p = cpl_parameter_new_value("midi.midi_kappamatrix.medianwindow", 
            CPL_TYPE_INT,
            "The window size of the median filter. No influence on the main product",
            "midi.midi_kappamatrix",9);
    cpl_parameter_set_alias(p, CPL_PARAMETER_MODE_CLI, "medianwindow");
    cpl_parameter_disable(p, CPL_PARAMETER_MODE_ENV);
    cpl_parameterlist_append(recipe->parameters, p);


/*     /\* --boolopt *\/ */
/*     p = cpl_parameter_new_value("midi.midi_kappamatrix.bool_option",  */
/*             CPL_TYPE_BOOL, "a flag", "midi.midi_kappamatrix", TRUE); */
/*     cpl_parameter_set_alias(p, CPL_PARAMETER_MODE_CLI, "boolopt"); */
/*     cpl_parameter_disable(p, CPL_PARAMETER_MODE_ENV); */
/*     cpl_parameterlist_append(recipe->parameters, p); */
 
    return 0;
}

/*----------------------------------------------------------------------------*/
/**
  @brief    Execute the plugin instance given by the interface
  @param    plugin  the plugin
  @return   0 if everything is ok
 */
/*----------------------------------------------------------------------------*/
static int midi_kappamatrix_exec(cpl_plugin * plugin)
{

    cpl_recipe * recipe;                                                   
    int recipe_status;                                                     
    cpl_errorstate initial_errorstate = cpl_errorstate_get();              
                                                                           
    /* Return immediately if an error code is already set */               
    if (cpl_error_get_code() != CPL_ERROR_NONE) {                          
        cpl_msg_error(cpl_func, "%s():%d: An error is already set: %s",    
                      cpl_func, __LINE__, cpl_error_get_where());          
        return (int)cpl_error_get_code();                                  
    }                                                                      
                                                                           
    if (plugin == NULL) {                                                  
        cpl_msg_error(cpl_func, "Null plugin");                            
        cpl_ensure_code(0, (int)CPL_ERROR_NULL_INPUT);                     
    }                                                                      
                                                                           
    /* Verify plugin type */                                               
    if (cpl_plugin_get_type(plugin) != CPL_PLUGIN_TYPE_RECIPE) {           
        cpl_msg_error(cpl_func, "Plugin is not a recipe");                 
        cpl_ensure_code(0, (int)CPL_ERROR_TYPE_MISMATCH);                  
    }                                                                      
                                                                           
    /* Get the recipe */                                                   
    recipe = (cpl_recipe *)plugin;                                         
                                                                           
    /* Verify parameter and frame lists */                                 
    if (recipe->parameters == NULL) {                                      
        cpl_msg_error(cpl_func, "Recipe invoked with NULL parameter list");
        cpl_ensure_code(0, (int)CPL_ERROR_NULL_INPUT);                     
    }                                                                      
    if (recipe->frames == NULL) {                                          
        cpl_msg_error(cpl_func, "Recipe invoked with NULL frame set");     
        cpl_ensure_code(0, (int)CPL_ERROR_NULL_INPUT);                     
    }                                                                      
                                                                           
    /* Invoke the recipe */                                                
    recipe_status = midi_kappamatrix(recipe->frames, recipe->parameters);
                                                                           
    /* Ensure DFS-compliance of the products */                            
    if (cpl_dfs_update_product_header(recipe->frames)) {                   
        if (!recipe_status) recipe_status = (int)cpl_error_get_code();                         
    }                                                                      
                                                                           
    if (!cpl_errorstate_is_equal(initial_errorstate)) {                    
        /* Dump the error history since recipe execution start.            
           At this point the recipe cannot recover from the error */       
        cpl_errorstate_dump(initial_errorstate, CPL_FALSE, NULL);          
    }                                                                      
                                                                           
    return recipe_status;                                                  
}

/*----------------------------------------------------------------------------*/
/**
  @brief    Destroy what has been created by the 'create' function
  @param    plugin  the plugin
  @return   0 if everything is ok
 */
/*----------------------------------------------------------------------------*/
static int midi_kappamatrix_destroy(cpl_plugin * plugin)
{
    cpl_recipe * recipe;                                          
                                                                  
    if (plugin == NULL) {                                         
        cpl_msg_error(cpl_func, "Null plugin");                   
        cpl_ensure_code(0, (int)CPL_ERROR_NULL_INPUT);            
    }                                                             
                                                                  
    /* Verify plugin type */                                      
    if (cpl_plugin_get_type(plugin) != CPL_PLUGIN_TYPE_RECIPE) {  
        cpl_msg_error(cpl_func, "Plugin is not a recipe");        
        cpl_ensure_code(0, (int)CPL_ERROR_TYPE_MISMATCH);         
    }                                                             
                                                                  
    /* Get the recipe */                                          
    recipe = (cpl_recipe *)plugin;                                
                                                                  
    cpl_parameterlist_delete(recipe->parameters);             
                                                                  
    return 0;                                                    
}

/*----------------------------------------------------------------------------*/
/**
  @brief    Interpret the command line options and execute the data processing
  @param    frameset   the frames list
  @param    parlist    the parameters list
  @return   0 if everything is ok
 */
/*----------------------------------------------------------------------------*/
static int midi_kappamatrix(cpl_frameset            * frameset,
                            const cpl_parameterlist * parlist)
{
   const cpl_parameter   * param;
   cpl_frame             * cur_frame;
   cpl_table             * table=NULL;
   cpl_table             * kappamatrix_table=NULL;
   cpl_propertylist      * plist;

   cpl_propertylist      * qclist11;
   cpl_propertylist      * qclist12;
   cpl_propertylist      * qclist21;
   cpl_propertylist      * qclist22;
   cpl_propertylist      * qclist11_nomask;
   cpl_propertylist      * qclist12_nomask;
   cpl_propertylist      * qclist21_nomask;
   cpl_propertylist      * qclist22_nomask;
   cpl_propertylist      * qclist11_filtered;
   cpl_propertylist      * qclist12_filtered;
   cpl_propertylist      * qclist21_filtered;
   cpl_propertylist      * qclist22_filtered;
   cpl_propertylist      * qclist_all;
   cpl_propertylist      * qclist_all_extension;
   cpl_mask 			 * mask=NULL;

   const char            * shutter_id =NULL; 
   cpl_errorstate          prestate = cpl_errorstate_get();
   char                  * tag=NULL;
/*   cpl_matrix            * kernel ;   */
   char                  * cubename;

   char     gris_name[100]= ""; 
   int      ext_imaging_data;    
   int      isPHOTAdata=0;
   int      isPHOTBdata=0;
   double * pmask_grism_DATA1=NULL;
   double * pmask_grism_DATA1_norm=NULL;
   double * pmask_grism_DATA2=NULL;
   double * pmask_grism_DATA2_norm=NULL;
   double * pmask_grism_DATA3=NULL;
   double * pmask_grism_DATA3_norm=NULL;
   double * pmask_grism_DATA4=NULL;
   double * pmask_grism_DATA4_norm=NULL;

   double * pmask_prism_DATA1=NULL;
   double * pmask_prism_DATA1_norm=NULL;
   double * pmask_prism_DATA2=NULL;
   double * pmask_prism_DATA2_norm=NULL;
   double * pmask_prism_DATA3=NULL;
   double * pmask_prism_DATA3_norm=NULL;
   double * pmask_prism_DATA4=NULL;
   double * pmask_prism_DATA4_norm=NULL;

   double kappamatrix11_stdev=0.;
   double kappamatrix11_stdev_nomask=0.;
   double kappamatrix11_stdev_filtered=0.;
   double kappamatrix12_stdev=0.;
   double kappamatrix12_stdev_nomask=0.;
   double kappamatrix12_stdev_filtered=0.;
   double kappamatrix21_stdev=0.;
   double kappamatrix21_stdev_nomask=0.;
   double kappamatrix21_stdev_filtered=0.;
   double kappamatrix22_stdev=0.;
   double kappamatrix22_stdev_nomask=0.;
   double kappamatrix22_stdev_filtered=0.;

   double kappamatrix11_median=0.;
   double kappamatrix11_median_nomask=0.;
   double kappamatrix11_median_filtered=0.;
   double kappamatrix12_median=0.;
   double kappamatrix12_median_nomask=0.;
   double kappamatrix12_median_filtered=0.;
   double kappamatrix21_median=0.;
   double kappamatrix21_median_nomask=0.;
   double kappamatrix21_median_filtered=0.;
   double kappamatrix22_median=0.;
   double kappamatrix22_median_nomask=0.;
   double kappamatrix22_median_filtered=0.;
   

   int nx_DATA1=0,nx_DATA2=0,nx_DATA3=0,nx_DATA4=0; 
   int ny_DATA1=0,ny_DATA2=0,ny_DATA3=0,ny_DATA4=0; 


      int xpos=0, ypos=0;


   cpl_image * image_AOPEN_DATA2_T = NULL;       
   cpl_image * image_AOPEN_DATA3_T = NULL;       
   cpl_image * image_AOPEN_DATA4_T = NULL;       

   cpl_image * image_BOPEN_DATA1_T = NULL;       
   cpl_image * image_BOPEN_DATA2_T = NULL;       
   cpl_image * image_BOPEN_DATA3_T = NULL;       

   cpl_image * image_AOPEN_DATA2_T_collapsed = NULL;
   cpl_image * image_AOPEN_DATA3_T_collapsed = NULL;
   cpl_image * image_AOPEN_DATA4_T_collapsed = NULL;
   cpl_image * image_BOPEN_DATA1_T_collapsed = NULL;
   cpl_image * image_BOPEN_DATA2_T_collapsed = NULL;
   cpl_image * image_BOPEN_DATA3_T_collapsed = NULL;

   cpl_image * image_AOPEN_DATA2_T_collapsed_nomask = NULL;
   cpl_image * image_AOPEN_DATA3_T_collapsed_nomask = NULL;
   cpl_image * image_AOPEN_DATA4_T_collapsed_nomask = NULL;
   cpl_image * image_BOPEN_DATA1_T_collapsed_nomask = NULL;
   cpl_image * image_BOPEN_DATA2_T_collapsed_nomask = NULL;
   cpl_image * image_BOPEN_DATA3_T_collapsed_nomask = NULL;


   cpl_image * mask_grism_DATA1_collapsed = NULL;
   cpl_image * mask_grism_DATA2_collapsed = NULL;
   cpl_image * mask_grism_DATA3_collapsed = NULL;
   cpl_image * mask_grism_DATA4_collapsed = NULL;

   cpl_image * mask_prism_DATA1_collapsed = NULL;
   cpl_image * mask_prism_DATA2_collapsed = NULL;
   cpl_image * mask_prism_DATA3_collapsed = NULL;
   cpl_image * mask_prism_DATA4_collapsed = NULL;


   
   cpl_image * kappamatrix11 = NULL;
   cpl_image * kappamatrix12 = NULL;
   cpl_image * kappamatrix21 = NULL;
   cpl_image * kappamatrix22 = NULL;
   cpl_image * kappamatrix11_nomask = NULL;
   cpl_image * kappamatrix12_nomask = NULL;
   cpl_image * kappamatrix21_nomask = NULL;
   cpl_image * kappamatrix22_nomask = NULL;
   cpl_image * kappamatrix11_filtered = NULL;
   cpl_image * kappamatrix12_filtered = NULL;
   cpl_image * kappamatrix21_filtered = NULL;
   cpl_image * kappamatrix22_filtered = NULL;
   
   cpl_image * mask_grism_DATA1 = NULL;
   cpl_image * mask_grism_DATA2 = NULL;
   cpl_image * mask_grism_DATA3 = NULL;
   cpl_image * mask_grism_DATA4 = NULL;
   
   cpl_image * mask_grism_DATA1_norm = NULL;
   cpl_image * mask_grism_DATA2_norm = NULL;
   cpl_image * mask_grism_DATA3_norm = NULL;
   cpl_image * mask_grism_DATA4_norm = NULL;
   
   cpl_image * mask_prism_DATA1 = NULL;
   cpl_image * mask_prism_DATA2 = NULL;
   cpl_image * mask_prism_DATA3 = NULL;
   cpl_image * mask_prism_DATA4 = NULL;
   
   
   cpl_image * mask_prism_DATA1_norm = NULL;
   cpl_image * mask_prism_DATA2_norm = NULL;
   cpl_image * mask_prism_DATA3_norm = NULL;
   cpl_image * mask_prism_DATA4_norm = NULL;
  
/*    cpl_imagelist * imglst_AOPEN_DATA1_S; */
/*    cpl_imagelist * imglst_AOPEN_DATA1_T; */
   cpl_imagelist * imglst_AOPEN_DATA2_S;
   cpl_imagelist * imglst_AOPEN_DATA2_T;
   cpl_imagelist * imglst_AOPEN_DATA3_S;
   cpl_imagelist * imglst_AOPEN_DATA3_T;
   cpl_imagelist * imglst_AOPEN_DATA4_S;
   cpl_imagelist * imglst_AOPEN_DATA4_T;

   cpl_imagelist * imglst_BOPEN_DATA1_S;
   cpl_imagelist * imglst_BOPEN_DATA1_T;
   cpl_imagelist * imglst_BOPEN_DATA2_S;
   cpl_imagelist * imglst_BOPEN_DATA2_T;
   cpl_imagelist * imglst_BOPEN_DATA3_S;
   cpl_imagelist * imglst_BOPEN_DATA3_T;
/*    cpl_imagelist * imglst_BOPEN_DATA4_S; */
/*    cpl_imagelist * imglst_BOPEN_DATA4_T; */

   cpl_imagelist * imglst_mask_prism;
   cpl_imagelist * imglst_mask_grism;

   int medianwindow=0;
   int lowlimit=0;
   int highlimit=0;
   cpl_frameset_iterator * it = NULL;
   /* Check if SOF is empty */
   cur_frame = cpl_frameset_get_position(frameset, 0);
   if (cur_frame == NULL) {
	   return (int)cpl_error_set_message(cpl_func, CPL_ERROR_DATA_NOT_FOUND,
			   "SOF does not have any file");
   }

   
/*    imglst_AOPEN_DATA1_S=cpl_imagelist_new(); */
/*    imglst_AOPEN_DATA1_T=cpl_imagelist_new(); */
   imglst_AOPEN_DATA2_S=cpl_imagelist_new();
   imglst_AOPEN_DATA2_T=cpl_imagelist_new();
   imglst_AOPEN_DATA3_S=cpl_imagelist_new();
   imglst_AOPEN_DATA3_T=cpl_imagelist_new();
   imglst_AOPEN_DATA4_S=cpl_imagelist_new();
   imglst_AOPEN_DATA4_T=cpl_imagelist_new();
   
   imglst_BOPEN_DATA1_S=cpl_imagelist_new();
   imglst_BOPEN_DATA1_T=cpl_imagelist_new();
   imglst_BOPEN_DATA2_S=cpl_imagelist_new();
   imglst_BOPEN_DATA2_T=cpl_imagelist_new();
   imglst_BOPEN_DATA3_S=cpl_imagelist_new();
   imglst_BOPEN_DATA3_T=cpl_imagelist_new();
/*    imglst_BOPEN_DATA4_S=cpl_imagelist_new(); */
/*    imglst_BOPEN_DATA4_T=cpl_imagelist_new(); */

   imglst_mask_prism=cpl_imagelist_new();
   imglst_mask_grism=cpl_imagelist_new();

   
   
   /* RETRIEVE THE INPUT PARAMETERS */
/*    param = cpl_parameterlist_find_const(parlist, "midi.midi_kappamatrix.outputfilename"); */
/*    outputfilename = cpl_parameter_get_string(param); */
 
   param = cpl_parameterlist_find_const(parlist,
                                        "midi.midi_kappamatrix.medianwindow");
   medianwindow = cpl_parameter_get_int(param);

   if(medianwindow%2 == 0)/*  even number  */
   {
      cpl_msg_warning(cpl_func, "The window size of the median filter is not odd,");
      cpl_msg_warning(cpl_func, "therefore the size is increased by unity");
      medianwindow=medianwindow+1;   
   }

   if(medianwindow > 81)
   {
      cpl_msg_warning(cpl_func, "The window size of the median filter exceeds the maximal supported value,");
      cpl_msg_warning(cpl_func, "therefore the size is reset to 81");
      medianwindow=81;   
   }

   if(medianwindow < 0)
   {
      cpl_msg_warning(cpl_func, "The window size of the median filter must be positive,");
      cpl_msg_warning(cpl_func, "therefore the size is reset to 1");
      medianwindow=1;   
   }


  
   if (!cpl_errorstate_is_equal(prestate)) {
      return (int)cpl_error_set_message(cpl_func, cpl_error_get_code(), "Could not retrieve the input parameters");
   }
   
   /* Identify the RAW and CALIB frames in the input frameset */
   cpl_ensure_code(midi_dfs_set_groups(frameset) == CPL_ERROR_NONE,
                   cpl_error_get_code());
   
   
   /* 
      Walk through the whole Set of Frames SOF and search for KAPPA_SP_SCIENCE and/or KAPPA_SP_CALIB 
   */
   
   
   
/*    Start  1   */
   
   cur_frame = cpl_frameset_get_position(frameset, 0);
   if (cur_frame == NULL) {
      return (int)cpl_error_set_message(cpl_func, CPL_ERROR_DATA_NOT_FOUND,
                                        "SOF does not have any file");
   }
   it = cpl_frameset_iterator_new(frameset);
   cur_frame = cpl_frameset_iterator_get(it);

   /* Append the images from the tables to the various imagelists */     
   while(cur_frame)
   {
      /*      Check the right tags */
      tag = (char*)cpl_frame_get_tag(cur_frame);
      if (strcmp(tag, MIDI_PHOTOM_SP_CALIB) && strcmp(tag, MIDI_PHOTOM_SP_SCIENCE)) {
          cpl_frameset_iterator_advance(it, 1);
          cur_frame = cpl_frameset_iterator_get(it);
         continue;
      }
      
      cpl_msg_info(cpl_func, "Processing file  %s",cpl_frame_get_filename(cur_frame));
      ext_imaging_data=cpl_fits_find_extension(cpl_frame_get_filename(cur_frame),"IMAGING_DATA");
      
/*       Check if the file is AOPEN or BOPEN */
      
      plist = cpl_propertylist_load(cpl_frame_get_filename(cur_frame), 0);
      if (cpl_propertylist_has(plist, "ESO INS SHUT ID") == 1)
      {
         shutter_id=(cpl_propertylist_get_string(plist, "ESO INS SHUT ID"));
      }
      
/*       Checking if PRISM or GRISM */
      if (cpl_propertylist_has(plist, "ESO INS GRIS NAME") == 1)
      {
         strcpy(gris_name,cpl_propertylist_get_string(plist, "ESO INS GRIS NAME"));
      }
      
      if (!cpl_errorstate_is_equal(prestate)) {
         return (int)cpl_error_set_message(cpl_func, cpl_error_get_code(), "Could not ...");
      }
      
      if (strcmp(shutter_id,"AOPEN")==0)
      {
/*          PhotA Data found */
         isPHOTAdata=1;
         
         /* Load extension  Imaging Data  */ 
         table = cpl_table_load(cpl_frame_get_filename(cur_frame), ext_imaging_data, 1);
         if (table == NULL) {
            return (int)cpl_error_set_message(cpl_func, cpl_error_get_code(),
                                              "Could not load the table");
         }
         cpl_ensure_code(cpl_errorstate_is_equal(prestate), cpl_error_get_code());      
         
/*          No Signal in DATA1 for AOPEN */
         
         /*         cpl_msg_info(cpl_func, "Scanning for DATA2 ..."); */
         if (cpl_table_has_column(table,"DATA2")){
            table_to_imglst_sky_target("DATA2","TARTYP2",imglst_AOPEN_DATA2_S,imglst_AOPEN_DATA2_T,table);
         }
/*          cpl_msg_info(cpl_func, "Scanning for DATA3 ..."); */
         if (cpl_table_has_column(table,"DATA3")){
            table_to_imglst_sky_target("DATA3","TARTYP2",imglst_AOPEN_DATA3_S,imglst_AOPEN_DATA3_T,table);
         }
/*          cpl_msg_info(cpl_func, "Scanning for DATA4 ..."); */
         if (cpl_table_has_column(table,"DATA4")){
            table_to_imglst_sky_target("DATA4","TARTYP2",imglst_AOPEN_DATA4_S,imglst_AOPEN_DATA4_T,table);
         }
         
         cpl_msg_info(cpl_func, "Number of so far processed AOPEN patches: % " CPL_SIZE_FORMAT " ",cpl_imagelist_get_size(imglst_AOPEN_DATA2_T));
         
         cpl_ensure_code(cpl_errorstate_is_equal(prestate), cpl_error_get_code());
         
         cpl_table_delete(table);
      }
      
      if (strcmp(shutter_id,"BOPEN")==0)
      {
/*          PhotB Data found */
         isPHOTBdata=1;
         
         /* Load extension  Imaging Data  */ 
         table = cpl_table_load(cpl_frame_get_filename(cur_frame), ext_imaging_data, 1);
         if (table == NULL) {
            return (int)cpl_error_set_message(cpl_func, cpl_error_get_code(),
                                              "Could not load the table");
         }
         cpl_ensure_code(cpl_errorstate_is_equal(prestate), cpl_error_get_code());      
         
         
         /*         cpl_msg_info(cpl_func, "Scanning for DATA1 ..."); */
         if (cpl_table_has_column(table,"DATA1")){
            table_to_imglst_sky_target("DATA1","TARTYP2",imglst_BOPEN_DATA1_S,imglst_BOPEN_DATA1_T,table);
         }
         
/*          cpl_msg_info(cpl_func, "Scanning for DATA2 ..."); */
         if (cpl_table_has_column(table,"DATA2")){
            table_to_imglst_sky_target("DATA2","TARTYP2",imglst_BOPEN_DATA2_S,imglst_BOPEN_DATA2_T,table);
         }
/*          cpl_msg_info(cpl_func, "Scanning for DATA3 ..."); */
         if (cpl_table_has_column(table,"DATA3")){
            table_to_imglst_sky_target("DATA3","TARTYP2",imglst_BOPEN_DATA3_S,imglst_BOPEN_DATA3_T,table);
         }
         
         
/*          No Signal in DATA4 for BOPEN */
         
         cpl_msg_info(cpl_func, "Number of so far processed BOPEN patches: % " CPL_SIZE_FORMAT " ",cpl_imagelist_get_size(imglst_BOPEN_DATA2_T));
         
         
         cpl_ensure_code(cpl_errorstate_is_equal(prestate), cpl_error_get_code());
         
         cpl_table_delete(table);
      }
      
      
      cpl_propertylist_delete(plist);
      
      /* Get next frame from SOF */
      cpl_frameset_iterator_advance(it, 1);
      cur_frame = cpl_frameset_iterator_get(it);
      
   } /* while more frames */
   cpl_frameset_iterator_delete(it);

   if (isPHOTAdata==0 || isPHOTBdata==0)
   {
      cpl_msg_error(cpl_func, "No suitable SetOfFrame fround");              
      (void)cpl_error_set_where(cpl_func);
      return (int)cpl_error_set_message(cpl_func, cpl_error_get_code(), "Mask for filter name PRISM needed but not found");
      
   }
   
   
/*    Subtract the sky frame from the target frame*/
   
   cpl_msg_info(cpl_func, "Doing sky subtraction ...");
   
   if(cpl_imagelist_get_size(imglst_AOPEN_DATA2_T)==cpl_imagelist_get_size(imglst_AOPEN_DATA2_S)
      && cpl_imagelist_get_size(imglst_BOPEN_DATA1_T)==cpl_imagelist_get_size(imglst_BOPEN_DATA1_S)
      )
   {
      cpl_imagelist_subtract(imglst_AOPEN_DATA2_T,imglst_AOPEN_DATA2_S); 
      cpl_imagelist_subtract(imglst_AOPEN_DATA3_T,imglst_AOPEN_DATA3_S); 
      cpl_imagelist_subtract(imglst_AOPEN_DATA4_T,imglst_AOPEN_DATA4_S); 
      cpl_imagelist_subtract(imglst_BOPEN_DATA1_T,imglst_BOPEN_DATA1_S); 
      cpl_imagelist_subtract(imglst_BOPEN_DATA2_T,imglst_BOPEN_DATA2_S); 
      cpl_imagelist_subtract(imglst_BOPEN_DATA3_T,imglst_BOPEN_DATA3_S); 
   }
   else 
   {
      cpl_msg_info(cpl_func, "The number of Skyframes and Targetframes differ");
      return (int)cpl_error_set_message(cpl_func, cpl_error_get_code(),
                                        "The number of Skyframes and Targetframes differ"); 
   }
   
   
   cpl_msg_info(cpl_func, "Collapsing the images in the time domain ...");
   
/*    Collapse the sky-subtracted targets */
   image_AOPEN_DATA2_T=cpl_imagelist_collapse_create(imglst_AOPEN_DATA2_T);
   image_AOPEN_DATA3_T=cpl_imagelist_collapse_create(imglst_AOPEN_DATA3_T);
   image_AOPEN_DATA4_T=cpl_imagelist_collapse_create(imglst_AOPEN_DATA4_T);
   image_BOPEN_DATA1_T=cpl_imagelist_collapse_create(imglst_BOPEN_DATA1_T);
   image_BOPEN_DATA2_T=cpl_imagelist_collapse_create(imglst_BOPEN_DATA2_T);
   image_BOPEN_DATA3_T=cpl_imagelist_collapse_create(imglst_BOPEN_DATA3_T);
   

   cpl_imagelist_delete(imglst_AOPEN_DATA2_T);
   cpl_imagelist_delete(imglst_AOPEN_DATA3_T);
   cpl_imagelist_delete(imglst_AOPEN_DATA4_T);
   cpl_imagelist_delete(imglst_AOPEN_DATA2_S);
   cpl_imagelist_delete(imglst_AOPEN_DATA3_S);
   cpl_imagelist_delete(imglst_AOPEN_DATA4_S);
   
   cpl_imagelist_delete(imglst_BOPEN_DATA1_T);
   cpl_imagelist_delete(imglst_BOPEN_DATA2_T);
   cpl_imagelist_delete(imglst_BOPEN_DATA3_T);
   cpl_imagelist_delete(imglst_BOPEN_DATA1_S);
   cpl_imagelist_delete(imglst_BOPEN_DATA2_S);
   cpl_imagelist_delete(imglst_BOPEN_DATA3_S);
   
/*    END  1   */
   
   
/* Extract the masks from the table and store them in a cpl_image */
   
/*    The masks are stored in the two imagelists imglst_mask_grism and imglst_mask_prism */
/*    !! DATA1 is read twice, therefore DATA1 is stored in imglst_mask_grism[1] and so on !! */
   
   cpl_msg_info(cpl_func, "Extracting the masks ...");
   
   cur_frame = cpl_frameset_get_position(frameset, 0);
   if (cur_frame == NULL) {
      return (int)cpl_error_set_message(cpl_func, CPL_ERROR_DATA_NOT_FOUND,
                                        "SOF does not have any file");
   }

   it = cpl_frameset_iterator_new(frameset);
   cur_frame = cpl_frameset_iterator_get(it);
   
   while(cur_frame)
   {
      ext_imaging_data=cpl_fits_find_extension(cpl_frame_get_filename(cur_frame),"IMAGING_DATA");
      
      /*      Check the right tags */
      tag = (char*)cpl_frame_get_tag(cur_frame);
      
      if (!strcmp(tag, MIDI_KAPPA_SP_MASK_GRISM)) {
         table = cpl_table_load(cpl_frame_get_filename(cur_frame), ext_imaging_data, 1);
         if (table == NULL) {
            return (int)cpl_error_set_message(cpl_func, cpl_error_get_code(),
                                              "Could not load the table");
         }
         cpl_ensure_code(cpl_errorstate_is_equal(prestate), cpl_error_get_code());      
         
         table_to_imglst_mask("DATA1",imglst_mask_grism,table); 
         table_to_imglst_mask("DATA1",imglst_mask_grism,table); 
         table_to_imglst_mask("DATA2",imglst_mask_grism,table); 
         table_to_imglst_mask("DATA3",imglst_mask_grism,table); 
         table_to_imglst_mask("DATA4",imglst_mask_grism,table); 
         
/* Normalizing the mask to 1*/
         
         cpl_msg_info(cpl_func, "Normalizing the integral of the masks (GRISM) to unity ...");         
         mask_grism_DATA1=cpl_image_cast(cpl_imagelist_get(imglst_mask_grism, 1),CPL_TYPE_DOUBLE);
         mask_grism_DATA2=cpl_image_cast(cpl_imagelist_get(imglst_mask_grism, 2),CPL_TYPE_DOUBLE);
         mask_grism_DATA3=cpl_image_cast(cpl_imagelist_get(imglst_mask_grism, 3),CPL_TYPE_DOUBLE);
         mask_grism_DATA4=cpl_image_cast(cpl_imagelist_get(imglst_mask_grism, 4),CPL_TYPE_DOUBLE);
          
         mask_grism_DATA1_norm=cpl_image_collapse_create(mask_grism_DATA1,0);
         mask_grism_DATA2_norm=cpl_image_collapse_create(mask_grism_DATA2,0);
         mask_grism_DATA3_norm=cpl_image_collapse_create(mask_grism_DATA3,0);
         mask_grism_DATA4_norm=cpl_image_collapse_create(mask_grism_DATA4,0);
          
         nx_DATA1=cpl_image_get_size_x(mask_grism_DATA1);
         ny_DATA1=cpl_image_get_size_y(mask_grism_DATA1);
         nx_DATA2=cpl_image_get_size_x(mask_grism_DATA2);
         ny_DATA2=cpl_image_get_size_y(mask_grism_DATA2);
         nx_DATA3=cpl_image_get_size_x(mask_grism_DATA3);
         ny_DATA3=cpl_image_get_size_y(mask_grism_DATA3);
         nx_DATA4=cpl_image_get_size_x(mask_grism_DATA4);
         ny_DATA4=cpl_image_get_size_y(mask_grism_DATA4);
         
         pmask_grism_DATA1     = cpl_image_get_data_double(mask_grism_DATA1);
         pmask_grism_DATA1_norm= cpl_image_get_data_double(mask_grism_DATA1_norm);
         pmask_grism_DATA2     = cpl_image_get_data_double(mask_grism_DATA2);
         pmask_grism_DATA2_norm= cpl_image_get_data_double(mask_grism_DATA2_norm);
         pmask_grism_DATA3     = cpl_image_get_data_double(mask_grism_DATA3);
         pmask_grism_DATA3_norm= cpl_image_get_data_double(mask_grism_DATA3_norm);
         pmask_grism_DATA4     = cpl_image_get_data_double(mask_grism_DATA4);
         pmask_grism_DATA4_norm= cpl_image_get_data_double(mask_grism_DATA4_norm);
         
         
         
         for (xpos=0; xpos<nx_DATA1; ++xpos)
         {
            for(ypos=0; ypos<ny_DATA1;++ypos)
            {
               pmask_grism_DATA1[xpos+nx_DATA1*ypos]  /= pmask_grism_DATA1_norm[xpos];
            }
         }
         
         for (xpos=0; xpos<nx_DATA2; ++xpos)
         {
            for(ypos=0; ypos<ny_DATA2;++ypos)
            {
               pmask_grism_DATA2[xpos+nx_DATA2*ypos]  /= pmask_grism_DATA2_norm[xpos];
            }
         }
         
         
         for (xpos=0; xpos<nx_DATA3; ++xpos)
         {
            for(ypos=0; ypos<ny_DATA3;++ypos)
            {
               pmask_grism_DATA3[xpos+nx_DATA3*ypos]  /= pmask_grism_DATA3_norm[xpos];
            }
         }
         
         
         for (xpos=0; xpos<nx_DATA4; ++xpos)
         {
            for(ypos=0; ypos<ny_DATA4;++ypos)
            {
               pmask_grism_DATA4[xpos+nx_DATA4*ypos]  /= pmask_grism_DATA4_norm[xpos];
            }
         }
         
         cpl_image_delete(mask_grism_DATA1_norm);
         cpl_image_delete(mask_grism_DATA2_norm);
         cpl_image_delete(mask_grism_DATA3_norm);
         cpl_image_delete(mask_grism_DATA4_norm);
         
         cpl_table_delete(table);

      }
      
      if (!strcmp(tag, MIDI_KAPPA_SP_MASK_PRISM)) {
         table = cpl_table_load(cpl_frame_get_filename(cur_frame), ext_imaging_data, 1);
         if (table == NULL) {
            return (int)cpl_error_set_message(cpl_func, cpl_error_get_code(),
                                              "Could not load the table");
         }
         cpl_ensure_code(cpl_errorstate_is_equal(prestate), cpl_error_get_code());      
         
         table_to_imglst_mask("DATA1",imglst_mask_prism,table); 
         table_to_imglst_mask("DATA1",imglst_mask_prism,table); 
         table_to_imglst_mask("DATA2",imglst_mask_prism,table); 
         table_to_imglst_mask("DATA3",imglst_mask_prism,table); 
         table_to_imglst_mask("DATA4",imglst_mask_prism,table); 


/* Normalizing the mask to 1*/
         
         cpl_msg_info(cpl_func, "Normalizing the integral of the masks (PRISM) to unity ...");               
         mask_prism_DATA1=cpl_image_cast(cpl_imagelist_get(imglst_mask_prism, 1),CPL_TYPE_DOUBLE);
         mask_prism_DATA2=cpl_image_cast(cpl_imagelist_get(imglst_mask_prism, 2),CPL_TYPE_DOUBLE);
         mask_prism_DATA3=cpl_image_cast(cpl_imagelist_get(imglst_mask_prism, 3),CPL_TYPE_DOUBLE);
         mask_prism_DATA4=cpl_image_cast(cpl_imagelist_get(imglst_mask_prism, 4),CPL_TYPE_DOUBLE);
          
         mask_prism_DATA1_norm=cpl_image_collapse_create(mask_prism_DATA1,0);
         mask_prism_DATA2_norm=cpl_image_collapse_create(mask_prism_DATA2,0);
         mask_prism_DATA3_norm=cpl_image_collapse_create(mask_prism_DATA3,0);
         mask_prism_DATA4_norm=cpl_image_collapse_create(mask_prism_DATA4,0);
          
         nx_DATA1=cpl_image_get_size_x(mask_prism_DATA1);
         ny_DATA1=cpl_image_get_size_y(mask_prism_DATA1);
         nx_DATA2=cpl_image_get_size_x(mask_prism_DATA2);
         ny_DATA2=cpl_image_get_size_y(mask_prism_DATA2);
         nx_DATA3=cpl_image_get_size_x(mask_prism_DATA3);
         ny_DATA3=cpl_image_get_size_y(mask_prism_DATA3);
         nx_DATA4=cpl_image_get_size_x(mask_prism_DATA4);
         ny_DATA4=cpl_image_get_size_y(mask_prism_DATA4);
         
         pmask_prism_DATA1     = cpl_image_get_data_double(mask_prism_DATA1);
         pmask_prism_DATA1_norm= cpl_image_get_data_double(mask_prism_DATA1_norm);
         pmask_prism_DATA2     = cpl_image_get_data_double(mask_prism_DATA2);
         pmask_prism_DATA2_norm= cpl_image_get_data_double(mask_prism_DATA2_norm);
         pmask_prism_DATA3     = cpl_image_get_data_double(mask_prism_DATA3);
         pmask_prism_DATA3_norm= cpl_image_get_data_double(mask_prism_DATA3_norm);
         pmask_prism_DATA4     = cpl_image_get_data_double(mask_prism_DATA4);
         pmask_prism_DATA4_norm= cpl_image_get_data_double(mask_prism_DATA4_norm);
         
         
         
         for (xpos=0; xpos<nx_DATA1; ++xpos)
         {
            for(ypos=0; ypos<ny_DATA1;++ypos)
            {
               pmask_prism_DATA1[xpos+nx_DATA1*ypos]  /= pmask_prism_DATA1_norm[xpos];
            }
         }
         
         for (xpos=0; xpos<nx_DATA2; ++xpos)
         {
            for(ypos=0; ypos<ny_DATA2;++ypos)
            {
               pmask_prism_DATA2[xpos+nx_DATA2*ypos]  /= pmask_prism_DATA2_norm[xpos];
            }
         }
         
         
         for (xpos=0; xpos<nx_DATA3; ++xpos)
         {
            for(ypos=0; ypos<ny_DATA3;++ypos)
            {
               pmask_prism_DATA3[xpos+nx_DATA3*ypos]  /= pmask_prism_DATA3_norm[xpos];
            }
         }
         
         
         for (xpos=0; xpos<nx_DATA4; ++xpos)
         {
            for(ypos=0; ypos<ny_DATA4;++ypos)
            {
               pmask_prism_DATA4[xpos+nx_DATA4*ypos]  /= pmask_prism_DATA4_norm[xpos];
            }
         }
         
         cpl_image_delete(mask_prism_DATA1_norm);
         cpl_image_delete(mask_prism_DATA2_norm);
         cpl_image_delete(mask_prism_DATA3_norm);
         cpl_image_delete(mask_prism_DATA4_norm);
         
         
         cpl_table_delete(table);
      }
      
      /* Get next frame from SOF */
      cpl_frameset_iterator_advance(it, 1);
      cur_frame = cpl_frameset_iterator_get(it);
      
   } /* while more frames */
   cpl_frameset_iterator_delete(it);

/*  Check if the mask is the right one    */
   
   if (strcmp(gris_name,"GRISM")==0 && cpl_imagelist_get_size(imglst_mask_grism) <1)
   {
      cpl_msg_error(cpl_func, "Mask for  GRISM needed but not found");              
      (void)cpl_error_set_where(cpl_func);
      return (int)cpl_error_set_message(cpl_func, cpl_error_get_code(), "Mask for filter name GRISM needed but not found");
   }
   
   else if (strcmp(gris_name,"PRISM")==0 && cpl_imagelist_get_size(imglst_mask_prism) <1)
   {
      cpl_msg_error(cpl_func, "Mask for PRISM needed but not found");              
      (void)cpl_error_set_where(cpl_func);
      return (int)cpl_error_set_message(cpl_func, cpl_error_get_code(), "Mask for filter name PRISM needed but not found");
   }


   cpl_msg_info(cpl_func, "Extracting the signal without using a mask ...");

   image_AOPEN_DATA2_T_collapsed_nomask=cpl_image_collapse_create(image_AOPEN_DATA2_T,0);
   image_AOPEN_DATA3_T_collapsed_nomask=cpl_image_collapse_create(image_AOPEN_DATA3_T,0);
   image_AOPEN_DATA4_T_collapsed_nomask=cpl_image_collapse_create(image_AOPEN_DATA4_T,0);
   image_BOPEN_DATA1_T_collapsed_nomask=cpl_image_collapse_create(image_BOPEN_DATA1_T,0);
   image_BOPEN_DATA2_T_collapsed_nomask=cpl_image_collapse_create(image_BOPEN_DATA2_T,0);
   image_BOPEN_DATA3_T_collapsed_nomask=cpl_image_collapse_create(image_BOPEN_DATA3_T,0);



/* Multiply the image by the mask and collapse in the y-direction */

cpl_msg_info(cpl_func, "Extracting the signal using Horne 1986, PASP 98 209 ...");
cpl_msg_info(cpl_func, "and assuming that the frames are sky-dominated: V(x)-> V0 ...");


if (strcmp(gris_name,"GRISM")==0){
   cpl_msg_info(cpl_func, "Calculating the kappa matrix for the GRISM Data...");
   
/*    Multiply the images by the Mask */
   cpl_msg_info(cpl_func, "1) Multiplying the images by the mask ...");
   cpl_image_multiply(image_AOPEN_DATA2_T,mask_grism_DATA2);
   cpl_image_multiply(image_AOPEN_DATA3_T,mask_grism_DATA3);
   cpl_image_multiply(image_AOPEN_DATA4_T,mask_grism_DATA4);
   cpl_image_multiply(image_BOPEN_DATA1_T,mask_grism_DATA1);
   cpl_image_multiply(image_BOPEN_DATA2_T,mask_grism_DATA2);
   cpl_image_multiply(image_BOPEN_DATA3_T,mask_grism_DATA3);
   
/*   Collapse the images in the y-direction */
   cpl_msg_info(cpl_func, "2) Integrating the images perpendicular to the dispersion direction ...");
   image_AOPEN_DATA2_T_collapsed=cpl_image_collapse_create(image_AOPEN_DATA2_T,0);
   image_AOPEN_DATA3_T_collapsed=cpl_image_collapse_create(image_AOPEN_DATA3_T,0);
   image_AOPEN_DATA4_T_collapsed=cpl_image_collapse_create(image_AOPEN_DATA4_T,0);
   image_BOPEN_DATA1_T_collapsed=cpl_image_collapse_create(image_BOPEN_DATA1_T,0);
   image_BOPEN_DATA2_T_collapsed=cpl_image_collapse_create(image_BOPEN_DATA2_T,0);
   image_BOPEN_DATA3_T_collapsed=cpl_image_collapse_create(image_BOPEN_DATA3_T,0);
   
/* Derive P^2 following Horne et al. */

   cpl_msg_info(cpl_func, "3) Deriving mask^2 ");

   cpl_image_power(mask_grism_DATA1,2.0);
   cpl_image_power(mask_grism_DATA2,2.0);
   cpl_image_power(mask_grism_DATA3,2.0);
   cpl_image_power(mask_grism_DATA4,2.0);


/* Summing up P^2 in y direction*/
  cpl_msg_info(cpl_func, "4) Integrating mask^2 perpendicular to the dispersion direction");
  mask_grism_DATA1_collapsed=cpl_image_collapse_create(mask_grism_DATA1,0);
  mask_grism_DATA2_collapsed=cpl_image_collapse_create(mask_grism_DATA2,0);
  mask_grism_DATA3_collapsed=cpl_image_collapse_create(mask_grism_DATA3,0);
  mask_grism_DATA4_collapsed=cpl_image_collapse_create(mask_grism_DATA4,0);


  cpl_msg_info(cpl_func, "5) Extracting the final flux by dividing 2) through 4)");  
  cpl_image_divide(image_AOPEN_DATA2_T_collapsed,mask_grism_DATA2_collapsed);
  cpl_image_divide(image_AOPEN_DATA3_T_collapsed,mask_grism_DATA3_collapsed);
  cpl_image_divide(image_AOPEN_DATA4_T_collapsed,mask_grism_DATA4_collapsed);
  cpl_image_divide(image_BOPEN_DATA1_T_collapsed,mask_grism_DATA1_collapsed);
  cpl_image_divide(image_BOPEN_DATA2_T_collapsed,mask_grism_DATA2_collapsed);
  cpl_image_divide(image_BOPEN_DATA3_T_collapsed,mask_grism_DATA3_collapsed);

/* Free the memory */
  cpl_image_delete(mask_grism_DATA1_collapsed);
  cpl_image_delete(mask_grism_DATA2_collapsed);
  cpl_image_delete(mask_grism_DATA3_collapsed);
  cpl_image_delete(mask_grism_DATA4_collapsed);

  }
/* Free the memory */
  if(mask_grism_DATA1 != NULL)cpl_image_delete(mask_grism_DATA1);
  if(mask_grism_DATA2 != NULL)cpl_image_delete(mask_grism_DATA2);
  if(mask_grism_DATA3 != NULL)cpl_image_delete(mask_grism_DATA3);
  if(mask_grism_DATA4 != NULL)cpl_image_delete(mask_grism_DATA4);



if (strcmp(gris_name,"PRISM")==0){
   cpl_msg_info(cpl_func, "Calculating the kappa matrix for the PRISM Data...");
   
/*    Multiply the images by the Mask */
   cpl_msg_info(cpl_func, "1) Multiplying the images by the mask ...");
   cpl_image_multiply(image_AOPEN_DATA2_T,mask_prism_DATA2);
   cpl_image_multiply(image_AOPEN_DATA3_T,mask_prism_DATA3);
   cpl_image_multiply(image_AOPEN_DATA4_T,mask_prism_DATA4);
   cpl_image_multiply(image_BOPEN_DATA1_T,mask_prism_DATA1);
   cpl_image_multiply(image_BOPEN_DATA2_T,mask_prism_DATA2);
   cpl_image_multiply(image_BOPEN_DATA3_T,mask_prism_DATA3);

/*   Collapse the images in the y-direction */
   cpl_msg_info(cpl_func, "2) Integrating the images perpendicular to the dispersion direction ...");
   image_AOPEN_DATA2_T_collapsed=cpl_image_collapse_create(image_AOPEN_DATA2_T,0);
   image_AOPEN_DATA3_T_collapsed=cpl_image_collapse_create(image_AOPEN_DATA3_T,0);
   image_AOPEN_DATA4_T_collapsed=cpl_image_collapse_create(image_AOPEN_DATA4_T,0);
   image_BOPEN_DATA1_T_collapsed=cpl_image_collapse_create(image_BOPEN_DATA1_T,0);
   image_BOPEN_DATA2_T_collapsed=cpl_image_collapse_create(image_BOPEN_DATA2_T,0);
   image_BOPEN_DATA3_T_collapsed=cpl_image_collapse_create(image_BOPEN_DATA3_T,0);
   
/* Derive P^2 following Horne et al. */

   cpl_msg_info(cpl_func, "3) Deriving mask^2 ");

   cpl_image_power(mask_prism_DATA1,2.0);
   cpl_image_power(mask_prism_DATA2,2.0);
   cpl_image_power(mask_prism_DATA3,2.0);
   cpl_image_power(mask_prism_DATA4,2.0);


/* Summing up P^2 in y direction*/

  cpl_msg_info(cpl_func, "4) Integrating mask^2 perpendicular to the dispersion direction");
  mask_prism_DATA1_collapsed=cpl_image_collapse_create(mask_prism_DATA1,0);
  mask_prism_DATA2_collapsed=cpl_image_collapse_create(mask_prism_DATA2,0);
  mask_prism_DATA3_collapsed=cpl_image_collapse_create(mask_prism_DATA3,0);
  mask_prism_DATA4_collapsed=cpl_image_collapse_create(mask_prism_DATA4,0);
  
  cpl_msg_info(cpl_func, "5) Extracting the final flux by dividing 2) through 4)");  
  cpl_image_divide(image_AOPEN_DATA2_T_collapsed,mask_prism_DATA2_collapsed);
  cpl_image_divide(image_AOPEN_DATA3_T_collapsed,mask_prism_DATA3_collapsed);
  cpl_image_divide(image_AOPEN_DATA4_T_collapsed,mask_prism_DATA4_collapsed);
  cpl_image_divide(image_BOPEN_DATA1_T_collapsed,mask_prism_DATA1_collapsed);
  cpl_image_divide(image_BOPEN_DATA2_T_collapsed,mask_prism_DATA2_collapsed);
  cpl_image_divide(image_BOPEN_DATA3_T_collapsed,mask_prism_DATA3_collapsed);

/* Free the memory */
  cpl_image_delete(mask_prism_DATA1_collapsed);
  cpl_image_delete(mask_prism_DATA2_collapsed);
  cpl_image_delete(mask_prism_DATA3_collapsed);
  cpl_image_delete(mask_prism_DATA4_collapsed);
}

/* Free the memory */
  if(mask_prism_DATA1 != NULL)cpl_image_delete(mask_prism_DATA1);
  if(mask_prism_DATA2 != NULL)cpl_image_delete(mask_prism_DATA2);
  if(mask_prism_DATA3 != NULL)cpl_image_delete(mask_prism_DATA3);
  if(mask_prism_DATA4 != NULL)cpl_image_delete(mask_prism_DATA4);


/* Derive the Kappa matrix as defined in the VINCI paper (A&A 425 1161)  */
cpl_msg_info(cpl_func, "Deriving the KAPPA matrix following  A&A 425 1161 ...");

kappamatrix11=cpl_image_divide_create (image_BOPEN_DATA2_T_collapsed, image_BOPEN_DATA1_T_collapsed);
kappamatrix12=cpl_image_divide_create (image_AOPEN_DATA2_T_collapsed, image_AOPEN_DATA4_T_collapsed);
kappamatrix21=cpl_image_divide_create (image_BOPEN_DATA3_T_collapsed, image_BOPEN_DATA1_T_collapsed);
kappamatrix22=cpl_image_divide_create (image_AOPEN_DATA3_T_collapsed, image_AOPEN_DATA4_T_collapsed);

kappamatrix11_nomask=cpl_image_divide_create (image_BOPEN_DATA2_T_collapsed_nomask, image_BOPEN_DATA1_T_collapsed_nomask);
kappamatrix12_nomask=cpl_image_divide_create (image_AOPEN_DATA2_T_collapsed_nomask, image_AOPEN_DATA4_T_collapsed_nomask);
kappamatrix21_nomask=cpl_image_divide_create (image_BOPEN_DATA3_T_collapsed_nomask, image_BOPEN_DATA1_T_collapsed_nomask);
kappamatrix22_nomask=cpl_image_divide_create (image_AOPEN_DATA3_T_collapsed_nomask, image_AOPEN_DATA4_T_collapsed_nomask);

   
/* Free the memory */

cpl_image_delete(image_AOPEN_DATA2_T);
cpl_image_delete(image_AOPEN_DATA3_T);       
cpl_image_delete(image_AOPEN_DATA4_T);       

cpl_image_delete(image_BOPEN_DATA1_T);       
cpl_image_delete(image_BOPEN_DATA2_T);       
cpl_image_delete(image_BOPEN_DATA3_T);       

cpl_image_delete(image_AOPEN_DATA2_T_collapsed);
cpl_image_delete(image_AOPEN_DATA3_T_collapsed);
cpl_image_delete(image_AOPEN_DATA4_T_collapsed);

cpl_image_delete(image_BOPEN_DATA1_T_collapsed);
cpl_image_delete(image_BOPEN_DATA2_T_collapsed);
cpl_image_delete(image_BOPEN_DATA3_T_collapsed);

cpl_image_delete(image_AOPEN_DATA2_T_collapsed_nomask);
cpl_image_delete(image_AOPEN_DATA3_T_collapsed_nomask);
cpl_image_delete(image_AOPEN_DATA4_T_collapsed_nomask);

cpl_image_delete(image_BOPEN_DATA1_T_collapsed_nomask);
cpl_image_delete(image_BOPEN_DATA2_T_collapsed_nomask);
cpl_image_delete(image_BOPEN_DATA3_T_collapsed_nomask);

/* Smooth the kappa matrix */

/* Create the kernel */


/*
kernel = cpl_matrix_new(1, medianwindow);
cpl_matrix_fill(kernel, 1.0);
*/

cpl_msg_info(cpl_func, "Smoothing the KAPPA matrix ...");

mask = cpl_mask_new(medianwindow,1);
cpl_mask_not(mask);

kappamatrix11_filtered=cpl_image_duplicate(kappamatrix11);
kappamatrix12_filtered=cpl_image_duplicate(kappamatrix12);
kappamatrix21_filtered=cpl_image_duplicate(kappamatrix21);
kappamatrix22_filtered=cpl_image_duplicate(kappamatrix22);


cpl_image_filter_mask(kappamatrix11_filtered,kappamatrix11, mask,CPL_FILTER_MEDIAN ,CPL_BORDER_FILTER);
cpl_image_filter_mask(kappamatrix12_filtered,kappamatrix12, mask,CPL_FILTER_MEDIAN ,CPL_BORDER_FILTER);
cpl_image_filter_mask(kappamatrix21_filtered,kappamatrix21, mask,CPL_FILTER_MEDIAN ,CPL_BORDER_FILTER);
cpl_image_filter_mask(kappamatrix22_filtered,kappamatrix22, mask,CPL_FILTER_MEDIAN ,CPL_BORDER_FILTER);

cpl_mask_delete(mask);
/*

kappamatrix11_filtered=cpl_image_filter_median(kappamatrix11, kernel);
kappamatrix12_filtered=cpl_image_filter_median(kappamatrix12, kernel);
kappamatrix21_filtered=cpl_image_filter_median(kappamatrix21, kernel);
kappamatrix22_filtered=cpl_image_filter_median(kappamatrix22, kernel);
*/


/* Deriving QC Parameters  */

if (strcmp(gris_name,"GRISM")==0)
{
   lowlimit=140;
   highlimit=160;
}

else
{
   lowlimit=65;
   highlimit=85;
}



   /* Derives the standard deviation */
   kappamatrix11_stdev=cpl_image_get_stdev_window (kappamatrix11,lowlimit,1,highlimit,1);
   kappamatrix11_stdev_nomask=cpl_image_get_stdev_window (kappamatrix11_nomask,lowlimit,1,highlimit,1);
   kappamatrix11_stdev_filtered=cpl_image_get_stdev_window (kappamatrix11_filtered,lowlimit,1,highlimit,1);

   kappamatrix12_stdev=cpl_image_get_stdev_window (kappamatrix12,lowlimit,1,highlimit,1);
   kappamatrix12_stdev_nomask=cpl_image_get_stdev_window (kappamatrix12_nomask,lowlimit,1,highlimit,1);
   kappamatrix12_stdev_filtered=cpl_image_get_stdev_window (kappamatrix12_filtered,lowlimit,1,highlimit,1);

   kappamatrix21_stdev=cpl_image_get_stdev_window (kappamatrix21,lowlimit,1,highlimit,1);
   kappamatrix21_stdev_nomask=cpl_image_get_stdev_window (kappamatrix21_nomask,lowlimit,1,highlimit,1);
   kappamatrix21_stdev_filtered=cpl_image_get_stdev_window (kappamatrix21_filtered,lowlimit,1,highlimit,1);

   kappamatrix22_stdev=cpl_image_get_stdev_window (kappamatrix22,lowlimit,1,highlimit,1);
   kappamatrix22_stdev_nomask=cpl_image_get_stdev_window (kappamatrix22_nomask,lowlimit,1,highlimit,1);
   kappamatrix22_stdev_filtered=cpl_image_get_stdev_window (kappamatrix22_filtered,lowlimit,1,highlimit,1);

   /* Derives the median */
   kappamatrix11_median=cpl_image_get_median_window (kappamatrix11,lowlimit,1,highlimit,1);
   kappamatrix11_median_nomask=cpl_image_get_median_window (kappamatrix11_nomask,lowlimit,1,highlimit,1);
   kappamatrix11_median_filtered=cpl_image_get_median_window (kappamatrix11_filtered,lowlimit,1,highlimit,1);

   kappamatrix12_median=cpl_image_get_median_window (kappamatrix12,lowlimit,1,highlimit,1);
   kappamatrix12_median_nomask=cpl_image_get_median_window (kappamatrix12_nomask,lowlimit,1,highlimit,1);
   kappamatrix12_median_filtered=cpl_image_get_median_window (kappamatrix12_filtered,lowlimit,1,highlimit,1);

   kappamatrix21_median=cpl_image_get_median_window (kappamatrix21,lowlimit,1,highlimit,1);
   kappamatrix21_median_nomask=cpl_image_get_median_window (kappamatrix21_nomask,lowlimit,1,highlimit,1);
   kappamatrix21_median_filtered=cpl_image_get_median_window (kappamatrix21_filtered,lowlimit,1,highlimit,1);

   kappamatrix22_median=cpl_image_get_median_window (kappamatrix22,lowlimit,1,highlimit,1);
   kappamatrix22_median_nomask=cpl_image_get_median_window (kappamatrix22_nomask,lowlimit,1,highlimit,1);
   kappamatrix22_median_filtered=cpl_image_get_median_window (kappamatrix22_filtered,lowlimit,1,highlimit,1);


   


cpl_msg_info(cpl_func, "Saving the KAPPA matrix(es) ...");


    /* Save the KAPPA matrix(es) and add QC parameters  */
    qclist11 = cpl_propertylist_new();
    qclist12 = cpl_propertylist_new();
    qclist21 = cpl_propertylist_new();
    qclist22 = cpl_propertylist_new();

    qclist11_nomask = cpl_propertylist_new();
    qclist12_nomask = cpl_propertylist_new();
    qclist21_nomask = cpl_propertylist_new();
    qclist22_nomask = cpl_propertylist_new();

    qclist11_filtered = cpl_propertylist_new();
    qclist12_filtered = cpl_propertylist_new();
    qclist21_filtered = cpl_propertylist_new();
    qclist22_filtered = cpl_propertylist_new();

   /* Adding the standard median */
    cpl_propertylist_append_double(qclist11, "ESO QC KAPPA MEDIAN 11", kappamatrix11_median);
    cpl_propertylist_append_double(qclist12, "ESO QC KAPPA MEDIAN 12", kappamatrix12_median);
    cpl_propertylist_append_double(qclist21, "ESO QC KAPPA MEDIAN 21", kappamatrix21_median);
    cpl_propertylist_append_double(qclist22, "ESO QC KAPPA MEDIAN 22", kappamatrix22_median);

    cpl_propertylist_append_double(qclist11_nomask, "ESO QC KAPPA NOMASK MEDIAN 11", kappamatrix11_median_nomask);
    cpl_propertylist_append_double(qclist12_nomask, "ESO QC KAPPA NOMASK MEDIAN 12", kappamatrix12_median_nomask);
    cpl_propertylist_append_double(qclist21_nomask, "ESO QC KAPPA NOMASK MEDIAN 21", kappamatrix21_median_nomask);
    cpl_propertylist_append_double(qclist22_nomask, "ESO QC KAPPA NOMASK MEDIAN 22", kappamatrix22_median_nomask);

    cpl_propertylist_append_double(qclist11_filtered, "ESO QC KAPPA FILTERED MEDIAN 11", kappamatrix11_median_filtered);
    cpl_propertylist_append_double(qclist12_filtered, "ESO QC KAPPA FILTERED MEDIAN 12", kappamatrix12_median_filtered);
    cpl_propertylist_append_double(qclist21_filtered, "ESO QC KAPPA FILTERED MEDIAN 21", kappamatrix21_median_filtered);
    cpl_propertylist_append_double(qclist22_filtered, "ESO QC KAPPA FILTERED MEDIAN 22", kappamatrix22_median_filtered);

   /* Adding the standard deviation */
    cpl_propertylist_append_double(qclist11, "ESO QC KAPPA STDEV 11", kappamatrix11_stdev);
    cpl_propertylist_append_double(qclist12, "ESO QC KAPPA STDEV 12", kappamatrix12_stdev);
    cpl_propertylist_append_double(qclist21, "ESO QC KAPPA STDEV 21", kappamatrix21_stdev);
    cpl_propertylist_append_double(qclist22, "ESO QC KAPPA STDEV 22", kappamatrix22_stdev);

    cpl_propertylist_append_double(qclist11_nomask, "ESO QC KAPPA NOMASK STDEV 11", kappamatrix11_stdev_nomask);
    cpl_propertylist_append_double(qclist12_nomask, "ESO QC KAPPA NOMASK STDEV 12", kappamatrix12_stdev_nomask);
    cpl_propertylist_append_double(qclist21_nomask, "ESO QC KAPPA NOMASK STDEV 21", kappamatrix21_stdev_nomask);
    cpl_propertylist_append_double(qclist22_nomask, "ESO QC KAPPA NOMASK STDEV 22", kappamatrix22_stdev_nomask);

    cpl_propertylist_append_double(qclist11_filtered, "ESO QC KAPPA FILTERED STDEV 11", kappamatrix11_stdev_filtered);
    cpl_propertylist_append_double(qclist12_filtered, "ESO QC KAPPA FILTERED STDEV 12", kappamatrix12_stdev_filtered);
    cpl_propertylist_append_double(qclist21_filtered, "ESO QC KAPPA FILTERED STDEV 21", kappamatrix21_stdev_filtered);
    cpl_propertylist_append_double(qclist22_filtered, "ESO QC KAPPA FILTERED STDEV 22", kappamatrix22_stdev_filtered);



cpl_propertylist_update_string(qclist11, CPL_DFS_PRO_CATG, "MIDI_KAPPAMATRIX11"); 			
cpl_propertylist_update_string(qclist12, CPL_DFS_PRO_CATG, "MIDI_KAPPAMATRIX12"); 			
cpl_propertylist_update_string(qclist21, CPL_DFS_PRO_CATG, "MIDI_KAPPAMATRIX21"); 			
cpl_propertylist_update_string(qclist22, CPL_DFS_PRO_CATG, "MIDI_KAPPAMATRIX22"); 			

cpl_propertylist_update_string(qclist11_nomask, CPL_DFS_PRO_CATG, "MIDI_KAPPAMATRIX11_NOMASK"); 			
cpl_propertylist_update_string(qclist12_nomask, CPL_DFS_PRO_CATG, "MIDI_KAPPAMATRIX12_NOMASK"); 			
cpl_propertylist_update_string(qclist21_nomask, CPL_DFS_PRO_CATG, "MIDI_KAPPAMATRIX21_NOMASK"); 			
cpl_propertylist_update_string(qclist22_nomask, CPL_DFS_PRO_CATG, "MIDI_KAPPAMATRIX22_NOMASK"); 			

cpl_propertylist_update_string(qclist11_filtered, CPL_DFS_PRO_CATG, "MIDI_KAPPAMATRIX11_FILTERED"); 			
cpl_propertylist_update_string(qclist12_filtered, CPL_DFS_PRO_CATG, "MIDI_KAPPAMATRIX12_FILTERED"); 			
cpl_propertylist_update_string(qclist21_filtered, CPL_DFS_PRO_CATG, "MIDI_KAPPAMATRIX21_FILTERED"); 			
cpl_propertylist_update_string(qclist22_filtered, CPL_DFS_PRO_CATG, "MIDI_KAPPAMATRIX22_FILTERED"); 			







if (cpl_dfs_save_image(frameset, NULL, parlist, frameset, NULL,  kappamatrix11,
                       CPL_BPP_IEEE_FLOAT, "midi_kappamatrix",
                       qclist11, NULL,
                       PACKAGE "/" PACKAGE_VERSION,
                       "midi_kappamatrix11.fits")) {
   /* Propagate the error */
   (void)cpl_error_set_where(cpl_func);
}
if (cpl_dfs_save_image(frameset, NULL, parlist, frameset, NULL,  kappamatrix12,
                       CPL_BPP_IEEE_FLOAT, "midi_kappamatrix",
                       qclist12, NULL,
                       PACKAGE "/" PACKAGE_VERSION,
                       "midi_kappamatrix12.fits")) {
   /* Propagate the error */
   (void)cpl_error_set_where(cpl_func);
}
if (cpl_dfs_save_image(frameset, NULL, parlist, frameset, NULL,  kappamatrix21,
                       CPL_BPP_IEEE_FLOAT, "midi_kappamatrix",
                       qclist21, NULL,
                       PACKAGE "/" PACKAGE_VERSION,
                       "midi_kappamatrix21.fits")) {
   /* Propagate the error */
   (void)cpl_error_set_where(cpl_func);
}
if (cpl_dfs_save_image(frameset, NULL, parlist, frameset, NULL,  kappamatrix22,
                       CPL_BPP_IEEE_FLOAT, "midi_kappamatrix",
                       qclist22, NULL,
                       PACKAGE "/" PACKAGE_VERSION,
                       "midi_kappamatrix22.fits")) {
   /* Propagate the error */
   (void)cpl_error_set_where(cpl_func);
}


if (cpl_dfs_save_image(frameset, NULL, parlist, frameset, NULL,  kappamatrix11_nomask,
                       CPL_BPP_IEEE_FLOAT, "midi_kappamatrix",
                       qclist11_nomask, NULL,
                       PACKAGE "/" PACKAGE_VERSION,
                       "midi_kappamatrix11_nomask.fits")) {
   /* Propagate the error */
   (void)cpl_error_set_where(cpl_func);
}
if (cpl_dfs_save_image(frameset, NULL, parlist, frameset, NULL,  kappamatrix12_nomask,
                       CPL_BPP_IEEE_FLOAT, "midi_kappamatrix",
                       qclist12_nomask, NULL,
                       PACKAGE "/" PACKAGE_VERSION,
                       "midi_kappamatrix12_nomask.fits")) {
   /* Propagate the error */
   (void)cpl_error_set_where(cpl_func);
}
if (cpl_dfs_save_image(frameset, NULL, parlist, frameset, NULL,  kappamatrix21_nomask,
                       CPL_BPP_IEEE_FLOAT, "midi_kappamatrix",
                       qclist21_nomask, NULL,
                       PACKAGE "/" PACKAGE_VERSION,
                       "midi_kappamatrix21_nomask.fits")) {
   /* Propagate the error */
   (void)cpl_error_set_where(cpl_func);
}
if (cpl_dfs_save_image(frameset, NULL, parlist, frameset, NULL,  kappamatrix22_nomask,
                       CPL_BPP_IEEE_FLOAT, "midi_kappamatrix",
                       qclist22_nomask, NULL,
                       PACKAGE "/" PACKAGE_VERSION,
                       "midi_kappamatrix22_nomask.fits")) {
   /* Propagate the error */
   (void)cpl_error_set_where(cpl_func);
}


if (cpl_dfs_save_image(frameset, NULL, parlist, frameset, NULL,  kappamatrix11_filtered,
                       CPL_BPP_IEEE_FLOAT, "midi_kappamatrix",
                       qclist11_filtered, NULL,
                       PACKAGE "/" PACKAGE_VERSION,
                       "midi_kappamatrix11_filtered.fits")) {
   /* Propagate the error */
   (void)cpl_error_set_where(cpl_func);
}
if (cpl_dfs_save_image(frameset, NULL, parlist, frameset, NULL,  kappamatrix12_filtered,
                       CPL_BPP_IEEE_FLOAT, "midi_kappamatrix",
                       qclist12_filtered, NULL,
                       PACKAGE "/" PACKAGE_VERSION,
                       "midi_kappamatrix12_filtered.fits")) {
   /* Propagate the error */
   (void)cpl_error_set_where(cpl_func);
}
if (cpl_dfs_save_image(frameset, NULL, parlist, frameset, NULL,  kappamatrix21_filtered,
                       CPL_BPP_IEEE_FLOAT, "midi_kappamatrix",
                       qclist21_filtered, NULL,
                       PACKAGE "/" PACKAGE_VERSION,
                       "midi_kappamatrix21_filtered.fits")) {
   /* Propagate the error */
   (void)cpl_error_set_where(cpl_func);
}
if (cpl_dfs_save_image(frameset, NULL, parlist, frameset, NULL,  kappamatrix22_filtered,
                       CPL_BPP_IEEE_FLOAT, "midi_kappamatrix",
                       qclist22_filtered, NULL,
                       PACKAGE "/" PACKAGE_VERSION,
                       "midi_kappamatrix22_filtered.fits")) {
   /* Propagate the error */
   (void)cpl_error_set_where(cpl_func);
}


/* Save it to a fits table */

kappamatrix_table=cpl_table_new(cpl_image_get_size_x(kappamatrix11));
cpl_table_new_column(kappamatrix_table, "kappam11",  CPL_TYPE_FLOAT);
cpl_table_new_column(kappamatrix_table, "kappam11_error",  CPL_TYPE_FLOAT);
cpl_table_new_column(kappamatrix_table, "kappam12",  CPL_TYPE_FLOAT);
cpl_table_new_column(kappamatrix_table, "kappam12_error",  CPL_TYPE_FLOAT);
cpl_table_new_column(kappamatrix_table, "kappam21",  CPL_TYPE_FLOAT);
cpl_table_new_column(kappamatrix_table, "kappam21_error",  CPL_TYPE_FLOAT);
cpl_table_new_column(kappamatrix_table, "kappam22",  CPL_TYPE_FLOAT);
cpl_table_new_column(kappamatrix_table, "kappam22_error",  CPL_TYPE_FLOAT);

cpl_table_copy_data_float(kappamatrix_table, "kappam11", cpl_image_get_data_float((kappamatrix11)));
cpl_table_copy_data_float(kappamatrix_table, "kappam12", cpl_image_get_data_float((kappamatrix12)));
cpl_table_copy_data_float(kappamatrix_table, "kappam21", cpl_image_get_data_float((kappamatrix21)));
cpl_table_copy_data_float(kappamatrix_table, "kappam22", cpl_image_get_data_float((kappamatrix22)));

cpl_table_fill_column_window_float (kappamatrix_table, "kappam11_error", 0, cpl_image_get_size_x(kappamatrix11), 0.05);
cpl_table_fill_column_window_float (kappamatrix_table, "kappam12_error", 0, cpl_image_get_size_x(kappamatrix12), 0.05);
cpl_table_fill_column_window_float (kappamatrix_table, "kappam21_error", 0, cpl_image_get_size_x(kappamatrix21), 0.05);
cpl_table_fill_column_window_float (kappamatrix_table, "kappam22_error", 0, cpl_image_get_size_x(kappamatrix22), 0.05);

qclist_all = cpl_propertylist_new();
cpl_propertylist_append(qclist_all,qclist11);
cpl_propertylist_append(qclist_all,qclist12);
cpl_propertylist_append(qclist_all,qclist21);
cpl_propertylist_append(qclist_all,qclist22);
cpl_propertylist_erase_regexp(qclist_all, CPL_DFS_PRO_CATG,0);


/* Propertylist to write to the extension */
qclist_all_extension = cpl_propertylist_new();
cpl_propertylist_append_string (qclist_all_extension, "EXTNAME", "MIDI_KAPPAMATRIX");


if (strcmp(gris_name,"GRISM")==0)
{

cpl_propertylist_update_string(qclist_all, CPL_DFS_PRO_CATG, "MIDI_KAPPAMATRIX_GRISM");

cpl_dfs_save_table(frameset, NULL, parlist, frameset, NULL, kappamatrix_table, 
                      qclist_all_extension, "midi_kappamatrix",
                      qclist_all, NULL, 
                      PACKAGE "/" PACKAGE_VERSION,
                      "midi_kappamatrix_grism.fits");
   cpl_table_delete(kappamatrix_table);   
}

if (strcmp(gris_name,"PRISM")==0)
{
cpl_propertylist_update_string(qclist_all, CPL_DFS_PRO_CATG, "MIDI_KAPPAMATRIX_PRISM");
cpl_dfs_save_table(frameset, NULL, parlist, frameset, NULL, kappamatrix_table, 
                      qclist_all_extension, "midi_kappamatrix",
                      qclist_all, NULL, 
                      PACKAGE "/" PACKAGE_VERSION,
                      "midi_kappamatrix_prism.fits");
   cpl_table_delete(kappamatrix_table);    
}

/* Save it to an ascii file */



if (strcmp(gris_name,"GRISM")==0)
{
   cubename = cpl_sprintf("%s_%s%s", "transferRatios", "GRISM", ".dat");
   kappa_to_ascii( kappamatrix11, kappamatrix12, kappamatrix21, kappamatrix22, cubename);
   cpl_free(cubename);

   cubename = cpl_sprintf("%s_%s%s", "transferRatios", "GRISM", "_nomask.dat");
   kappa_to_ascii( kappamatrix11_nomask, kappamatrix12_nomask, kappamatrix21_nomask, kappamatrix22_nomask, cubename);
   cpl_free(cubename);

   cubename = cpl_sprintf("%s_%s%s", "transferRatios", "GRISM", "_filtered.dat");
   kappa_to_ascii( kappamatrix11_filtered, kappamatrix12_filtered, kappamatrix21_filtered, kappamatrix22_filtered, cubename);
   cpl_free(cubename);


}

if (strcmp(gris_name,"PRISM")==0)
{
   cubename = cpl_sprintf("%s_%s%s", "transferRatios", "PRISM", ".dat");
   kappa_to_ascii( kappamatrix11, kappamatrix12, kappamatrix21, kappamatrix22, cubename);
   cpl_free(cubename);

   cubename = cpl_sprintf("%s_%s%s", "transferRatios", "PRISM", "_nomask.dat");
   kappa_to_ascii( kappamatrix11_nomask, kappamatrix12_nomask, kappamatrix21_nomask, kappamatrix22_nomask, cubename);
   cpl_free(cubename);

   cubename = cpl_sprintf("%s_%s%s", "transferRatios", "PRISM", "_filtered.dat");
   kappa_to_ascii( kappamatrix11_filtered, kappamatrix12_filtered, kappamatrix21_filtered, kappamatrix22_filtered, cubename);
   cpl_free(cubename);

}




/* Free the memory */
cpl_image_delete(kappamatrix11); 
cpl_image_delete(kappamatrix12); 
cpl_image_delete(kappamatrix21); 
cpl_image_delete(kappamatrix22); 

cpl_image_delete(kappamatrix11_nomask); 
cpl_image_delete(kappamatrix12_nomask); 
cpl_image_delete(kappamatrix21_nomask); 
cpl_image_delete(kappamatrix22_nomask); 

cpl_image_delete(kappamatrix11_filtered); 
cpl_image_delete(kappamatrix12_filtered); 
cpl_image_delete(kappamatrix21_filtered); 
cpl_image_delete(kappamatrix22_filtered); 


/*cpl_matrix_delete(kernel) ;*/

   while(cpl_imagelist_get_size(imglst_mask_prism)>0){
      cpl_image_delete(cpl_imagelist_unset(imglst_mask_prism,0));
   }

   while(cpl_imagelist_get_size(imglst_mask_grism)>0){
      cpl_image_delete(cpl_imagelist_unset(imglst_mask_grism,0));
   }


cpl_imagelist_delete(imglst_mask_prism);
cpl_imagelist_delete(imglst_mask_grism);





cpl_propertylist_delete(qclist11);
cpl_propertylist_delete(qclist12);
cpl_propertylist_delete(qclist21);
cpl_propertylist_delete(qclist22);
cpl_propertylist_delete(qclist11_nomask);
cpl_propertylist_delete(qclist12_nomask);
cpl_propertylist_delete(qclist21_nomask);
cpl_propertylist_delete(qclist22_nomask);
cpl_propertylist_delete(qclist11_filtered);
cpl_propertylist_delete(qclist12_filtered);
cpl_propertylist_delete(qclist21_filtered);
cpl_propertylist_delete(qclist22_filtered);
cpl_propertylist_delete(qclist_all);
cpl_propertylist_delete(qclist_all_extension);


return (int)cpl_error_get_code();
}



static int table_to_imglst_sky_target(const char * columname,
                    const char * columntype,
                    cpl_imagelist * imglst_sky,
                    cpl_imagelist * imglst_target,
                    cpl_table * table)
{

   int dimenDATA; 
   int   i,csky=0, ctarget=0,csky_tmp=0, ctarget_tmp=0,first_unknown=0;
   cpl_array * array_data=NULL;
   cpl_image * image_data_int=NULL;
   cpl_errorstate          prestate = cpl_errorstate_get();
   char ** target_type; 	
   cpl_type ctype;   
   int   flag_issky=0;
   int   flag_istarget=0;

   cpl_imagelist * imglst_sky_tmp=NULL;
   cpl_imagelist * imglst_target_tmp=NULL;

   imglst_sky_tmp=cpl_imagelist_new();
   imglst_target_tmp=cpl_imagelist_new();

   /* Load extension  Imaging Data  */ 
   
   dimenDATA=cpl_table_get_column_dimensions(table, columname);
   cpl_ensure_code(cpl_errorstate_is_equal(prestate), cpl_error_get_code());
   if (dimenDATA != 2) {
      return (int)cpl_error_set_message(cpl_func, cpl_error_get_code(),
                                        "DATA has a wrong dimension");
   }
   /* Read target type T(arget),S(ky),B(ackground) and store them in the image list */
   /* Loop over all "images" stored in DATA and add them to the imagelist */
   
   
   if (cpl_table_has_column(table, columntype))
   {
      target_type=cpl_table_get_data_string(table, columntype);
      
   }
   else 
   {
      return (int)cpl_error_set_message(cpl_func, cpl_error_get_code(),
                                        "TYPE of the Column not found");
   }
/*    cpl_msg_info(cpl_func, "Number of images in the Table: %d",cpl_table_get_nrow(table)); */
/*    cpl_msg_info(cpl_func, "Extracting %d images: please wait ...",cpl_table_get_nrow(table)); */
   
   
   ctype=cpl_table_get_column_type(table, columname);   
/*    cpl_msg_info(cpl_func, "Type of the table column: %d",ctype); */



   csky=cpl_imagelist_get_size(imglst_sky);
   
   ctarget=cpl_imagelist_get_size(imglst_target);
   
/*    cpl_msg_info(cpl_func, "Length of imagelist S T : %d %d",csky,ctarget ); */



   flag_issky=0;
   flag_istarget=0;

/* Find the first "U" image and discard the images befor  */
   
   for (i=0; i<cpl_table_get_nrow(table);i++){
      if(strcmp(target_type[i],"U")== 0){
         first_unknown=i;
         break;
      }
   }

/*    cpl_msg_info(cpl_func, "First unknown: %d",first_unknown); */
   
   for (i=first_unknown; i<cpl_table_get_nrow(table);i++){
      
      array_data=(cpl_array *)cpl_table_get_array(table,columname, i); 
      if(ctype&CPL_TYPE_INT){
         image_data_int=cpl_image_wrap_int(cpl_table_get_column_dimension(table,columname,0 ), cpl_table_get_column_dimension(table,columname,1 ), cpl_array_get_data_int(array_data) );
      }
      if(ctype&CPL_TYPE_FLOAT){
         image_data_int=cpl_image_wrap_float(cpl_table_get_column_dimension(table,columname,0 ), cpl_table_get_column_dimension(table,columname,1 ), cpl_array_get_data_float(array_data) );
      }
      /* Cast the image to float */
/*       image_data=cpl_image_cast(image_data_int, CPL_TYPE_FLOAT); */
      
/*        Append the image to the imagelists */
      
      if(strcmp(target_type[i],"S")== 0){
         cpl_imagelist_set(imglst_sky_tmp,cpl_image_cast(image_data_int, CPL_TYPE_FLOAT),csky_tmp++);
         flag_issky=1;
         cpl_image_unwrap(image_data_int);
         continue;
      }
      if(strcmp(target_type[i],"T")== 0){
         cpl_imagelist_set(imglst_target_tmp,cpl_image_cast(image_data_int, CPL_TYPE_FLOAT),ctarget_tmp++);
         flag_istarget=1;
         cpl_image_unwrap(image_data_int);
         continue;
      }

      /*Unwrap processed image manualy if nor Target nor Sky*/
      if(image_data_int!=NULL){
         cpl_image_unwrap(image_data_int);
      }
      if(flag_issky >0 && flag_istarget >0){
         /*Collapse the temporary imagelist and add the image to the "final" imagelist */
         cpl_imagelist_set(imglst_sky,cpl_imagelist_collapse_create(imglst_sky_tmp),csky++);
         cpl_imagelist_set(imglst_target,cpl_imagelist_collapse_create(imglst_target_tmp),ctarget++);
         csky_tmp=0;
         ctarget_tmp=0;
         flag_issky=0;
         flag_istarget=0;

         /*Empty imagelist and free the image buffer*/
         while(cpl_imagelist_get_size(imglst_sky_tmp)>0){
            cpl_image_delete(cpl_imagelist_unset(imglst_sky_tmp,0));
         }
         while(cpl_imagelist_get_size(imglst_target_tmp)>0){
            cpl_image_delete(cpl_imagelist_unset(imglst_target_tmp,0));
         }

      }
      
   }
 
   
/*          Empty imagelist and free the image buffer*/
   while(cpl_imagelist_get_size(imglst_sky_tmp)>0){
      cpl_image_delete(cpl_imagelist_unset(imglst_sky_tmp,0));
   }
   while(cpl_imagelist_get_size(imglst_target_tmp)>0){
      cpl_image_delete(cpl_imagelist_unset(imglst_target_tmp,0));
   }
   
   cpl_imagelist_delete(imglst_sky_tmp);
   cpl_imagelist_delete(imglst_target_tmp);
   
   
   
   return (int)cpl_error_get_code();
}







static int table_to_imglst_mask(const char * columname,
                    cpl_imagelist * imglst,
                    cpl_table * table)
{

   int dimenDATA; 
   int   i=0, counter=0;
   cpl_array * array_data=NULL;
   cpl_image * image_data_int=NULL;
   cpl_errorstate          prestate = cpl_errorstate_get();
   cpl_type ctype;   

   /* Load extension  Imaging Data  */ 
   
   dimenDATA=cpl_table_get_column_dimensions(table, columname);
   cpl_ensure_code(cpl_errorstate_is_equal(prestate), cpl_error_get_code());
   if (dimenDATA != 2) {
      return (int)cpl_error_set_message(cpl_func, cpl_error_get_code(),
                                        "DATA has a wrong dimension");
   }
   /* Read target type T(arget),S(ky),B(ackground) and store them in the image list */
   /* Loop over all "images" stored in DATA and add them to the imagelist */
   
   
/*    cpl_msg_info(cpl_func, "Number of images in the Table: %d",cpl_table_get_nrow(table)); */
/*    cpl_msg_info(cpl_func, "Extracting %d images: please wait ...",cpl_table_get_nrow(table)); */
   
   
   ctype=cpl_table_get_column_type(table, columname);   
/*    cpl_msg_info(cpl_func, "Type of the table column: %d",ctype); */
   


   counter=cpl_imagelist_get_size(imglst);


   for (i=0; i<cpl_table_get_nrow(table);i++){
      
      array_data=(cpl_array *)cpl_table_get_array(table,columname, i); 
      if(ctype&CPL_TYPE_INT){
         image_data_int=cpl_image_wrap_int(cpl_table_get_column_dimension(table,columname,0 ), cpl_table_get_column_dimension(table,columname,1 ), cpl_array_get_data_int(array_data) );
      }
      if(ctype&CPL_TYPE_FLOAT){
         image_data_int=cpl_image_wrap_float(cpl_table_get_column_dimension(table,columname,0 ), cpl_table_get_column_dimension(table,columname,1 ), cpl_array_get_data_float(array_data) );
      }
      /* Cast the image to float */
/*       image_data=cpl_image_cast(image_data_int, CPL_TYPE_FLOAT); */
      
/*        Append the image to the imagelists */
      
      cpl_imagelist_set(imglst,cpl_image_cast(image_data_int, CPL_TYPE_FLOAT),counter++);

      /*Unwrap processed image manualy if nor Target nor Sky*/
      if(image_data_int!=NULL){
         cpl_image_unwrap(image_data_int);
      }
      
      
   }
   
   

   return (int)cpl_error_get_code();
}







static void kappa_to_ascii(const cpl_image * kappa11, 
                          const cpl_image * kappa12,
                          const cpl_image * kappa21,
                          const cpl_image * kappa22,
                          const char * filename)
{
   FILE * fp ;
   int pis_rejected;
   int i;
    if ( filename == NULL)
    {
        cpl_msg_error(cpl_func,"please give the filename!") ;
        return ;
    }

    if ( NULL == (fp = fopen ( filename, "w" ) ) )
    {
        cpl_msg_error(cpl_func,"cannot open %s", filename) ;
        return ;
    }


    fprintf (fp, "%d  \n",cpl_image_get_size_x(kappa11));
 
    for ( i = 1 ; i <= cpl_image_get_size_x(kappa11) ; i++ )
    {
       fprintf (fp, "%f 0.05 %f 0.05  %f 0.05  %f  0.05 \n", 
                cpl_image_get(kappa11, i, 1,&pis_rejected), 
                cpl_image_get(kappa12, i, 1,&pis_rejected),
                cpl_image_get(kappa21, i, 1,&pis_rejected), 
                cpl_image_get(kappa22, i, 1,&pis_rejected)) ;
    }
    fclose (fp ) ;
}


/**@}*/
